@@ -3,7 +3,7 @@
     comedi kernel module
 
     COMEDI - Linux Control and Measurement Device Interface
-    Copyright (C) 1997-2000 David A. Schleef <ds@stm.lbl.gov>
+    Copyright (C) 1997-2000 David A. Schleef <ds@schleef.org>
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -36,13 +36,15 @@
 #include <linux/kmod.h>
 #include <linux/poll.h>
 #include <linux/init.h>
+#include <linux/devfs_fs_kernel.h>
+#include <linux/vmalloc.h>
 
 #include <linux/comedidev.h>
 
 #include <asm/io.h>
 #include <asm/uaccess.h>
 
-#include "kvmem.h"
+//#include "kvmem.h"
 
 MODULE_AUTHOR("David Schleef <ds@schleef.org>");
 MODULE_DESCRIPTION("Comedi core module");
@@ -52,9 +54,15 @@ MODULE_LICENSE("GPL");
 #define KILL_FASYNC(a,b,c) kill_fasync(&(a),(b),(c))
 #endif
 
-comedi_device *comedi_devices;
+#ifdef CONFIG_COMEDI_DEBUG
+int comedi_debug;
+MODULE_PARM(comedi_debug, "i");
+#endif
 
-static int do_devconfig_ioctl(comedi_device *dev,comedi_devconfig *arg,kdev_t minor);
+comedi_device *comedi_devices;
+spinlock_t big_comedi_lock = SPIN_LOCK_UNLOCKED;
+
+static int do_devconfig_ioctl(comedi_device *dev,comedi_devconfig *arg,unsigned int minor);
 static int do_bufconfig_ioctl(comedi_device *dev,void *arg);
 static int do_devinfo_ioctl(comedi_device *dev,comedi_devinfo *arg);
 static int do_subdinfo_ioctl(comedi_device *dev,comedi_subdinfo *arg,void *file);
@@ -70,15 +78,17 @@ static int do_insn_ioctl(comedi_device *dev,void *arg,void *file);
 static int do_poll_ioctl(comedi_device *dev,unsigned int subd,void *file);
 
 void do_become_nonbusy(comedi_device *dev,comedi_subdevice *s);
-int resize_buf(comedi_device *dev,comedi_async *s, unsigned int size);
 static int do_cancel(comedi_device *dev,comedi_subdevice *s);
 
+#if LINUX_VERSION_CODE >= 0x020100
 static int comedi_fasync (int fd, struct file *file, int on);
+#endif
+static void init_async_buf( comedi_async *async );
 
 static int comedi_ioctl(struct inode * inode,struct file * file,
 	unsigned int cmd,unsigned long arg)
 {
-	kdev_t minor=MINOR(inode->i_rdev);
+	unsigned int minor=minor(inode->i_rdev);
 	comedi_device *dev=comedi_get_device_by_minor(minor);
 
 	/* Device config is special, because it must work on
@@ -141,9 +151,12 @@ static int comedi_ioctl(struct inode * inode,struct file * file,
 	writes:
 		none
 */
-static int do_devconfig_ioctl(comedi_device *dev,comedi_devconfig *arg,kdev_t minor)
+static int do_devconfig_ioctl(comedi_device *dev,comedi_devconfig *arg, unsigned int minor)
 {
 	comedi_devconfig it;
+	int ret;
+	unsigned char *aux_data = NULL;
+	int aux_len;
 
 	if(!suser())
 		return -EPERM;
@@ -157,7 +170,28 @@ static int do_devconfig_ioctl(comedi_device *dev,comedi_devconfig *arg,kdev_t mi
 
 	it.board_name[COMEDI_NAMELEN-1]=0;
 
-	return comedi_device_attach(dev,&it);
+	if(it.options[COMEDI_DEVCONF_AUX_DATA] &&
+	   it.options[COMEDI_DEVCONF_AUX_DATA_LENGTH]){
+		aux_len = it.options[COMEDI_DEVCONF_AUX_DATA_LENGTH];
+		if(aux_len<0)return -EFAULT;
+
+		aux_data = kmalloc(aux_len, GFP_KERNEL);
+		if(!aux_data)return -EFAULT;
+
+		if(copy_from_user(aux_data,
+		    (void *)it.options[COMEDI_DEVCONF_AUX_DATA], aux_len)){
+			kfree(aux_data);
+			return -EFAULT;
+		}
+
+		it.options[COMEDI_DEVCONF_AUX_DATA] = (unsigned long)aux_data;
+	}
+
+	ret = comedi_device_attach(dev,&it);
+
+	if(aux_data) kfree(aux_data);
+
+	return ret;
 }
 
 /*
@@ -217,10 +251,17 @@ static int do_bufconfig_ioctl(comedi_device *dev,void *arg)
 		if(!async->prealloc_buf)
 			return -EINVAL;
 
-		ret = resize_buf(dev,async,bc.size);
+		/* make sure buffer is an integral number of pages
+		 * (we round up) */
+		bc.size = (bc.size + 1)&PAGE_MASK;
 
-		if(ret < 0)
-			return ret;
+		ret = comedi_buf_alloc(dev, s, bc.size);
+		if(ret < 0) return ret;
+
+		if(s->buf_change){
+			ret = s->buf_change(dev,s,bc.size);
+			if(ret < 0) return ret;
+		}
 
 		DPRINTK("comedi%i subd %d buffer resized to %i bytes\n",
 			dev->minor, bc.subdevice, async->prealloc_bufsz);
@@ -236,37 +277,6 @@ copyback:
 	return 0;
 }
 
-/* utility function that resizes the prealloc_buf for
- * a subdevice
- */
-int resize_buf(comedi_device *dev, comedi_async *async, unsigned int size)
-{
-	void *old_buf;
-
-	// make sure buffer is an integral number of pages (we round up)
-	size = ((size + PAGE_SIZE - 1) / PAGE_SIZE) * PAGE_SIZE;
-
-	// if no change is required, do nothing
-	if(async->prealloc_buf && async->prealloc_bufsz &&
-		async->prealloc_bufsz == size)
-	{
-		return 0;
-	}
-
-	old_buf = async->prealloc_buf;
-	async->prealloc_buf = rvmalloc(size);
-	// restore old buffer on error
-	if(async->prealloc_buf == NULL){
-		async->prealloc_buf = old_buf;
-		return -ENOMEM;
-	}
-
-	rvfree(old_buf, async->prealloc_bufsz);
-	async->prealloc_bufsz = size;
-
-	return 0;
-}
-
 /*
 	COMEDI_DEVINFO
 	device info ioctl
@@ -374,6 +384,8 @@ static int do_subdinfo_ioctl(comedi_device *dev,comedi_subdinfo *arg,void *file)
 			us->subd_flags |= SDF_RANGETYPE;
 		if(s->do_cmd)
 			us->subd_flags |= SDF_CMD;
+
+		us->settling_time_0 = s->settling_time_0;
 	}
 	
 	ret=copy_to_user(arg,tmp,dev->n_subdevices*sizeof(comedi_subdinfo));
@@ -460,9 +472,6 @@ static int do_bufinfo_ioctl(comedi_device *dev,void *arg)
 	comedi_bufinfo bi;
 	comedi_subdevice *s;
 	comedi_async *async;
-	static spinlock_t bufinfo_lock = SPIN_LOCK_UNLOCKED;
-	unsigned long irq_flags;
-	int m;
 
 	if(copy_from_user(&bi,arg, sizeof(comedi_bufinfo)))
 		return -EFAULT;
@@ -473,6 +482,8 @@ static int do_bufinfo_ioctl(comedi_device *dev,void *arg)
 	s=dev->subdevices + bi.subdevice;
 	async=s->async;
 
+	if(s!=dev->read_subdev && s!=dev->write_subdev)return -EINVAL;
+
 	if(!async){
 		DPRINTK("subdevice does not have async capability\n");
 		bi.buf_int_ptr = 0;
@@ -482,38 +493,33 @@ static int do_bufinfo_ioctl(comedi_device *dev,void *arg)
 		goto copyback;
 	}
 
-	if(bi.bytes_read){
+	if(bi.bytes_read && s==dev->read_subdev){
+		comedi_buf_read_free(async, bi.bytes_read);
 
-		// check for buffer underflow
-		m = async->buf_int_count - async->buf_user_count;
-		if(bi.bytes_read > m)
-		{
-			DPRINTK("buffer underflow\n");
-			return -EIO;
-		}
-
-		async->buf_user_ptr += bi.bytes_read;
-		async->buf_user_count += bi.bytes_read;
-
-		// check for buffer overrun
-		if(m > async->data_len){
-			async->buf_user_count = async->buf_int_count;
-			async->buf_user_ptr = async->buf_int_ptr;
-			do_cancel(dev, dev->read_subdev);
-			DPRINTK("buffer overrun\n");
-			return -EIO;
-		}
-		if(!(s->subdev_flags&SDF_RUNNING) && async->buf_int_count==async->buf_user_count){
+		if(!(s->subdev_flags&SDF_RUNNING) &&
+		   !(s->runflags & SRF_ERROR) &&
+		   async->buf_write_count==async->buf_read_count){
 			do_become_nonbusy(dev,s);
 		}
 	}
 
-	comedi_spin_lock_irqsave(&bufinfo_lock, irq_flags);
-	bi.buf_int_count = async->buf_int_count;
-	bi.buf_int_ptr = async->buf_int_ptr;
-	comedi_spin_unlock_irqrestore(&bufinfo_lock, irq_flags);
-	bi.buf_user_count = async->buf_user_count;
-	bi.buf_user_ptr = async->buf_user_ptr;
+	if(bi.bytes_written && s==dev->write_subdev){
+		bi.bytes_written = comedi_buf_write_alloc( async, bi.bytes_written );
+		comedi_buf_munge(dev, s, bi.bytes_written);
+		comedi_buf_write_free(async, bi.bytes_written);
+	}
+
+	if(s==dev->read_subdev){
+		bi.buf_int_count = async->buf_write_count;
+		bi.buf_int_ptr = async->buf_write_ptr;
+		bi.buf_user_count = async->buf_read_count;
+		bi.buf_user_ptr = async->buf_read_ptr;
+	}else{
+		bi.buf_int_count = async->buf_read_count;
+		bi.buf_int_ptr = async->buf_read_ptr;
+		bi.buf_user_count = async->buf_write_count;
+		bi.buf_user_ptr = async->buf_write_ptr;
+	}
 
 copyback:
 	if(copy_to_user(arg, &bi, sizeof(comedi_bufinfo)))
@@ -553,34 +559,41 @@ static int do_insnlist_ioctl(comedi_device *dev,void *arg,void *file)
 	if(copy_from_user(&insnlist,arg,sizeof(comedi_insnlist)))
 		return -EFAULT;
 
-	if(insnlist.n_insns>=MAX_INSNS)
+	if(insnlist.n_insns>=MAX_INSNS){
+		DPRINTK("insnlist too long\n");
 		return -EINVAL;
+	}
 
 	data=kmalloc(sizeof(lsampl_t)*MAX_SAMPLES,GFP_KERNEL);
 	if(!data){
+		DPRINTK("kmalloc failed\n");
 		ret = -ENOMEM;
 		goto error;
 	}
 
 	insns=kmalloc(sizeof(comedi_insn)*insnlist.n_insns,GFP_KERNEL);
 	if(!insns){
+		DPRINTK("kmalloc failed\n");
 		ret = -ENOMEM;
 		goto error;
 	}
 
 	if(copy_from_user(insns,insnlist.insns,sizeof(comedi_insn)*insnlist.n_insns)){
+		DPRINTK("copy_from_user failed\n");
 		ret=-EFAULT;
 		goto error;
 	}
 
 	for(i=0;i<insnlist.n_insns;i++){
 		if(insns[i].n>MAX_SAMPLES){
+			DPRINTK("number of samples too large\n");
 			ret=-EINVAL;
 			goto error;
 		}
 		if(insns[i].insn&INSN_MASK_WRITE){
 			if(copy_from_user(data,insns[i].data,
 					insns[i].n*sizeof(lsampl_t))){
+				DPRINTK("copy_from_user failed\n");
 				ret=-EFAULT;
 				goto error;
 			}
@@ -595,6 +608,7 @@ static int do_insnlist_ioctl(comedi_device *dev,void *arg,void *file)
 		if(insns[i].insn&INSN_MASK_READ){
 			if(copy_to_user(insns[i].data,data,
 					insns[i].n*sizeof(lsampl_t))){
+				DPRINTK("copy_to_user failed\n");
 				ret=-EFAULT;
 				goto error;
 			}
@@ -643,33 +657,45 @@ static int parse_insn(comedi_device *dev,comedi_insn *insn,lsampl_t *data,void *
 			ret=1;
 			break;
 		case INSN_INTTRIG:
+			if(insn->n!=1){
+				ret=-EINVAL;
+				break;
+			}
 			if(insn->subdev>=dev->n_subdevices){
-				DPRINTK("%d not useable subdevice\n",insn->subdev);
+				DPRINTK("%d not usable subdevice\n",insn->subdev);
 				ret=-EINVAL;
 				break;
 			}
 			s=dev->subdevices+insn->subdev;
-			if(!s->async || !s->async->inttrig){
-				DPRINTK("no async or no inttrig\n");
+			if(!s->async){
+				DPRINTK("no async\n");
 				ret=-EINVAL;
 				break;
 			}
-			ret = s->async->inttrig(dev,s,0);
+			if(!s->async->inttrig){
+				DPRINTK("no inttrig\n");
+				ret=-EAGAIN;
+				break;
+			}
+			ret = s->async->inttrig(dev,s,insn->data[0]);
+			if(ret>=0)ret = 1;
 			break;
 		default:
 			DPRINTK("invalid insn\n");
 			ret=-EINVAL;
+			break;
 		}
 	}else{
 		/* a subdevice instruction */
 		if(insn->subdev>=dev->n_subdevices){
+			DPRINTK("subdevice %d out of range\n",insn->subdev);
 			ret=-EINVAL;
 			goto out;
 		}
 		s=dev->subdevices+insn->subdev;
 
 		if(s->type==COMEDI_SUBD_UNUSED){
-			DPRINTK("%d not useable subdevice\n",insn->subdev);
+			DPRINTK("%d not usable subdevice\n",insn->subdev);
 			ret = -EIO;
 			goto out;
 		}
@@ -766,6 +792,7 @@ static int do_insn_ioctl(comedi_device *dev,void *arg,void *file)
 			goto error;
 		}
 	}
+	ret = insn.n;
 
 error:
 	if(data)kfree(data);
@@ -890,27 +917,13 @@ static int do_cmd_ioctl(comedi_device *dev,void *arg,void *file)
 		goto cleanup;
 	}
 
-#if 0
-	/* XXX this needs to be removed when the drivers are ready */
-	/* They should be ready now. */
-	async->cmd.data = async->prealloc_buf;
-	async->cmd.data_len=async->prealloc_bufsz;
-#endif
-
-	async->data = async->prealloc_buf;
-	async->data_len=async->prealloc_bufsz;
-
-	async->buf_int_ptr=0;
-	async->buf_int_count=0;
-	async->buf_user_ptr=0;
-	async->buf_user_count=0;
-
-	async->cur_chan = 0;
+	init_async_buf( async );
 
 	async->cb_mask = COMEDI_CB_EOA|COMEDI_CB_BLOCK|COMEDI_CB_ERROR;
 	if(async->cmd.flags & TRIG_WAKE_EOS){
 		async->cb_mask |= COMEDI_CB_EOS;
 	}
+
 	async->events = 0;
 
 	s->runflags=SRF_USER;
@@ -1042,26 +1055,12 @@ cleanup:
 	writes:
 		none
 
-	non-RT linux always controls rtcomedi_lock_semaphore.  If an
-	RT-linux process wants the lock, it first checks rtcomedi_lock_semaphore.
-	If it is 1, it knows it is pre-empting this function, and fails.
-	Obviously, if RT-linux fails to get a lock, it *must* allow
-	linux to run, since that is the only way to free the lock.
-
-	This function is not SMP compatible.
-
-	necessary locking:
-	- ioctl/rt lock  (this type)
-	- lock while subdevice busy
-	- lock while subdevice being programmed
-
 */
 
-volatile int rtcomedi_lock_semaphore=0;
-
 static int do_lock_ioctl(comedi_device *dev,unsigned int arg,void * file)
 {
 	int ret=0;
+	unsigned long flags;
 	comedi_subdevice *s;
 
 	if(arg>=dev->n_subdevices)
@@ -1071,7 +1070,7 @@ static int do_lock_ioctl(comedi_device *dev,unsigned int arg,void * file)
 	if(s->busy)
 		return -EBUSY;
 
-	rtcomedi_lock_semaphore=1;
+	comedi_spin_lock_irqsave(&big_comedi_lock, flags);
 
 	if(s->lock && s->lock!=file){
 		ret=-EACCES;
@@ -1079,7 +1078,7 @@ static int do_lock_ioctl(comedi_device *dev,unsigned int arg,void * file)
 		s->lock=file;
 	}
 
-	rtcomedi_lock_semaphore=0;
+	comedi_spin_unlock_irqrestore(&big_comedi_lock, flags);
 
 	if(ret<0)
 		return ret;
@@ -1178,7 +1177,7 @@ static int do_cancel_ioctl(comedi_device *dev,unsigned int arg,void *file)
 	
 	reads:
 		nothing
-	
+
 	writes:
 		nothing
 
@@ -1243,16 +1242,20 @@ static struct vm_operations_struct comedi_vm_ops={
 };
 /*
    comedi_mmap_v22
-
-   issues:
-      what happens when the underlying buffer gets changed?
-
  */
+#if LINUX_VERSION_CODE < KERNEL_VERSION(2,3,17)
+#define page_address(x) x
+#endif
+
 static int comedi_mmap_v22(struct file * file, struct vm_area_struct *vma)
 {
-	kdev_t minor=MINOR(RDEV_OF_FILE(file));
+	unsigned int minor=minor(RDEV_OF_FILE(file));
 	comedi_device *dev=comedi_get_device_by_minor(minor);
 	comedi_async *async = NULL;
+	unsigned long start = vma->vm_start;
+	unsigned long size;
+	int n_pages;
+	int i;
 
 	if(!dev->attached)
 	{
@@ -1269,20 +1272,35 @@ static int comedi_mmap_v22(struct file * file, struct vm_area_struct *vma)
 		return -EINVAL;
 	}
 
-#if LINUX_VERSION_CODE < KERNEL_VERSION(2,3,0)
+#if LINUX_VERSION_CODE < 0x020300
 	if(vma->vm_offset != 0){
-#else
-	if(vma->vm_pgoff != 0){
-#endif
 		DPRINTK("comedi: mmap() offset must be 0.\n");
 		return -EINVAL;
 	}
+#else
+	if(vma->vm_pgoff != 0){
+		DPRINTK("comedi: mmap() offset must be 0.\n");
+		return -EINVAL;
+	}
+#endif
 
-	rvmmap(async->prealloc_buf,async->prealloc_bufsz,vma);
+	size = vma->vm_end - vma->vm_start;
+	if(size>async->prealloc_bufsz)
+		return -EFAULT;
+	if(size&(~PAGE_MASK))
+		return -EFAULT;
+
+	n_pages = size >> PAGE_SHIFT;
+	for(i=0;i<n_pages;i++){
+		if(REMAP_PAGE_RANGE(vma, start, __pa(async->buf_page_list[i]),
+				PAGE_SIZE, PAGE_SHARED)){
+			return -EAGAIN;
+		}
+		start += PAGE_SIZE;
+	}
 
-	//vma->vm_file = file;
 	vma->vm_ops = &comedi_vm_ops;
-#if LINUX_VERSION_CODE < KERNEL_VERSION(2,3,17)
+#if LINUX_VERSION_CODE < 0x020300
 	(void *)vma->vm_pte = async;
 #else
 	vma->vm_private_data = async;
@@ -1303,7 +1321,7 @@ static unsigned int comedi_poll_v22(struct file *file, poll_table * wait)
 	comedi_async *async;
 	unsigned int mask;
 
-	dev=comedi_get_device_by_minor(MINOR(RDEV_OF_FILE(file)));
+	dev=comedi_get_device_by_minor(minor(RDEV_OF_FILE(file)));
 
 	if(!dev->attached)
 	{
@@ -1317,21 +1335,19 @@ static unsigned int comedi_poll_v22(struct file *file, poll_table * wait)
 	if(dev->read_subdev && dev->read_subdev->async){
 		s = dev->read_subdev;
 		async = s->async;
-		if(s->busy){
-			if((!s->subdev_flags&SDF_RUNNING) ||
-			   (async->buf_user_count < async->buf_int_count)){
-				mask |= POLLIN | POLLRDNORM;
-			}
+		if(!s->busy
+		   || comedi_buf_read_n_available(async)>0
+		   || !(s->subdev_flags&SDF_RUNNING)){
+			mask |= POLLIN | POLLRDNORM;
 		}
 	}
 	if(dev->write_subdev && dev->write_subdev->async){
 		s = dev->write_subdev;
 		async = s->async;
-		if(s->busy){
-			if((!s->subdev_flags&SDF_RUNNING) ||
-			   (async->buf_user_count < async->buf_int_count)){
-				mask |= POLLOUT | POLLWRNORM;
-			}
+		if(!s->busy
+		   || !(s->subdev_flags&SDF_RUNNING)
+		   || comedi_buf_write_n_available(async)>0){
+			mask |= POLLOUT | POLLWRNORM;
 		}
 	}
 
@@ -1346,9 +1362,8 @@ static ssize_t comedi_write_v22(struct file *file,const char *buf,size_t nbytes,
 	comedi_async *async;
 	int n,m,count=0,retval=0;
 	DECLARE_WAITQUEUE(wait,current);
-	int sample_size;
 
-	dev=comedi_get_device_by_minor(MINOR(RDEV_OF_FILE(file)));
+	dev=comedi_get_device_by_minor(minor(RDEV_OF_FILE(file)));
 
 	if(!dev->attached)
 	{
@@ -1360,14 +1375,6 @@ static ssize_t comedi_write_v22(struct file *file,const char *buf,size_t nbytes,
 	s = dev->write_subdev;
 	async = s->async;
 
-	if(s->subdev_flags&SDF_LSAMPL){
-		sample_size=sizeof(lsampl_t);
-	}else{
-		sample_size=sizeof(sampl_t);
-	}
-	if(nbytes%sample_size)
-		nbytes-=nbytes%sample_size;
-
 	if(!nbytes)return 0;
 
 	if(!s->busy)
@@ -1382,13 +1389,13 @@ static ssize_t comedi_write_v22(struct file *file,const char *buf,size_t nbytes,
 
 		n=nbytes;
 
-		m=async->data_len+async->buf_int_count-async->buf_user_count;
-
-		if(async->buf_user_ptr+m > async->data_len){
-			m=async->data_len - async->buf_user_ptr;
+		m = n;
+		if(async->buf_write_ptr + m > async->prealloc_bufsz){
+			m = async->prealloc_bufsz - async->buf_write_ptr;
 		}
+		m = comedi_buf_write_alloc(async, m);
 
-		if(m<n)n=m;
+		if(m < n) n = m;
 
 		if(n==0){
 			if(file->f_flags&O_NONBLOCK){
@@ -1400,24 +1407,29 @@ static ssize_t comedi_write_v22(struct file *file,const char *buf,size_t nbytes,
 				break;
 			}
 			if(!(s->subdev_flags&SDF_RUNNING)){
+				if(s->runflags & SRF_ERROR){
+					retval = -EPIPE;
+				}else{
+					retval = 0;
+				}
 				do_become_nonbusy(dev,s);
 				break;
 			}
 			schedule();
 			continue;
 		}
-		m=copy_from_user(async->data+async->buf_user_ptr,buf,n);
-		if(m) retval=-EFAULT;
-		n-=m;
-		
+
+		m = copy_from_user(async->prealloc_buf + async->buf_write_ptr,
+			buf, n);
+		if(m){
+			n -= m;
+			retval = -EFAULT;
+		}
+		comedi_buf_munge(dev, s, n);
+		comedi_buf_write_free(async, n);
+
 		count+=n;
 		nbytes-=n;
-		async->buf_user_ptr+=n;
-		async->buf_user_count+=n;
-
-		if(async->buf_user_ptr>=async->data_len ){
-			async->buf_user_ptr=0;
-		}
 
 		buf+=n;
 		break;	/* makes device work like a pipe */
@@ -1436,9 +1448,8 @@ static ssize_t comedi_read_v22(struct file * file,char *buf,size_t nbytes,loff_t
 	comedi_async *async;
 	int n,m,count=0,retval=0;
 	DECLARE_WAITQUEUE(wait,current);
-	int sample_size;
 
-	dev=comedi_get_device_by_minor(MINOR(RDEV_OF_FILE(file)));
+	dev=comedi_get_device_by_minor(minor(RDEV_OF_FILE(file)));
 
 	if(!dev->attached)
 	{
@@ -1450,14 +1461,6 @@ static ssize_t comedi_read_v22(struct file * file,char *buf,size_t nbytes,loff_t
 	if(s == NULL)return -EIO;
 	async = s->async;
 
-	if(s->subdev_flags&SDF_LSAMPL){
-		sample_size=sizeof(lsampl_t);
-	}else{
-		sample_size=sizeof(sampl_t);
-	}
-	if(nbytes%sample_size)
-		nbytes-=nbytes%sample_size;
-
 	if(!nbytes)return 0;
 
 	if(!s->busy)
@@ -1472,21 +1475,19 @@ static ssize_t comedi_read_v22(struct file * file,char *buf,size_t nbytes,loff_t
 
 		n=nbytes;
 
-		m=async->buf_int_count-async->buf_user_count;
-
-		if(async->buf_user_ptr+m > async->data_len){
-			m=async->data_len - async->buf_user_ptr;
-#if 0
-printk("m is %d\n",m);
-#endif
+		m = comedi_buf_read_n_available(async);
+//printk("%d available\n",m);
+		if(async->buf_read_ptr + m > async->prealloc_bufsz){
+			m = async->prealloc_bufsz - async->buf_read_ptr;
 		}
+//printk("%d contiguous\n",m);
 		if(m<n)n=m;
 
 		if(n==0){
 			if(!(s->subdev_flags&SDF_RUNNING)){
 				do_become_nonbusy(dev,s);
 				if(s->runflags & SRF_ERROR){
-					retval = -EIO;
+					retval = -EPIPE;
 				}else{
 					retval = 0;
 				}
@@ -1503,35 +1504,25 @@ printk("m is %d\n",m);
 			schedule();
 			continue;
 		}
-		m=copy_to_user(buf,async->data+async->buf_user_ptr,n);
-		if(m) retval=-EFAULT;
-		n-=m;
 
-		// check for buffer overrun
-		if(async->buf_int_count - async->buf_user_count > async->data_len){
-			async->buf_user_count = async->buf_int_count;
-			async->buf_user_ptr = async->buf_int_ptr;
-			retval=-EIO;
-			do_cancel(dev, dev->read_subdev);
-			DPRINTK("buffer overrun\n");
-			break;
+		m = copy_to_user(buf, async->prealloc_buf +
+			async->buf_read_ptr, n);
+		if(m){
+			n -= m;
+			retval = -EFAULT;
 		}
 
+		comedi_buf_read_free(async, n);
+
 		count+=n;
 		nbytes-=n;
-		async->buf_user_ptr+=n;
-		async->buf_user_count+=n;
-
-		if(async->buf_user_ptr>=async->data_len ){
-			async->buf_user_ptr=0;
-		}
 
 		buf+=n;
 		break;	/* makes device work like a pipe */
 	}
 	if(!(s->subdev_flags&SDF_RUNNING) &&
 		!(s->runflags & SRF_ERROR) &&
-		async->buf_int_count - async->buf_user_count == 0)
+		async->buf_read_count - async->buf_write_count == 0)
 	{
 		do_become_nonbusy(dev,s);
 	}
@@ -1547,9 +1538,6 @@ printk("m is %d\n",m);
 void do_become_nonbusy(comedi_device *dev,comedi_subdevice *s)
 {
 	comedi_async *async = s->async;
-#if 0
-	printk("becoming non-busy\n");
-#endif
 	/* we do this because it's useful for the non-standard cases */
 	s->subdev_flags &= ~SDF_RUNNING;
 
@@ -1561,10 +1549,7 @@ void do_become_nonbusy(comedi_device *dev,comedi_subdevice *s)
 #endif
 
 	if(async){
-		async->buf_user_ptr=0;
-		async->buf_int_ptr=0;
-		async->buf_user_count=0;
-		async->buf_int_count=0;
+		init_async_buf( async );
 	}else{
 		printk("BUG: (?) do_become_nonbusy called with async=0\n");
 	}
@@ -1582,7 +1567,7 @@ static loff_t comedi_lseek_v22(struct file *file,loff_t offset,int origin)
 	comedi_device *dev;
 	loff_t new_offset;
 
-	dev=comedi_get_device_by_minor(MINOR(RDEV_OF_FILE(file)));
+	dev=comedi_get_device_by_minor(minor(RDEV_OF_FILE(file)));
 
 	switch(origin){
 	case SEEK_SET:
@@ -1605,7 +1590,7 @@ static loff_t comedi_lseek_v22(struct file *file,loff_t offset,int origin)
 
 static int comedi_fop_open(struct inode *inode,struct file *file)
 {
-	kdev_t minor=MINOR(inode->i_rdev);
+	unsigned int minor=minor(inode->i_rdev);
 	comedi_device *dev;
 	char mod[32];
 
@@ -1647,11 +1632,17 @@ static int comedi_fop_open(struct inode *inode,struct file *file)
 	return -ENODEV;
 
 ok:
-	MOD_INC_USE_COUNT;
+	if(!try_module_get(THIS_MODULE))
+		return -ENOSYS;
 
-	if(dev->attached && dev->driver->module){
-		__MOD_INC_USE_COUNT(dev->driver->module);
+	if(dev->attached){
+		try_module_get( dev->driver->module );
 	}
+
+	if(dev->attached && dev->use_count==0 && dev->open){
+		dev->open(dev);
+	}
+
 	dev->use_count++;
 
 	return 0;
@@ -1659,7 +1650,7 @@ ok:
 
 static int comedi_close_v22(struct inode *inode,struct file *file)
 {
-	comedi_device *dev=comedi_get_device_by_minor(MINOR(inode->i_rdev));
+	comedi_device *dev=comedi_get_device_by_minor(minor(inode->i_rdev));
 	comedi_subdevice *s = NULL;
 	int i;
 
@@ -1674,9 +1665,13 @@ static int comedi_close_v22(struct inode *inode,struct file *file)
 		}
 	}
 
-	MOD_DEC_USE_COUNT;
-	if(dev->attached && dev->driver->module){
-		__MOD_DEC_USE_COUNT(dev->driver->module);
+	if(dev->attached && dev->use_count==1 && dev->close){
+		dev->close(dev);
+	}
+
+	module_put(THIS_MODULE);
+	if(dev->attached){
+		module_put(dev->driver->module);
 	}
 
 	dev->use_count--;
@@ -1693,7 +1688,7 @@ static int comedi_close_v22(struct inode *inode,struct file *file)
 #if LINUX_VERSION_CODE >= 0x020100
 static int comedi_fasync (int fd, struct file *file, int on)
 {
-	comedi_device *dev=comedi_get_device_by_minor(MINOR(RDEV_OF_FILE(file)));
+	comedi_device *dev=comedi_get_device_by_minor(minor(RDEV_OF_FILE(file)));
 
 	return fasync_helper(fd,file,on,&dev->async_queue);
 }
@@ -1765,7 +1760,7 @@ static int __init comedi_init(void)
 	int i;
 
 	printk("comedi: version " COMEDI_RELEASE " - David Schleef <ds@schleef.org>\n");
-	if(register_chrdev(COMEDI_MAJOR,"comedi",&comedi_fops)){
+	if(devfs_register_chrdev(COMEDI_MAJOR,"comedi",&comedi_fops)){
 		printk("comedi: unable to get major %d\n",COMEDI_MAJOR);
 		return -EIO;
 	}
@@ -1780,6 +1775,13 @@ static int __init comedi_init(void)
 
 	/* XXX requires /proc interface */
 	comedi_proc_init();
+
+	for(i=0;i<COMEDI_NDEVICES;i++){
+		char name[20];
+		sprintf(name, "comedi%d", i);
+		devfs_register(NULL, name, DEVFS_FL_DEFAULT,
+			COMEDI_MAJOR, i, 0666 | S_IFCHR, &comedi_fops, NULL);
+	}
 	
 	comedi_rt_init();
 
@@ -1793,7 +1795,14 @@ static void __exit comedi_cleanup(void)
 	if(MOD_IN_USE)
 		printk("comedi: module in use -- remove delayed\n");
 
-	unregister_chrdev(COMEDI_MAJOR,"comedi");
+	for(i=0;i<4;i++){
+		char name[20];
+		sprintf(name, "comedi%d", i);
+		devfs_unregister(devfs_find_handle(NULL, name,
+			COMEDI_MAJOR, i, DEVFS_SPECIAL_CHR, 0));
+	}
+
+	devfs_unregister_chrdev(COMEDI_MAJOR,"comedi");
 
 	comedi_proc_cleanup();
 
@@ -1812,19 +1821,34 @@ static void __exit comedi_cleanup(void)
 module_init(comedi_init);
 module_exit(comedi_cleanup);
 
-void comedi_error(comedi_device *dev,const char *s)
+void comedi_error(const comedi_device *dev,const char *s)
 {
 	rt_printk("comedi%d: %s: %s\n",dev->minor,dev->driver->driver_name,s);
 }
 
-void comedi_event(comedi_device *dev,comedi_subdevice *s,unsigned int mask)
+void comedi_event(comedi_device *dev,comedi_subdevice *s, unsigned int mask)
 {
 	comedi_async *async = s->async;
 
 	mask = s->async->events;
+	s->async->events = 0;
+
+	mask |= COMEDI_CB_BLOCK;
 
 	//DPRINTK("comedi_event %x\n",mask);
 
+	if( (s->subdev_flags & SDF_RUNNING) == 0)
+		return;
+
+	if(mask&(COMEDI_CB_EOA|COMEDI_CB_ERROR|COMEDI_CB_OVERFLOW)){
+		s->subdev_flags &= ~SDF_RUNNING;
+	}
+
+	/* remember if an error event has occured, so an error
+	 * can be returned the next time the user does a read() */
+	if(mask & (COMEDI_CB_ERROR|COMEDI_CB_OVERFLOW)){
+		s->runflags |= SRF_ERROR;
+	}
 	if(async->cb_mask&mask){
 		if(s->runflags&SRF_USER){
 
@@ -1857,65 +1881,19 @@ void comedi_event(comedi_device *dev,comedi_subdevice *s,unsigned int mask)
 			 * common, I'm not going to worry about it. */
 		}
 	}
-
-	if(mask&COMEDI_CB_EOA){
-		s->subdev_flags &= ~SDF_RUNNING;
-	}
-
-	/* remember if an error event has occured, so an error
-	 * can be returned the next time the user does a read() */
-	if(mask & COMEDI_CB_ERROR){
-		s->runflags |= SRF_ERROR;
-	}
-
-	s->async->events = 0;
 }
 
-#if 0
-/*
-   this function should be called by your interrupt routine
-   at the end of acquisition
- */
-void comedi_done(comedi_device *dev,comedi_subdevice *s)
+static void init_async_buf( comedi_async *async )
 {
-	comedi_event(dev,s,COMEDI_CB_EOA);
+	async->buf_write_alloc_count = 0;
+	async->buf_write_count = 0;
+	async->buf_read_count = 0;
+
+	async->buf_write_ptr = 0;
+	async->buf_read_ptr = 0;
+
+	async->cur_chan = 0;
+	async->scan_progress = 0;
+	async->munge_chan = 0;
 }
 
-/*
-   this function should be called by your interrupt routine
-   at errors causing termination of acquisition
- */
-void comedi_error_done(comedi_device *dev,comedi_subdevice *s)
-{
-	comedi_event(dev,s,COMEDI_CB_ERROR|COMEDI_CB_EOA);
-}
-
-/*
-   this function should be called by your interrupt routine
-   at convenient block sizes
- */
-void comedi_bufcheck(comedi_device *dev,comedi_subdevice *s)
-{
-	comedi_event(dev,s,COMEDI_CB_BLOCK);
-}
-
-/*
-   this function should be called by your interrupt routine
-   at end-of-scan events
- */
-void comedi_eos(comedi_device *dev,comedi_subdevice *s)
-{
-	comedi_event(dev,s,COMEDI_CB_EOS);
-}
-
-/*
-   this function should be called by your interrupt routine
-   at buffer rollover events
- */
-void comedi_eobuf(comedi_device *dev,comedi_subdevice *s)
-{
-	comedi_event(dev,s,COMEDI_CB_EOBUF);
-}
-#endif
-
-
