@@ -1,7 +1,7 @@
 /*
  * comedi/drivers/adv_pci1710.c
  *
- * Author: Michal Dobes <majkl@tesnet.cz>
+ * Author: Michal Dobes <dobes@tesnet.cz>
  *
  * Thanks to ZhenGang Shang <ZhenGang.Shang@Advantech.com.cn>
  * for testing and informations.
@@ -20,7 +20,7 @@
 Driver: adv_pci1710.o
 Description: Advantech PCI-1710, PCI-1710HG, PCI-1711, PCI-1713,
              Advantech PCI-1720, PCI-1731
-Author: Michal Dobes <majkl@tesnet.cz>
+Author: Michal Dobes <dobes@tesnet.cz>
 Devices: [Advantech] PCI-1710 (pci1710), PCI-1710HG (pci1710hg),
   PCI-1711 (pci1711), PCI-1713 (pci1713), PCI-1720 (pci1720),
   PCI-1731 (pci1731)
@@ -41,26 +41,14 @@ Configuration options:
           device will be used.
 */
 
-#include <linux/kernel.h>
-#include <linux/module.h>
-#include <linux/sched.h>
-#include <linux/mm.h>
-#include <linux/slab.h>
-#include <linux/errno.h>
-#include <linux/ioport.h>
-#include <linux/delay.h>
-#include <linux/interrupt.h>
-#include <linux/timex.h>
-#include <linux/timer.h>
-#include <linux/pci.h>
-#include <asm/io.h>
 #include <linux/comedidev.h>
+
+#include <linux/pci.h>
+
 #include "8253.h"
 #include "amcc_s5933.h"
 
 
-#define ADVANTECH_VENDOR	0x13fe	/* Advantech PCI vendor ID */
-
 #define PCI171x_PARANOIDCHECK		/* if defined, then is used code which control correct channel number on every 12 bit sample */
 
 #undef PCI171X_EXTDEBUG
@@ -189,7 +177,6 @@ static unsigned short	pci_list_builded=0;	/*=1 list of card is know */
 
 typedef struct {
 	char 		*name;		// driver name
-	int		vendor_id;	// PCI vendor a device ID of card
 	int		device_id;
 	int		iorange;	// I/O range len
 	char		have_irq;	// 1=card support IRQ
@@ -209,43 +196,43 @@ typedef struct {
 } boardtype;
 
 static struct pci_device_id pci1710_pci_table[] = __devinitdata {
-	{ ADVANTECH_VENDOR, 0x1710, PCI_ANY_ID, PCI_ANY_ID, 0, 0, 0 },
-	{ ADVANTECH_VENDOR, 0x1711, PCI_ANY_ID, PCI_ANY_ID, 0, 0, 0 },
-	{ ADVANTECH_VENDOR, 0x1713, PCI_ANY_ID, PCI_ANY_ID, 0, 0, 0 },
-	{ ADVANTECH_VENDOR, 0x1720, PCI_ANY_ID, PCI_ANY_ID, 0, 0, 0 },
-	{ ADVANTECH_VENDOR, 0x1731, PCI_ANY_ID, PCI_ANY_ID, 0, 0, 0 },
+	{ PCI_VENDOR_ID_ADVANTECH, 0x1710, PCI_ANY_ID, PCI_ANY_ID, 0, 0, 0 },
+	{ PCI_VENDOR_ID_ADVANTECH, 0x1711, PCI_ANY_ID, PCI_ANY_ID, 0, 0, 0 },
+	{ PCI_VENDOR_ID_ADVANTECH, 0x1713, PCI_ANY_ID, PCI_ANY_ID, 0, 0, 0 },
+	{ PCI_VENDOR_ID_ADVANTECH, 0x1720, PCI_ANY_ID, PCI_ANY_ID, 0, 0, 0 },
+	{ PCI_VENDOR_ID_ADVANTECH, 0x1731, PCI_ANY_ID, PCI_ANY_ID, 0, 0, 0 },
 	{ 0 }
 };
 MODULE_DEVICE_TABLE(pci, pci1710_pci_table);
 
 static boardtype boardtypes[] =
 {
-	{"pci1710", ADVANTECH_VENDOR, 0x1710,
+	{"pci1710", 0x1710,
 	 IORANGE_171x, 1, TYPE_PCI171X,
 	 16, 8, 2, 16, 16,  0x0fff, 0x0fff,
 	 &range_pci1710_3, range_codes_pci1710_3, &range_pci171x_da,
 	 10000, 2048 },
-	{"pci1710hg", ADVANTECH_VENDOR, 0x1710,
+	{"pci1710hg", 0x1710,
 	 IORANGE_171x, 1, TYPE_PCI171X,
 	 16, 8, 2, 16, 16,  0x0fff, 0x0fff,
 	 &range_pci1710hg, range_codes_pci1710hg, &range_pci171x_da,
 	 10000, 2048 },
-	{"pci1711", ADVANTECH_VENDOR, 0x1711,
+	{"pci1711", 0x1711,
 	 IORANGE_171x, 1, TYPE_PCI171X,
 	 16, 0, 2, 16, 16,  0x0fff, 0x0fff,
 	 &range_pci17x1, range_codes_pci17x1, &range_pci171x_da,
 	 10000, 512 },
-	{"pci1713", ADVANTECH_VENDOR, 0x1713,
+	{"pci1713", 0x1713,
 	 IORANGE_171x, 1, TYPE_PCI1713,
 	 32,16, 0,  0,  0,  0x0fff, 0x0000,
 	 &range_pci17x1, range_codes_pci1710_3, NULL,
 	 10000, 2048 },
-	{"pci1720", ADVANTECH_VENDOR, 0x1720,
+	{"pci1720", 0x1720,
 	 IORANGE_1720, 0, TYPE_PCI1720,
 	 0, 0, 4, 0, 0,  0x0000, 0x0fff,
 	 NULL, NULL, &range_pci1720,
 	 0, 0 },
-	{"pci1731", ADVANTECH_VENDOR, 0x1731,
+	{"pci1731", 0x1731,
 	 IORANGE_171x, 1, TYPE_PCI171X,
 	 16, 0, 0, 16, 16,  0x0fff, 0x0000,
 	 &range_pci17x1, range_codes_pci17x1, NULL,
@@ -274,6 +261,10 @@ typedef struct{
 	unsigned int		ai_act_chan;// actual position in actual scan
 	unsigned int 		ai_buf_ptr;	// data buffer ptr in samples
 	unsigned char		ai_eos;		// 1=EOS wake up
+	unsigned char		ai_et;
+	unsigned int		ai_et_CntrlReg;
+	unsigned int		ai_et_MuxVal;
+	unsigned int		ai_et_div1, ai_et_div2;
 	unsigned int		act_chanlist[32];// list of scaned channel
 	unsigned char		act_chanlist_len;// len of scanlist
 	unsigned char 		act_chanlist_pos;// actual position in MUX list
@@ -332,7 +323,7 @@ static int pci171x_insn_read_ai(comedi_device * dev, comedi_subdevice * s, comed
 	for (n=0; n<insn->n; n++) {
 		outw(0, dev->iobase+PCI171x_SOFTTRG); /* start conversion */
     		DPRINTK("adv_pci1710 B n=%d ST=%4x\n",n,inw(dev->iobase+PCI171x_STATUS));
-		//udelay(1);
+		//comedi_udelay(1);
     		DPRINTK("adv_pci1710 C n=%d ST=%4x\n",n,inw(dev->iobase+PCI171x_STATUS));
     		timeout=100;
     		while (timeout--) {
@@ -464,10 +455,10 @@ static int pci1720_insn_write_ao(comedi_device * dev, comedi_subdevice * s, come
 	return n;
 }
 
-/* 
+/*
 ==============================================================================
 */
-static void interrupt_pci1710_every_sample(void *d) 
+static void interrupt_pci1710_every_sample(void *d)
 {
 	comedi_device 	*dev = d;
 	comedi_subdevice *s = dev->subdevices + 0;
@@ -508,29 +499,17 @@ static void interrupt_pci1710_every_sample(void *d)
 				comedi_event(dev,s,s->async->events);
 				return;
 			}
-		*(sampl_t *)((void *)(devpriv->ai_data)+s->async->buf_int_ptr)=sampl & 0x0fff;
 		DPRINTK("%8d %2d %8d~",s->async->buf_int_ptr,s->async->cur_chan,s->async->buf_int_count);
+		comedi_buf_put( s->async, sampl & 0x0fff);
 #else
-		*(sampl_t *)((void *)(devpriv->ai_data)+s->async->buf_int_ptr)=inw(dev->iobase+PCI171x_AD_DATA) & 0x0fff;
+		comedi_buf_put( s->async, inw(dev->iobase+PCI171x_AD_DATA) & 0x0fff);
 #endif
-		s->async->buf_int_count+=sizeof(sampl_t);
-		s->async->buf_int_ptr+=sizeof(sampl_t);
-		s->async->cur_chan++;
 
-		if (s->async->buf_int_ptr>=devpriv->ai_data_len) {	// buffer rollover
-			s->async->buf_int_ptr = 0;
-        		DPRINTK("adv_pci1710 EDBG: EOBUF1 bic %d bip %d buc %d bup %d\n",s->async->buf_int_count,s->async->buf_int_ptr, s->async->buf_user_count, s->async->buf_user_ptr);
-			s->async->events |= COMEDI_CB_BLOCK;
-        		DPRINTK("adv_pci1710 EDBG: EOBUF2\n");
-		}
-
-		if(s->async->cur_chan>=devpriv->ai_n_chan){	// one scan done
-			s->async->cur_chan=0;
+		if(s->async->cur_chan == 0){	// one scan done
 		        devpriv->ai_act_scan++;
         		DPRINTK("adv_pci1710 EDBG: EOS1 bic %d bip %d buc %d bup %d\n",s->async->buf_int_count,s->async->buf_int_ptr, s->async->buf_user_count, s->async->buf_user_ptr);
-			s->async->events |= COMEDI_CB_EOS;
         		DPRINTK("adv_pci1710 EDBG: EOS2\n");
-    			if ((!devpriv->neverending_ai)&&(devpriv->ai_act_scan>=devpriv->ai_scans)) { // all data sampled 
+    			if ((!devpriv->neverending_ai)&&(devpriv->ai_act_scan>=devpriv->ai_scans)) { // all data sampled
 		    		pci171x_ai_cancel(dev,s);
 				s->async->events |= COMEDI_CB_EOA;
 				comedi_event(dev,s,s->async->events);
@@ -548,7 +527,7 @@ static void interrupt_pci1710_every_sample(void *d)
 /* 
 ==============================================================================
 */
-static int move_block_from_fifo(comedi_device *dev,comedi_subdevice *s,sampl_t *data,int n,int turn)
+static int move_block_from_fifo(comedi_device *dev,comedi_subdevice *s, int n, int turn)
 {
 	int i,j;
 #ifdef PCI171x_PARANOIDCHECK
@@ -563,26 +542,21 @@ static int move_block_from_fifo(comedi_device *dev,comedi_subdevice *s,sampl_t *
 			if ((sampl & 0xf000)!=devpriv->act_chanlist[j]) {
 	    			rt_printk("comedi%d: A/D  FIFO data dropout: received data from channel %d, expected %d! (%d/%d/%d/%d/%d/%4x)\n",
 					 dev->minor, (sampl & 0xf000)>>12,(devpriv->act_chanlist[j] & 0xf000)>>12, i, j, devpriv->ai_act_scan, n, turn, sampl);
-				s->async->buf_int_count+=i*sizeof(sampl_t);
-				s->async->buf_int_ptr+=i*sizeof(sampl_t);
-				s->async->cur_chan=j;
 	    			pci171x_ai_cancel(dev,s);
 				s->async->events |= COMEDI_CB_EOA | COMEDI_CB_ERROR;
 				comedi_event(dev,s,s->async->events);
 				return 1;
 			}
-		*data=sampl & 0x0fff;
+		comedi_buf_put( s->async, sampl & 0x0fff );
 #else
-		*data=inw(dev->iobase+PCI171x_AD_DATA) & 0x0fff;
+		comedi_buf_put( s->async, inw(dev->iobase+PCI171x_AD_DATA) & 0x0fff );
 #endif
-		data++;
 		j++;
 		if(j>=devpriv->ai_n_chan){
 			j=0;
 		        devpriv->ai_act_scan++;
 		}
 	}
-	s->async->cur_chan=j;
 	DPRINTK("adv_pci1710 EDBG: END: move_block_from_fifo(...)\n");
 	return 0;
 }
@@ -614,28 +588,16 @@ static void interrupt_pci1710_half_fifo(void *d)
 	}
 
 	samplesinbuf=this_board->fifo_half_size;
-	if(s->async->buf_int_ptr+samplesinbuf*sizeof(sampl_t)>=devpriv->ai_data_len){
-		m=(devpriv->ai_data_len-s->async->buf_int_ptr)/sizeof(sampl_t);
-		if (move_block_from_fifo(dev,s,((void *)(devpriv->ai_data))+s->async->buf_int_ptr,m,0))
+	if(samplesinbuf*sizeof(sampl_t)>=devpriv->ai_data_len){
+		m=devpriv->ai_data_len/sizeof(sampl_t);
+		if (move_block_from_fifo(dev,s,m,0))
 			return;
-		s->async->buf_int_count+=m*sizeof(sampl_t);
 		samplesinbuf-=m;
-		s->async->buf_int_ptr=0;
-    		DPRINTK("adv_pci1710 EDBG: EOBUF1 bic %d bip %d buc %d bup %d\n",s->async->buf_int_count,s->async->buf_int_ptr, s->async->buf_user_count, s->async->buf_user_ptr);
-		s->async->events |= COMEDI_CB_EOBUF;
-    		DPRINTK("adv_pci1710 EDBG: EOBUF2\n");
 	}
 
 	if (samplesinbuf) {
-		if (move_block_from_fifo(dev,s,((void *)(devpriv->ai_data))+s->async->buf_int_ptr,samplesinbuf,1))
+		if (move_block_from_fifo(dev,s,samplesinbuf,1))
 			return;
-
-		s->async->buf_int_count+=samplesinbuf*sizeof(sampl_t);
-		s->async->buf_int_ptr+=samplesinbuf*sizeof(sampl_t);
-
-		DPRINTK("adv_pci1710 EDBG: BUFCHECK1 bic %d bip %d buc %d bup %d\n",s->async->buf_int_count,s->async->buf_int_ptr, s->async->buf_user_count, s->async->buf_user_ptr);
-		s->async->events |= COMEDI_CB_BLOCK;
-		DPRINTK("adv_pci1710 EDBG: BUFCHECK2\n");
 	}
 
 	if (!devpriv->neverending_ai)
@@ -648,34 +610,48 @@ static void interrupt_pci1710_half_fifo(void *d)
 	outb(0, dev->iobase + PCI171x_CLRINT);			// clear our INT request
 	DPRINTK("adv_pci1710 EDBG: END: interrupt_pci1710_half_fifo(...)\n");
 
-	comedi_event(dev,s,s->async->events); 
+	comedi_event(dev,s,s->async->events);
 }
 
-/* 
+/*
 ==============================================================================
 */
-static void interrupt_service_pci1710(int irq, void *d, struct pt_regs *regs) 
+static irqreturn_t interrupt_service_pci1710(int irq, void *d, struct pt_regs *regs)
 {
-        comedi_device *dev = d;
-	
+	comedi_device *dev = d;
+	int retval = 1;
+
 	DPRINTK("adv_pci1710 EDBG: BGN: interrupt_service_pci1710(%d,...)\n",irq);
 	if (!(inw(dev->iobase + PCI171x_STATUS) & Status_IRQ)) 	// is this interrupt from our board?
-	    return;						// no, exit
+		return IRQ_NONE; // no, exit
 
 	DPRINTK("adv_pci1710 EDBG: interrupt_service_pci1710() ST: %4x\n",inw(dev->iobase + PCI171x_STATUS));
 
+	if (devpriv->ai_et) {	// Switch from initial TRIG_EXT to TRIG_xxx.
+		devpriv->ai_et = 0;
+		outw(Control_SW, dev->iobase+PCI171x_CONTROL);
+		devpriv->CntrlReg=devpriv->ai_et_CntrlReg;
+		outb(0,dev->iobase + PCI171x_CLRFIFO);
+		outb(0,dev->iobase + PCI171x_CLRINT);
+		outw(devpriv->ai_et_MuxVal, dev->iobase+PCI171x_MUX);
+		outw(devpriv->CntrlReg, dev->iobase+PCI171x_CONTROL);
+		// start pacer
+		start_pacer(dev, 1, devpriv->ai_et_div1, devpriv->ai_et_div2);
+		return IRQ_RETVAL(retval);
+	}
 	if (devpriv->ai_eos) {  // We use FIFO half full INT or not?
 		interrupt_pci1710_every_sample(d);
 	} else {
 		interrupt_pci1710_half_fifo(d);
 	}
 	DPRINTK("adv_pci1710 EDBG: END: interrupt_service_pci1710(...)\n");
+	return IRQ_RETVAL(retval);
 }
 
-/* 
+/*
 ==============================================================================
 */
-static int pci171x_ai_docmd_and_mode(int mode, comedi_device * dev, comedi_subdevice * s) 
+static int pci171x_ai_docmd_and_mode(int mode, comedi_device * dev, comedi_subdevice * s)
 {
         unsigned int divisor1, divisor2;
 	unsigned int seglen;
@@ -711,12 +687,27 @@ static int pci171x_ai_docmd_and_mode(int mode, comedi_device * dev, comedi_subde
 							else { devpriv->neverending_ai=0; }
 	switch (mode) {
 	case 1:
+	case 2:
 		if (devpriv->ai_timer1<this_board->ai_ns_min) devpriv->ai_timer1=this_board->ai_ns_min;
 		devpriv->CntrlReg|=Control_PACER|Control_IRQEN;
+		if (mode == 2) {
+			devpriv->ai_et_CntrlReg=devpriv->CntrlReg;
+			devpriv->CntrlReg&=~(Control_PACER|Control_ONEFH|Control_GATE);
+			devpriv->CntrlReg|=Control_EXT;
+			devpriv->ai_et=1;
+		} else {
+			devpriv->ai_et=0;
+		}
 		i8253_cascade_ns_to_timer(devpriv->i8254_osc_base,&divisor1,&divisor2,&devpriv->ai_timer1,devpriv->ai_flags&TRIG_ROUND_MASK);
                 DPRINTK("adv_pci1710 EDBG: OSC base=%u div1=%u div2=%u timer=%u\n",devpriv->i8254_osc_base,divisor1,divisor2,devpriv->ai_timer1);
 		outw(devpriv->CntrlReg, dev->iobase+PCI171x_CONTROL);
-    		start_pacer(dev, mode, divisor1, divisor2);	// start pacer
+		if (mode != 2) {
+			// start pacer
+			start_pacer(dev, mode, divisor1, divisor2);
+		} else {
+			devpriv->ai_et_div1 = divisor1;
+			devpriv->ai_et_div2 = divisor2;
+		}
 		break;
 	case 3:
 		devpriv->CntrlReg|=Control_EXT|Control_IRQEN;
@@ -757,7 +748,7 @@ static int pci171x_ai_cmdtest(comedi_device *dev,comedi_subdevice *s,comedi_cmd
 	/* step 1: make sure trigger sources are trivially valid */
 
 	tmp=cmd->start_src;
-	cmd->start_src &= TRIG_NOW;
+	cmd->start_src &= TRIG_NOW|TRIG_EXT;
 	if(!cmd->start_src || tmp!=cmd->start_src)err++;
 
 	tmp=cmd->scan_begin_src;
@@ -786,7 +777,7 @@ static int pci171x_ai_cmdtest(comedi_device *dev,comedi_subdevice *s,comedi_cmd
 
 	/* step 2: make sure trigger sources are unique and mutually compatible */
 
-	if(cmd->start_src!=TRIG_NOW) {
+	if(cmd->start_src!=TRIG_NOW&&cmd->start_src!=TRIG_EXT) {
 		cmd->start_src=TRIG_NOW;
 		err++;
 	}
@@ -908,18 +899,18 @@ static int pci171x_ai_cmd(comedi_device *dev,comedi_subdevice *s)
 	devpriv->ai_n_chan=cmd->chanlist_len;
 	devpriv->ai_chanlist=cmd->chanlist;
 	devpriv->ai_flags=cmd->flags;
-	devpriv->ai_data_len=s->async->data_len;
-	devpriv->ai_data=s->async->data;
+	devpriv->ai_data_len=s->async->prealloc_bufsz;
+	devpriv->ai_data=s->async->prealloc_buf;
 	devpriv->ai_timer1=0;
 	devpriv->ai_timer2=0;
 
 	if (cmd->stop_src==TRIG_COUNT) { devpriv->ai_scans=cmd->stop_arg; }
 				else   { devpriv->ai_scans=0; }
 
-	if(cmd->scan_begin_src==TRIG_FOLLOW){ // mode 1 or 3
-		if (cmd->convert_src==TRIG_TIMER) { // mode 1
+	if(cmd->scan_begin_src==TRIG_FOLLOW){ // mode 1, 2, 3
+		if (cmd->convert_src==TRIG_TIMER) { // mode 1 and 2
 			devpriv->ai_timer1=cmd->convert_arg;
-			return pci171x_ai_docmd_and_mode(1,dev,s);
+			return pci171x_ai_docmd_and_mode(cmd->start_src==TRIG_EXT?2:1,dev,s);
 		}
 		if (cmd->convert_src==TRIG_EXT) { // mode 3
 			return pci171x_ai_docmd_and_mode(3,dev,s);
@@ -1004,7 +995,9 @@ static void setup_channel_list(comedi_device * dev, comedi_subdevice * s, unsign
 		DPRINTK("GS: %2d. [%4x]=%4x %4x\n", i, chanprog, range, devpriv->act_chanlist[i]);
 	}
 
-	outw(CR_CHAN(chanlist[0]) | (CR_CHAN(chanlist[seglen-1]) << 8) , dev->iobase+PCI171x_MUX); /* select channel interval to scan */
+	devpriv->ai_et_MuxVal =
+		CR_CHAN(chanlist[0]) | (CR_CHAN(chanlist[seglen-1]) << 8);
+	outw(devpriv->ai_et_MuxVal, dev->iobase+PCI171x_MUX); /* select channel interval to scan */
 	DPRINTK("MUX: %4x L%4x.H%4x\n", CR_CHAN(chanlist[0]) | (CR_CHAN(chanlist[seglen-1]) << 8), CR_CHAN(chanlist[0]), CR_CHAN(chanlist[seglen-1]));
 }
 
@@ -1126,14 +1119,14 @@ static int pci1710_reset(comedi_device *dev)
 static int pci1710_attach(comedi_device *dev,comedi_devconfig *it)
 {
 	comedi_subdevice *s;
-	int ret,subdev;
+	int ret,subdev,n_subdevices;
 	unsigned short io_addr[5],master,irq;
 	struct pcilst_struct *card=NULL;
         unsigned int iobase;
 	unsigned char pci_bus,pci_slot,pci_func;
-	
+
 	if (!pci_list_builded) {
-		pci_card_list_init(ADVANTECH_VENDOR,
+		pci_card_list_init(PCI_VENDOR_ID_ADVANTECH,
 #ifdef PCI171X_EXTDEBUG
 						    1
 #else
@@ -1145,20 +1138,21 @@ static int pci1710_attach(comedi_device *dev,comedi_devconfig *it)
 
 	rt_printk("comedi%d: adv_pci1710: board=%s",dev->minor,this_board->name);
 
-	if ((card=select_and_alloc_pci_card(ADVANTECH_VENDOR, this_board->device_id, it->options[0], it->options[1]))==NULL) 
+	/* this call pci_enable_device() */
+	if ((card=select_and_alloc_pci_card(PCI_VENDOR_ID_ADVANTECH, this_board->device_id, it->options[0], it->options[1], 0))==NULL)
 		return -EIO;
-	
+
 	if ((pci_card_data(card,&pci_bus,&pci_slot,&pci_func,
 	                    &io_addr[0],&irq,&master))<0) {
 		pci_card_free(card);
 		rt_printk(" - Can't get configuration data!\n");
 		return -EIO;
 	}
-	
+
 	iobase=io_addr[2];
-		
+
 	rt_printk(", b:s:f=%d:%d:%d, io=0x%4x",pci_bus,pci_slot,pci_func,iobase);
-	
+
         if (check_region(iobase, this_board->iorange) < 0) {
 		pci_card_free(card);
 		rt_printk("I/O port conflict\n");
@@ -1167,7 +1161,7 @@ static int pci1710_attach(comedi_device *dev,comedi_devconfig *it)
 
         request_region(iobase, this_board->iorange, "Advantech PCI-1710");
         dev->iobase=iobase;
-    
+
 	dev->board_name = this_board->name;
 
 	if((ret=alloc_private(dev,sizeof(pci1710_private)))<0) {
@@ -1175,14 +1169,14 @@ static int pci1710_attach(comedi_device *dev,comedi_devconfig *it)
 		pci_card_free(card);
 		return -ENOMEM;
 	}
-		
-        dev->n_subdevices = 0;
-	if (this_board->n_aichan) dev->n_subdevices++;
-	if (this_board->n_aochan) dev->n_subdevices++;
-	if (this_board->n_dichan) dev->n_subdevices++;
-	if (this_board->n_dochan) dev->n_subdevices++;
-	
-        if((ret=alloc_subdevices(dev))<0) {
+
+        n_subdevices = 0;
+	if (this_board->n_aichan) n_subdevices++;
+	if (this_board->n_aochan) n_subdevices++;
+	if (this_board->n_dichan) n_subdevices++;
+	if (this_board->n_dochan) n_subdevices++;
+
+        if((ret=alloc_subdevices(dev, n_subdevices))<0) {
     		release_region(dev->iobase, this_board->iorange);
 		pci_card_free(card);
     		return ret;
@@ -1232,7 +1226,7 @@ static int pci1710_attach(comedi_device *dev,comedi_devconfig *it)
 	if (this_board->n_aochan) {
 		s = dev->subdevices + subdev;
 		s->type = COMEDI_SUBD_AO;
-		s->subdev_flags = SDF_WRITEABLE|SDF_GROUND|SDF_COMMON;
+		s->subdev_flags = SDF_WRITABLE|SDF_GROUND|SDF_COMMON;
 		s->n_chan = this_board->n_aochan;
 		s->maxdata = this_board->ao_maxdata;
 		s->len_chanlist = this_board->n_aochan;
@@ -1265,7 +1259,7 @@ static int pci1710_attach(comedi_device *dev,comedi_devconfig *it)
 	if (this_board->n_dochan) {
 		s = dev->subdevices + subdev;
 		s->type = COMEDI_SUBD_DO;
-		s->subdev_flags = SDF_WRITEABLE|SDF_GROUND|SDF_COMMON;
+		s->subdev_flags = SDF_WRITABLE|SDF_GROUND|SDF_COMMON;
 		s->n_chan = this_board->n_dochan;
 		s->maxdata = 1;
 		s->len_chanlist = this_board->n_dochan;
@@ -1298,7 +1292,7 @@ static int pci1710_detach(comedi_device *dev)
 	if (dev->iobase) release_region(dev->iobase,this_board->iorange);
 
 	if (pci_list_builded) {
-	    	pci_card_list_cleanup(ADVANTECH_VENDOR);
+	    	pci_card_list_cleanup(PCI_VENDOR_ID_ADVANTECH);
 		pci_list_builded=0;
 	}
 
