@@ -27,24 +27,14 @@ Description: Driver for Amplicom PCI230 and PCI260 Multifunction I/O boards
 Author: Allan Willcox <allanwillcox@ozemail.com.au>
 Updated: Mon,  3 Sep 2001 17:37:12 -0700
 Devices: [Amplicon] PCI230 (amplc_pci230), PCI260
+Status: unknown
+
 */
 
-#include <linux/kernel.h>
-#include <linux/module.h>
-#include <linux/sched.h>
-#include <linux/mm.h>
-#include <linux/slab.h>
-#include <linux/errno.h>
-#include <linux/ioport.h>
-#include <linux/delay.h>
-#include <linux/interrupt.h>
-#include <linux/timex.h>
-#include <linux/timer.h>
 #include <linux/comedidev.h>
-#include <linux/pci.h>
-#include <linux/init.h>
 
-#include <asm/io.h>
+#include <linux/delay.h>
+#include <linux/pci.h>
 
 #include "8253.h"
 #include "8255.h"
@@ -259,7 +249,7 @@ static void pci230_z2_ct2(comedi_device *dev, unsigned int *ns,int round);
 static void pci230_cancel_ct0(comedi_device *dev);
 static void pci230_cancel_ct1(comedi_device *dev);
 static void pci230_cancel_ct2(comedi_device *dev);
-static void pci230_interrupt(int irq, void *d, struct pt_regs *regs);
+static irqreturn_t pci230_interrupt(int irq, void *d, struct pt_regs *regs);
 static int pci230_ao_cmdtest(comedi_device *dev,comedi_subdevice *s, comedi_cmd *cmd);
 static int pci230_ao_cmd(comedi_device *dev, comedi_subdevice *s);
 static int pci230_ao_cancel(comedi_device *dev, comedi_subdevice *s);
@@ -335,8 +325,8 @@ static int pci230_attach(comedi_device *dev,comedi_devconfig *it)
 	/* Read base addressses of the PCI230's two I/O regions from PCI configuration register. */
 	if(pci_enable_device(pci_dev)<0)return -EIO;
 
-	pci_iobase = pci_resource_start(pci_dev, 2) & PCI_BASE_ADDRESS_IO_MASK;
-	iobase = pci_resource_start(pci_dev, 3) & PCI_BASE_ADDRESS_IO_MASK;
+	pci_iobase = pci_resource_start(pci_dev, 2);
+	iobase = pci_resource_start(pci_dev, 3);
 
 	printk("comedi%d: amplc_pci230: I/O region 1 0x%04x I/O region 2 0x%04x\n",dev->minor, pci_iobase, iobase);
 
@@ -380,17 +370,15 @@ static int pci230_attach(comedi_device *dev,comedi_devconfig *it)
 
 /*
  * Allocate the subdevice structures.  alloc_subdevice() is a
- * convenient macro defined in comedidev.h.  It relies on
- * n_subdevices being set correctly.
+ * convenient macro defined in comedidev.h.
  */
-	dev->n_subdevices=4;
-	if(alloc_subdevices(dev)<0)
+	if(alloc_subdevices(dev, 4)<0)
 		return -ENOMEM;
 
 	s=dev->subdevices+0;
 	/* analog input subdevice */
 	s->type=COMEDI_SUBD_AI;
-	s->subdev_flags=SDF_READABLE;
+	s->subdev_flags=SDF_READABLE|SDF_DIFF|SDF_GROUND;
 	s->n_chan=thisboard->ai_chans;
 	s->maxdata=(1<<thisboard->ai_bits)-1;
 	s->range_table=&pci230_ai_range;
@@ -407,7 +395,7 @@ static int pci230_attach(comedi_device *dev,comedi_devconfig *it)
 	s=dev->subdevices+1;
 	/* analog output subdevice */
 	s->type=COMEDI_SUBD_AO;
-	s->subdev_flags=SDF_WRITEABLE;
+	s->subdev_flags=SDF_WRITABLE;
 	s->n_chan=thisboard->ao_chans;;
 	s->maxdata=(1<<thisboard->ao_bits)-1;
 	s->range_table=&pci230_ao_range;
@@ -530,9 +518,6 @@ static int pci230_ai_rinsn(comedi_device *dev,comedi_subdevice *s,comedi_insn *i
 	/* Specify uni/bip, se/diff, s/w conversion, and reset FIFO (even though we're not using it - MEV says so). */
 	outw_p(adccon, dev->iobase + PCI230_ADCCON);
 
-	/* Wait for mux to settle */
-	udelay(PCI230_MUX_SETTLE);
-
 	/* Convert n samples */
 	for(n=0;n<insn->n;n++){
 		/* trigger conversion */
@@ -583,14 +568,6 @@ static int pci230_ao_winsn(comedi_device *dev,comedi_subdevice *s,comedi_insn *i
 
 		/* Write value to DAC. */
 		pci230_ao_write(dev, data[i], chan);
-
-#if 0
-		/* XXX screw the user.  Only do this if the board gets upset if you don't */
-		/* If we're writing more than one sample, wait for output to settle between successive writes */
-		if (insn->n > 1) {
-	    	udelay(PCI230_DAC_SETTLE);
-		}
-#endif
 	}
 
 	/* return the number of samples read/written */
@@ -1146,11 +1123,12 @@ static void pci230_cancel_ct2(comedi_device *dev)
 }
 
 /* Interrupt handler */
-static void pci230_interrupt(int irq, void *d, struct pt_regs *regs)
+static irqreturn_t pci230_interrupt(int irq, void *d, struct pt_regs *regs)
 {
 	int status_int;
 	comedi_device *dev = (comedi_device*) d;
 	comedi_subdevice *s;
+	int retval = 1;
 
 	/* Read interrupt status/enable register. */
 	status_int = inb(devpriv->pci_iobase + PCI230_INT_SCE);
@@ -1186,7 +1164,7 @@ static void pci230_interrupt(int irq, void *d, struct pt_regs *regs)
 		s->async->events = 0;
 	}
 
-	return;
+	return IRQ_RETVAL(retval);
 }
 
 static void pci230_handle_ao(comedi_device *dev, comedi_subdevice *s) {
