@@ -4,10 +4,10 @@
     PCI-DDA series.
 
 	 Copyright (C) 2001 Ivan Martinez <ivanmr@altavista.com>
-    Copyright (C) 2001 Frank Mori Hess <fmhess@uiuc.edu>
+    Copyright (C) 2001 Frank Mori Hess <fmhess@users.sourceforge.net>
 
     COMEDI - Linux Control and Measurement Device Interface
-    Copyright (C) 1997-8 David A. Schleef <ds@stm.lbl.gov>
+    Copyright (C) 1997-8 David A. Schleef <ds@schleef.org>
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -40,27 +40,14 @@ Configuration options:
 
 Only simple analog output writing is supported.
 
-SO FAR IT HAS ONLY BEEN TESTED WITH:
+So far it has only been tested with:
   - PCI-DDA08/12
-PLEASE REPORT IF YOU ARE USING IT WITH A DIFFERENT CARD
-<ivanmr@altavista.com>.
+Please report sucess/failure with other different cards to
+<comedi@comedi.org>.
 */
 
-#include <linux/kernel.h>
-#include <linux/module.h>
-#include <linux/sched.h>
-#include <linux/mm.h>
-#include <linux/slab.h>
-#include <linux/errno.h>
-#include <linux/ioport.h>
-#include <linux/delay.h>
-#include <linux/interrupt.h>
-#include <linux/timex.h>
-#include <linux/timer.h>
-#include <linux/pci.h>
-#include <linux/init.h>
-#include <asm/io.h>
 #include <linux/comedidev.h>
+#include <linux/pci.h>
 
 #include "8255.h"
 
@@ -71,11 +58,11 @@ PLEASE REPORT IF YOU ARE USING IT WITH A DIFFERENT CARD
 
 /* PCI-DDA base addresses */
 #define DIGITALIO_BADRINDEX	2
-	// DIGITAL I/O is devpriv->pci_dev->base_address[2]
+	// DIGITAL I/O is pci_dev->resource[2]
 #define DIGITALIO_SIZE 8
 	// DIGITAL I/O uses 8 I/O port addresses
 #define DAC_BADRINDEX	3
-	// DAC is devpriv->pci_dev->base_address[3]
+	// DAC is pci_dev->resource[3]
 
 /* Digital I/O registers */
 #define PORT1A 0	// PORT 1A DATA
@@ -337,10 +324,8 @@ found:
 	 */
 	if(pci_enable_device(devpriv->pci_dev))
 		return -EIO;
-	digitalio = pci_resource_start(devpriv->pci_dev, DIGITALIO_BADRINDEX) &
-		PCI_BASE_ADDRESS_IO_MASK;
-	dac = pci_resource_start(devpriv->pci_dev, DAC_BADRINDEX) &
-		PCI_BASE_ADDRESS_IO_MASK;
+	digitalio = pci_resource_start(devpriv->pci_dev, DIGITALIO_BADRINDEX); 
+	dac = pci_resource_start(devpriv->pci_dev, DAC_BADRINDEX); 
 
 /*
  * Allocate the I/O ports.
@@ -379,14 +364,13 @@ found:
 /*
  * Allocate the subdevice structures.
  */
-	dev->n_subdevices = 3;
-	if(alloc_subdevices(dev) < 0)
+	if(alloc_subdevices( dev, 3 ) < 0)
 		return -ENOMEM;
 
 	s = dev->subdevices + 0;
 	/* analog output subdevice */
 	s->type = COMEDI_SUBD_AO;
-	s->subdev_flags = SDF_WRITEABLE;
+	s->subdev_flags = SDF_WRITABLE;
 	s->n_chan = thisboard->ao_chans;
 	s->maxdata = (1 << thisboard->ao_bits) - 1;
 	s->range_table = thisboard->ranges;
