@@ -36,10 +36,7 @@ option.  The latter is recommended, in order to save a bit of kernel
 memory.
 
 Configuration options:
-  [0] - pointer to FPGA initialization data
-          The pointer and size options are handled automatically
-          by comedi_config when you use the -i option.
-  [1] - size of FPGA data
+    none
 */
 /*
    This card was obviously never intended to leave the Windows world, 
@@ -115,26 +112,12 @@ Configuration options:
    
  */
 
-#include <linux/kernel.h>
-#include <linux/module.h>
-#include <linux/sched.h>
-#include <linux/mm.h>
-#include <linux/slab.h>
-#include <linux/errno.h>
-#include <linux/ioport.h>
-#include <linux/delay.h>
-#include <linux/interrupt.h>
-#include <linux/timex.h>
-#include <linux/timer.h>
-#include <linux/pci.h>
-#include <linux/init.h>
-#include <asm/io.h>
-#include <asm/uaccess.h>
 #include <linux/comedidev.h>
+
+#include <linux/delay.h>
+#include <linux/pci.h>
+
 #include "8255.h"
-#ifdef CONFIG_COMEDI_DAQBOARD2000_FPGA
-#include "daqboard2000_fpga.h"
-#endif
 
 #define DAQBOARD2000_SUBSYSTEM_IDS2 	0x00021616 /* Daqboard/2000 - 2 Dacs */
 #define DAQBOARD2000_SUBSYSTEM_IDS4 	0x00041616 /* Daqboard/2000 - 4 Dacs */
@@ -351,9 +334,9 @@ static void writeAcqScanListEntry(comedi_device *dev, u16 entry)
 {
   daqboard2000_hw *fpga = devpriv->daq;
 
-  udelay(4);
+  comedi_udelay(4);
   fpga->acqScanListFIFO = entry & 0x00ff;
-  udelay(4);
+  comedi_udelay(4);
   fpga->acqScanListFIFO = (entry >> 8) & 0x00ff;
 }
 
@@ -413,16 +396,16 @@ static int daqboard2000_ai_insn_read(comedi_device *dev, comedi_subdevice *s,
     fpga->acqControl = DAQBOARD2000_SeqStartScanList;
     for (timeout = 0 ; timeout < 20 ; timeout++) {
       if (fpga->acqControl & DAQBOARD2000_AcqConfigPipeFull) { break; }
-      //udelay(2);
+      //comedi_udelay(2);
     }
     fpga->acqControl = DAQBOARD2000_AdcPacerEnable;
     for (timeout = 0 ; timeout < 20 ; timeout++) {
       if (fpga->acqControl & DAQBOARD2000_AcqLogicScanning) { break; }
-      //udelay(2);
+      //comedi_udelay(2);
     }
     for (timeout = 0 ; timeout < 20 ; timeout++) {
       if (fpga->acqControl & DAQBOARD2000_AcqResultsFIFOHasValidData) { break;}
-      //udelay(2);
+      //comedi_udelay(2);
     }
     data[i] = fpga->acqResultsFIFO;
     fpga->acqControl = DAQBOARD2000_AdcPacerDisable;
@@ -459,16 +442,16 @@ static int daqboard2000_ao_insn_write(comedi_device *dev, comedi_subdevice *s,
      * OK, since it works OK without enabling the DAC's, let's keep
      * it as simple as possible...
      */
-    //fpga->dacControl = (chan + 2) * 0x0010 | 0x0001; udelay(1000);
+    //fpga->dacControl = (chan + 2) * 0x0010 | 0x0001; comedi_udelay(1000);
     fpga->dacSetting[chan] = data[i];
     for (timeout = 0 ; timeout < 20 ; timeout++) {
       if ((fpga->dacControl & ((chan + 1) * 0x0010)) == 0) { break; }
-      //udelay(2);
+      //comedi_udelay(2);
     }
     devpriv->ao_readback[chan] = data[i];
   /*  
    * Since we never enabled the DAC's, we don't need to disable it...
-   * fpga->dacControl = (chan + 2) * 0x0010 | 0x0000; udelay(1000);
+   * fpga->dacControl = (chan + 2) * 0x0010 | 0x0000; comedi_udelay(1000);
    */
   }
 
@@ -479,29 +462,29 @@ static void daqboard2000_resetLocalBus(comedi_device *dev)
 {
   printk("daqboard2000_resetLocalBus\n");
   writel(DAQBOARD2000_SECRLocalBusHi, devpriv->plx + 0x6c);
-  udelay(10000);
+  comedi_udelay(10000);
   writel(DAQBOARD2000_SECRLocalBusLo, devpriv->plx + 0x6c);
-  udelay(10000);
+  comedi_udelay(10000);
 }
 
 static void daqboard2000_reloadPLX(comedi_device *dev)
 {
   printk("daqboard2000_reloadPLX\n");
   writel(DAQBOARD2000_SECRReloadLo, devpriv->plx + 0x6c);
-  udelay(10000);
+  comedi_udelay(10000);
   writel(DAQBOARD2000_SECRReloadHi, devpriv->plx + 0x6c);
-  udelay(10000);
+  comedi_udelay(10000);
   writel(DAQBOARD2000_SECRReloadLo, devpriv->plx + 0x6c);
-  udelay(10000);
+  comedi_udelay(10000);
 }
 
 static void daqboard2000_pulseProgPin(comedi_device *dev)
 {
   printk("daqboard2000_pulseProgPin 1\n");
   writel(DAQBOARD2000_SECRProgPinHi, devpriv->plx + 0x6c);
-  udelay(10000);
+  comedi_udelay(10000);
   writel(DAQBOARD2000_SECRProgPinLo, devpriv->plx + 0x6c);
-  udelay(10000); /* Not in the original code, but I like symmetry... */
+  comedi_udelay(10000); /* Not in the original code, but I like symmetry... */
 }
 
 static int daqboard2000_pollCPLD(comedi_device *dev, int mask) {
@@ -516,9 +499,9 @@ static int daqboard2000_pollCPLD(comedi_device *dev, int mask) {
       result=1;
       break;
     }
-    udelay(100);
+    comedi_udelay(100);
   }
-  udelay(5);
+  comedi_udelay(5);
   return result;
 }
 
@@ -526,7 +509,7 @@ static int daqboard2000_writeCPLD(comedi_device *dev, int data)
 {
   int result = 0;
 
-  udelay(10);
+  comedi_udelay(10);
   writew(data, devpriv->daq + 0x1000);
   if ((readw(devpriv->daq + 0x1000) & DAQBOARD2000_CPLD_INIT) == 
       DAQBOARD2000_CPLD_INIT) {
@@ -601,17 +584,17 @@ static void daqboard2000_adcDisarm(comedi_device *dev)
   daqboard2000_hw *fpga = devpriv->daq;
 
   /* Disable hardware triggers */  
-  udelay(2);
+  comedi_udelay(2);
   fpga->trigControl = DAQBOARD2000_TrigAnalog | DAQBOARD2000_TrigDisable;
-  udelay(2);
+  comedi_udelay(2);
   fpga->trigControl = DAQBOARD2000_TrigTTL | DAQBOARD2000_TrigDisable; 
 
   /* Stop the scan list FIFO from loading the configuration pipe */
-  udelay(2);
+  comedi_udelay(2);
   fpga->acqControl = DAQBOARD2000_SeqStopScanList;
 
   /* Stop the pacer clock */
-  udelay(2);
+  comedi_udelay(2);
   fpga->acqControl = DAQBOARD2000_AdcPacerDisable; 
    
   /* Stop the input dma (abort channel 1) */
@@ -627,7 +610,7 @@ static void daqboard2000_activateReferenceDacs(comedi_device *dev)
   fpga->refDacs = 0x80 | DAQBOARD2000_PosRefDacSelect;
   for (timeout = 0 ; timeout < 20 ; timeout++) {
     if ((fpga->dacControl & DAQBOARD2000_RefBusy) == 0) { break; }
-    udelay(2);
+    comedi_udelay(2);
   }
 /*  printk("DAQBOARD2000_PosRefDacSelect %d\n", timeout);*/
 
@@ -635,7 +618,7 @@ static void daqboard2000_activateReferenceDacs(comedi_device *dev)
   fpga->refDacs = 0x80 | DAQBOARD2000_NegRefDacSelect;
   for (timeout = 0 ; timeout < 20 ; timeout++) {
     if ((fpga->dacControl & DAQBOARD2000_RefBusy) == 0) { break; }
-    udelay(2);
+    comedi_udelay(2);
   }
 /*  printk("DAQBOARD2000_NegRefDacSelect %d\n", timeout);*/
 }
@@ -695,15 +678,11 @@ static int daqboard2000_attach(comedi_device *dev, comedi_devconfig *it)
   int result = 0;
   comedi_subdevice *s;
   struct pci_dev *card = NULL;
+  void *aux_data;
+  unsigned int aux_len;
 
   printk("comedi%d: daqboard2000:", dev->minor);
 
-  if(!pci_present()) {
-    printk(" PCI bus not present!\n");
-    result = -EIO;
-    goto out;
-  }
-
   /* FIXME: we should handle multiple cards, have to make David decide 
             how, so we will be consistent among all PCI card drivers... */
   card = pci_find_device(0x1616, 0x0409, NULL);
@@ -741,17 +720,11 @@ static int daqboard2000_attach(comedi_device *dev, comedi_devconfig *it)
   result = alloc_private(dev,sizeof(daqboard2000_private));
   if(result<0)goto out;
 
-  dev->n_subdevices = 3;
-  result = alloc_subdevices(dev);
+  result = alloc_subdevices(dev, 3);
   if(result<0)goto out;
   
-#if LINUX_VERSION_CODE < 0x020300
-  devpriv->plx = ioremap(card->base_address[0], DAQBOARD2000_PLX_SIZE);
-  devpriv->daq = ioremap(card->base_address[2], DAQBOARD2000_DAQ_SIZE);
-#else
-  devpriv->plx = ioremap(card->resource[0].start, DAQBOARD2000_PLX_SIZE);
-  devpriv->daq = ioremap(card->resource[2].start, DAQBOARD2000_DAQ_SIZE);
-#endif
+  devpriv->plx = ioremap(pci_resource_start(card,0), DAQBOARD2000_PLX_SIZE);
+  devpriv->daq = ioremap(pci_resource_start(card,2), DAQBOARD2000_DAQ_SIZE);
   readl(devpriv->plx + 0x6c); 
 
   /*
@@ -760,33 +733,15 @@ static int daqboard2000_attach(comedi_device *dev, comedi_devconfig *it)
     pci_read_config_byte(card, PCI_INTERRUPT_LINE, &interrupt);
     printk("Interrupt before is: %x\n", interrupt);
   */
-  if(it->options[0] && it->options[1]){
-    void *ptr;
-    unsigned int size = it->options[1];
 
-    /* Note:  using copy_from_user here is a hack.  I would prefer
-     * to have a generic solution for copying extended data at
-     * device configuration, but that will require changes to the
-     * devconfig interface.  Wait until devconfig gets a little
-     * more crufty (than it already is?!?), and then fix it.  -ds */
-    ptr = kmalloc(size,GFP_KERNEL);
-    if(!ptr){
-      result=-ENOMEM;
-      goto out;
-    }
-    if(copy_from_user(ptr,(void *)it->options[0],size)){
-      result=-EFAULT;
-      goto out;
-    }
-    result = initialize_daqboard2000(dev,ptr,size);
-    kfree(ptr);
+  aux_data = (void *)it->options[COMEDI_DEVCONF_AUX_DATA];
+  aux_len = it->options[COMEDI_DEVCONF_AUX_DATA_LENGTH];
+
+  if(aux_data && aux_len){
+    result = initialize_daqboard2000(dev, aux_data, aux_len);
   }else{
-#ifdef CONFIG_COMEDI_DAQBOARD2000_FPGA
-    result = initialize_daqboard2000(dev,bTopBitArray,bTopBitSize);
-#else
     printk("no FPGA initialization code, aborting\n");
     result=-EIO;
-#endif
   }
   if(result<0)goto out;
   daqboard2000_initializeAdc(dev);
@@ -804,7 +759,7 @@ static int daqboard2000_attach(comedi_device *dev, comedi_devconfig *it)
   s = dev->subdevices + 0;
   /* ai subdevice */
   s->type = COMEDI_SUBD_AI;
-  s->subdev_flags = SDF_READABLE|SDF_RT;
+  s->subdev_flags = SDF_READABLE|SDF_GROUND;
   s->n_chan = 24;
   s->maxdata = 0xffff;
   s->insn_read = daqboard2000_ai_insn_read;
@@ -813,7 +768,7 @@ static int daqboard2000_attach(comedi_device *dev, comedi_devconfig *it)
   s = dev->subdevices + 1;
   /* ao subdevice */
   s->type = COMEDI_SUBD_AO;
-  s->subdev_flags = SDF_WRITEABLE|SDF_RT;
+  s->subdev_flags = SDF_WRITABLE;
   s->n_chan = 2;
   s->maxdata = 0xffff;
   s->insn_read = daqboard2000_ao_insn_read;
