@@ -3,9 +3,9 @@
     DAS16 driver
 
     COMEDI - Linux Control and Measurement Device Interface
-    Copyright (C) 2000 David A. Schleef <ds@stm.lbl.gov>
+    Copyright (C) 2000 David A. Schleef <ds@schleef.org>
     Copyright (C) 2000 Chris R. Baugher <baugher@enteract.com>
-    Copyright (C) 2001 Frank Mori Hess <fmhess@uiuc.edu>
+    Copyright (C) 2001,2002 Frank Mori Hess <fmhess@users.sourceforge.net>
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -26,82 +26,79 @@
 /*
 Driver: das16.o
 Description: DAS16 compatible boards
-Authors: Sam Moore, Warren Jasper, ds, Chris Baugher, Frank Hess
+Author: Sam Moore, Warren Jasper, ds, Chris Baugher, Frank Hess, Roman Fietze
 Devices: [Keithley Metrabyte] DAS-16 (das-16), DAS-16G (das-16g),
   DAS-16F (das-16f), DAS-1201 (das-1201), DAS-1202 (das-1202),
   DAS-1401 (das-1401), DAS-1402 (das-1402), DAS-1601 (das-1601),
   DAS-1602 (das-1602),
   [ComputerBoards] PC104-DAS16/JR (pc104-das16jr),
   PC104-DAS16JR/16 (pc104-das16jr/16),
+  CIO-DAS16JR/16 (cio-das16jr/16),
   CIO-DAS16/JR (cio-das16/jr), CIO-DAS1401/12 (cio-das1401/12),
   CIO-DAS1402/12 (cio-das1402/12), CIO-DAS1402/16 (cio-das1402/16),
   CIO-DAS1601/12 (cio-das1601/12), CIO-DAS1602/12 (cio-das1602/12),
   CIO-DAS1602/16 (cio-das1602/16), CIO-DAS16/330 (cio-das16/330)
-Status: works in das16 mode, das-1600 enhanced mode features untested.
-Updated: 2001-8-27
+Status: works
+Updated: 2002-09-30
 
 A rewrite of the das16 and das1600 drivers.
 Options:
-        [0] - base io address
-        [1] - irq (optional)
-        [2] - dma (optional)
-        [3] - master clock speed in MHz (optional, 1 or 10, ignored if
-	        board can probe clock, defaults to 1)
-        [4] - analog input range lowest voltage in microvolts (optional,
-	        only useful if your board does not have software
+	[0] - base io address
+	[1] - irq (optional)
+	[2] - dma (optional)
+	[3] - master clock speed in MHz (optional, 1 or 10, ignored if
+		board can probe clock, defaults to 1)
+	[4] - analog input range lowest voltage in microvolts (optional,
+		only useful if your board does not have software
 		programmable gain)
-        [5] - analog input range highest voltage in microvolts (optional,
-	        only useful if board does not have software programmable
+	[5] - analog input range highest voltage in microvolts (optional,
+		only useful if board does not have software programmable
 		gain)
-        [6] - analog output range lowest voltage in microvolts (optional)
-        [7] - analog output range highest voltage in microvolts (optional)
+	[6] - analog output range lowest voltage in microvolts (optional)
+	[7] - analog output range highest voltage in microvolts (optional)
+	[8] - use timer mode for DMA, needed e.g. for buggy DMA controller
+		in NS CS5530A (Geode Companion), and for 'jr' cards that
+		lack a hardware fifo.  If set, also allows
+		comedi_command() to be run without an irq.
 
 Passing a zero for an option is the same as leaving it unspecified.
 
-Both an irq line and dma channel are required for timed or externally
-triggered conversions.
+Both a dma channel and an irq (or use of 'timer mode', option 8) are required
+for timed or externally triggered conversions.  The JR versions of the boards
+should use 'timer mode' instead of an irq, due to their lack of a hardware
+fifo.
+
 */
 /*
 
 Testing and debugging help provided by Daniel Koch.
 
-Options:
-	[0] - base io address
-	[1] - irq (optional)
-	[2] - dma (optional)
-	[3] - master clock speed in MHz (optional, 1 or 10, ignored if board
-		can probe clock speed, defaults to 1 otherwise)
-	[4] - analog input range lowest in microvolts (optional, only useful if your
-		board does not have software programmable gain)
-	[5] - analog input range highest in microvolts (optional, only useful if your
-		board does not have software programmable gain)
-	[6] - analog output range lowest in microvolts (optional)
-	[7] - analog output range highest in microvolts (optional)
-
-Both an irq line and dma channel are required for timed or externally
-triggered conversions.
-
 Keithley Manuals:
 	2309.PDF (das16)
+	4919.PDF (das1400, 1600)
+	4922.PDF (das-1400)
 	4923.PDF (das1200, 1400, 1600)
 
 Computer boards manuals also available from their website www.measurementcomputing.com
 
 */
 
-#include <linux/kernel.h>
-#include <linux/module.h>
 #include <linux/comedidev.h>
-#include <linux/errno.h>
-#include <linux/ioport.h>
-#include <asm/io.h>
-#include <linux/slab.h>
-#include <linux/delay.h>
+#include <linux/pci.h>
 #include <asm/dma.h>
+
 #include "8253.h"
 #include "8255.h"
+#include "comedi_fc.h"
 
 #undef DEBUG
+//#define DEBUG
+
+#ifdef DEBUG
+#define DEBUG_PRINT(format, args...) rt_printk("das16: " format, ## args)
+#else
+#define DEBUG_PRINT(format, args...)
+#endif
 
 #define DAS16_SIZE 20	// number of ioports
 #define DAS16_DMA_SIZE 0xff00 // size in bytes of allocated dma buffer
@@ -147,6 +144,23 @@ Computer boards manuals also available from their website www.measurementcomputi
 
 */
 
+/*
+    cio-das16jr_16.pdf
+
+    "das16jr_16"
+
+  0	a/d bits 0-7		start 16 bit
+  1	a/d bits 8-15		unused
+  2	mux read		mux set
+  3	di 4 bit		do 4 bit
+  4567	unused			unused
+  8	status eoc uni/bip	interrupt reset
+  9	dma, int, trig ctrl	set dma, int
+  a	pacer control		unused
+  b	gain status		gain control
+  cdef	8254
+
+*/
 /*
     cio-das160x-1x.pdf
 
@@ -193,6 +207,7 @@ static const int sample_size = 2;	// size in bytes of a sample from board
 #define   DAS16_INTE			(1<<7)
 #define   DAS16_IRQ(x)			(((x) & 0x7) << 4)
 #define   DMA_ENABLE			(1<<2)
+#define   PACING_MASK	0x3
 #define   INT_PACER		0x03
 #define   EXT_PACER			0x02
 #define   DAS16_SOFT		0x00
@@ -262,34 +277,55 @@ static comedi_lrange range_das16jr = { 9, {
 	UNI_RANGE( 2.5 ),
 	UNI_RANGE( 1.25 ),
 }};
+static comedi_lrange range_das16jr_16 = { 8, {
+	BIP_RANGE( 10 ),
+	BIP_RANGE( 5 ),
+	BIP_RANGE( 2.5 ),
+	BIP_RANGE( 1.25 ),
+	UNI_RANGE( 10 ),
+	UNI_RANGE( 5 ),
+	UNI_RANGE( 2.5 ),
+	UNI_RANGE( 1.25 ),
+}};
 
 static int das16jr_gainlist[] = { 8, 0, 1, 2, 3, 4, 5, 6, 7 };
+static int das16jr_16_gainlist[] = { 0, 1, 2, 3, 4, 5, 6, 7 };
 static int das1600_gainlist[] = { 0, 1, 2, 3 };
 enum {
 	das16_pg_none = 0,
 	das16_pg_16jr,
+	das16_pg_16jr_16,
 	das16_pg_1601,
 	das16_pg_1602,
 };
 static int *das16_gainlists[] = {
 	NULL,
 	das16jr_gainlist,
+	das16jr_16_gainlist,
 	das1600_gainlist,
 	das1600_gainlist,
 };
 static comedi_lrange *das16_ai_uni_lranges[]={
 	&range_unknown,
 	&range_das16jr,
+	&range_das16jr_16,
 	&range_das1x01_unip,
 	&range_das1x02_unip,
 };
 static comedi_lrange *das16_ai_bip_lranges[]={
 	&range_unknown,
 	&range_das16jr,
+	&range_das16jr_16,
 	&range_das1x01_bip,
 	&range_das1x02_bip,
 };
 
+struct munge_info
+{
+	uint8_t byte;
+	unsigned have_byte : 1;
+};
+
 static int das16_ao_winsn(comedi_device *dev,comedi_subdevice *s,comedi_insn *insn,lsampl_t *data);
 static int das16_do_wbits(comedi_device *dev,comedi_subdevice *s,comedi_insn *insn,lsampl_t *data);
 static int das16_di_rbits(comedi_device *dev,comedi_subdevice *s,comedi_insn *insn,lsampl_t *data);
@@ -300,15 +336,20 @@ static int das16_cmd_exec(comedi_device *dev,comedi_subdevice *s);
 static int das16_cancel(comedi_device *dev, comedi_subdevice *s);
 
 static void das16_reset(comedi_device *dev);
-static void das16_interrupt(int irq, void *d, struct pt_regs *regs);
+static irqreturn_t das16_dma_interrupt(int irq, void *d, struct pt_regs *regs);
+static void das16_timer_interrupt(unsigned long arg);
+static void das16_interrupt(comedi_device *dev);
 
 static unsigned int das16_set_pacer(comedi_device *dev, unsigned int ns, int flags);
 static int das1600_mode_detect(comedi_device *dev);
-static unsigned int das16_suggest_transfer_size(comedi_cmd cmd);
+static unsigned int das16_suggest_transfer_size(comedi_device *dev, comedi_cmd cmd);
+
+static void init_munge_info( struct munge_info *info );
+static void write_byte_to_buffer( comedi_device *dev, comedi_subdevice *subd,
+	uint8_t raw_byte );
+static void das16_write_array_to_buffer( comedi_device *dev, void *data, unsigned int num_bytes );
 
-#ifdef DEBUG
 static void reg_dump(comedi_device *dev);
-#endif
 
 typedef struct das16_board_struct{
 	char		*name;
@@ -422,7 +463,7 @@ static struct das16_board_struct das16_boards[]={
 	name:		"pc104-das16jr",	// pc104-das16jr_xx.pdf
 	ai:		das16_ai_rinsn,
 	ai_nbits:	12,
-	ai_speed:	6667,
+	ai_speed:	3300,
 	ai_pg:		das16_pg_16jr,
 	ao:		NULL,
 	di:		das16_di_rbits,
@@ -433,11 +474,25 @@ static struct das16_board_struct das16_boards[]={
 	id:	0x00,
 	},
 	{
+	name:           "cio-das16jr/16",       // cio-das16jr_16.pdf
+	ai:             das16_ai_rinsn,
+	ai_nbits:       16,
+	ai_speed:       10000,
+	ai_pg:          das16_pg_16jr_16,
+	ao:             NULL,
+	di:             das16_di_rbits,
+	do_:            das16_do_wbits,
+	i8255_offset:   0,
+	i8254_offset:   0x0c,
+	size:           0x10,
+	id:             0x00,
+	},
+	{
 	name:		"pc104-das16jr/16",	// pc104-das16jr_xx.pdf
 	ai:		das16_ai_rinsn,
 	ai_nbits:	16,
 	ai_speed:	10000,
-	ai_pg:		das16_pg_16jr,
+	ai_pg:		das16_pg_16jr_16,
 	ao:		NULL,
 	di:		das16_di_rbits,
 	do_:		das16_do_wbits,
@@ -455,7 +510,7 @@ static struct das16_board_struct das16_boards[]={
 	ao:		NULL,
 	di:		das16_di_rbits,
 	do_:		das16_do_wbits,
-	i8255_offset:	0,
+	i8255_offset:	0x400,
 	i8254_offset:	0x0c,
 	size:		0x408,
 	id:	0x20,
@@ -469,13 +524,13 @@ static struct das16_board_struct das16_boards[]={
 	ao:		NULL,
 	di:		das16_di_rbits,
 	do_:		das16_do_wbits,
-	i8255_offset:	0,
+	i8255_offset:	0x400,
 	i8254_offset:	0x0c,
 	size:		0x408,
 	id:	0x20,
 	},
 	{
-	name:		"das-1401",	// 4922.pdf (keithley user's manual)
+	name:		"das-1401",	// 4919.pdf and 4922.pdf (keithley user's manual)
 	ai:		das16_ai_rinsn,
 	ai_nbits:	12,
 	ai_speed:	10000,
@@ -483,13 +538,13 @@ static struct das16_board_struct das16_boards[]={
 	ao:		NULL,
 	di:		das16_di_rbits,
 	do_:		das16_do_wbits,
-	i8255_offset:	0,
+	i8255_offset:	0x0,
 	i8254_offset:	0x0c,
 	size:		0x408,
-	id:	0xc0
+	id:	0xc0	// 4919.pdf says id bits are 0xe0, 4922.pdf says 0xc0
 	},
 	{
-	name:		"das-1402",	// 4922.pdf (keithley user's manual)
+	name:		"das-1402",	// 4919.pdf and 4922.pdf (keithley user's manual)
 	ai:		das16_ai_rinsn,
 	ai_nbits:	12,
 	ai_speed:	10000,
@@ -497,13 +552,13 @@ static struct das16_board_struct das16_boards[]={
 	ao:		NULL,
 	di:		das16_di_rbits,
 	do_:		das16_do_wbits,
-	i8255_offset:	0,
+	i8255_offset:	0x0,
 	i8254_offset:	0x0c,
 	size:		0x408,
-	id:	0xc0
+	id:	0xc0	// 4919.pdf says id bits are 0xe0, 4922.pdf says 0xc0
 	},
 	{
-	name:		"das-1601",
+	name:		"das-1601",	// 4919.pdf
 	ai:		das16_ai_rinsn,
 	ai_nbits:	12,
 	ai_speed:	10000,
@@ -512,13 +567,13 @@ static struct das16_board_struct das16_boards[]={
 	ao_nbits:	12,
 	di:		das16_di_rbits,
 	do_:		das16_do_wbits,
-	i8255_offset:	0,
+	i8255_offset:	0x400,
 	i8254_offset:	0x0c,
 	size:		0x408,
 	id:	0xc0
 	},
 	{
-	name:		"das-1602",
+	name:		"das-1602",	// 4919.pdf
 	ai:		das16_ai_rinsn,
 	ai_nbits:	12,
 	ai_speed:	10000,
@@ -527,7 +582,7 @@ static struct das16_board_struct das16_boards[]={
 	ao_nbits:	12,
 	di:		das16_di_rbits,
 	do_:		das16_do_wbits,
-	i8255_offset:	0,
+	i8255_offset:	0x400,
 	i8254_offset:	0x0c,
 	size:		0x408,
 	id:	0xc0
@@ -631,7 +686,7 @@ static struct das16_board_struct das16_boards[]={
 	i8255_offset:	0,
 	i8254_offset:	0x0c,
 	size:		0x14,
-	id:	0x00
+	id:	0xf0
 	},
 #if 0
 	{
@@ -643,7 +698,6 @@ static struct das16_board_struct das16_boards[]={
 	{
 	name:		"cio-das16/m1/16",	// cio-das16_m1_16.pdf, this board is a bit quirky, no dma
 	},
-	{
 #endif
 };
 #define n_das16_boards ((sizeof(das16_boards))/(sizeof(das16_board)))
@@ -660,25 +714,32 @@ static comedi_driver driver_das16={
 	offset:		sizeof(das16_boards[0]),
 };
 
-
 #define DAS16_TIMEOUT 1000
 
+static const int timer_period = HZ / 20 + 1;	// period for timer interrupt in jiffies (about 1/20 of a second)
 
-struct das16_private_struct {
+struct das16_private_struct
+{
 	unsigned int	ai_unipolar;	// unipolar flag
 	unsigned int	ai_singleended;	// single ended flag
 	unsigned int	clockbase;	// master clock speed in ns
-	unsigned int	control_state;	// dma, interrupt and trigger control bits
-	unsigned int	adc_count;	// number of samples remaining
-	unsigned int	do_bits;	// digital output bits
+	volatile unsigned int	control_state;	// dma, interrupt and trigger control bits
+	volatile unsigned long	adc_byte_count;	// number of samples remaining
 	unsigned int divisor1;	// divisor dividing master clock to get conversion frequency
 	unsigned int divisor2;	// divisor dividing master clock to get conversion frequency
 	unsigned int dma_chan;	// dma channel
-	sampl_t *dma_buffer;
-	unsigned int dma_transfer_size;	// number of bytes transfered per dma shot
+	uint16_t *dma_buffer[2];
+	dma_addr_t dma_buffer_addr[2];
+	unsigned int current_buffer;
+	volatile unsigned int dma_transfer_size;	// target number of bytes to transfer per dma shot
 	// user-defined analog input and output ranges defined from config options
 	comedi_lrange *user_ai_range_table;
 	comedi_lrange *user_ao_range_table;
+	struct munge_info ai_munge_info;
+
+	struct timer_list timer;                // for timed interrupt
+	volatile unsigned int timer_running : 1;
+	volatile unsigned int timer_mode : 1;   // true if using timer mode
 };
 #define devpriv ((struct das16_private_struct *)(dev->private))
 #define thisboard ((struct das16_board_struct *)(dev->board_ptr))
@@ -841,18 +902,18 @@ static int das16_cmd_exec(comedi_device *dev,comedi_subdevice *s)
 	unsigned long flags;
 	int range;
 
-	if(dev->irq == 0 || devpriv->dma_chan == 0)
+	if(devpriv->dma_chan == 0 || (dev->irq == 0 && devpriv->timer_mode == 0))
 	{
-		comedi_error(dev, "irq and dma required to execute comedi_cmd");
+		comedi_error(dev, "irq (or use of 'timer mode') dma required to execute comedi_cmd");
 		return -1;
 	}
 	if(cmd->flags & TRIG_RT)
 	{
-		comedi_error(dev, "dma transfers cannot be performed with TRIG_RT, aborting");
+		comedi_error(dev, "isa dma transfers cannot be performed with TRIG_RT, aborting");
 		return -1;
 	}
 
-	devpriv->adc_count = cmd->stop_arg * cmd->chanlist_len;
+	devpriv->adc_byte_count = cmd->stop_arg * cmd->chanlist_len * sizeof( uint16_t );
 
 	// disable conversions for das1600 mode
 	if(thisboard->size > 0x400)
@@ -875,7 +936,7 @@ static int das16_cmd_exec(comedi_device *dev,comedi_subdevice *s)
 
 	/* set counter mode and counts */
 	cmd->convert_arg = das16_set_pacer(dev, cmd->convert_arg, cmd->flags & TRIG_ROUND_MASK);
-	rt_printk("pacer period: %d ns\n", cmd->convert_arg);
+	DEBUG_PRINT("pacer period: %d ns\n", cmd->convert_arg);
 
 	/* enable counters */
 	byte = 0;
@@ -901,22 +962,32 @@ static int das16_cmd_exec(comedi_device *dev,comedi_subdevice *s)
 	/* clear flip-flop to make sure 2-byte registers for
 	 * count and address get set correctly */
 	clear_dma_ff(devpriv->dma_chan);
-	set_dma_addr(devpriv->dma_chan, virt_to_bus(devpriv->dma_buffer));
+	init_munge_info( &devpriv->ai_munge_info );
+	devpriv->current_buffer = 0;
+	set_dma_addr( devpriv->dma_chan,
+		devpriv->dma_buffer_addr[ devpriv->current_buffer ] );
 	// set appropriate size of transfer
-	devpriv->dma_transfer_size = das16_suggest_transfer_size(*cmd);
-	if(cmd->stop_src == TRIG_COUNT &&
-		devpriv->adc_count * sample_size < devpriv->dma_transfer_size)
-	{
-		devpriv->dma_transfer_size = devpriv->adc_count * sample_size;
-	}
+	devpriv->dma_transfer_size = das16_suggest_transfer_size(dev, *cmd);
 	set_dma_count(devpriv->dma_chan, devpriv->dma_transfer_size);
 	enable_dma(devpriv->dma_chan);
 	release_dma_lock(flags);
 
-	/* clear interrupt bit */
-	outb(0x00, dev->iobase + DAS16_STATUS);
-	/* enable interrupts, dma and pacer clocked conversions */
-	devpriv->control_state |= DAS16_INTE | DMA_ENABLE;
+	// set up interrupt
+	if( devpriv->timer_mode )
+	{
+		devpriv->timer_running = 1;
+		devpriv->timer.expires = jiffies + timer_period;
+		add_timer(&devpriv->timer);
+		devpriv->control_state &= ~DAS16_INTE;
+	}else
+	{
+		/* clear interrupt bit */
+		outb(0x00, dev->iobase + DAS16_STATUS);
+		/* enable interrupts */
+		devpriv->control_state |= DAS16_INTE;
+	}
+	devpriv->control_state |= DMA_ENABLE;
+	devpriv->control_state &= ~PACING_MASK;
 	if(cmd->convert_src == TRIG_EXT)
 		devpriv->control_state |= EXT_PACER;
 	else
@@ -934,27 +1005,39 @@ static int das16_cmd_exec(comedi_device *dev,comedi_subdevice *s)
 
 static int das16_cancel(comedi_device *dev, comedi_subdevice *s)
 {
+	unsigned long flags;
+
+	comedi_spin_lock_irqsave( &dev->spinlock, flags );
 	/* disable interrupts, dma and pacer clocked conversions */
-	devpriv->control_state &= ~DAS16_INTE & ~INT_PACER & ~DMA_ENABLE;
+	devpriv->control_state &= ~DAS16_INTE & ~PACING_MASK & ~DMA_ENABLE;
 	outb(devpriv->control_state, dev->iobase + DAS16_CONTROL);
 	if(devpriv->dma_chan)
 		disable_dma(devpriv->dma_chan);
 
+	// disable SW timer
+	if( devpriv->timer_mode && devpriv->timer_running )
+	{
+		devpriv->timer_running = 0;
+		del_timer(&devpriv->timer);
+	}
+
 	/* disable burst mode */
 	if(thisboard->size > 0x400)
 	{
 		outb(0, dev->iobase + DAS1600_BURST);
 	}
 
+	comedi_spin_unlock_irqrestore( &dev->spinlock, flags );
+
 	return 0;
 }
 
 static void das16_reset(comedi_device *dev)
 {
-	outb(0,dev->iobase+DAS16_STATUS);
-	outb(0,dev->iobase+DAS16_CONTROL);
-	outb(0,dev->iobase+DAS16_PACER);
-	outb(0,dev->iobase+DAS16_CNTR_CONTROL);
+	outb(0, dev->iobase + DAS16_STATUS);
+	outb(0, dev->iobase + DAS16_CONTROL);
+	outb(0, dev->iobase + DAS16_PACER);
+	outb(0, dev->iobase + DAS16_CNTR_CONTROL);
 }
 
 static int das16_ai_rinsn(comedi_device *dev,comedi_subdevice *s,comedi_insn *insn,lsampl_t *data)
@@ -964,33 +1047,41 @@ static int das16_ai_rinsn(comedi_device *dev,comedi_subdevice *s,comedi_insn *in
 	int chan;
 	int msb,lsb;
 
+	// disable interrupts and pacing
+	devpriv->control_state &= ~DAS16_INTE & ~DMA_ENABLE & ~PACING_MASK;
+	outb(devpriv->control_state, dev->iobase + DAS16_CONTROL);
+
 	/* set multiplexer */
 	chan = CR_CHAN(insn->chanspec);
 	chan |= CR_CHAN(insn->chanspec) << 4;
-	outb(chan,dev->iobase+DAS16_MUX);
+	outb(chan, dev->iobase + DAS16_MUX);
 
 	/* set gain */
-	if(thisboard->ai_pg != das16_pg_none){
+	if(thisboard->ai_pg != das16_pg_none)
+	{
 		range = CR_RANGE(insn->chanspec);
 		outb((das16_gainlists[thisboard->ai_pg])[range],
 			dev->iobase+DAS16_GAIN);
 	}
 
-	for(n=0;n<insn->n;n++){
+	for(n = 0; n < insn->n; n++)
+	{
 		/* trigger conversion */
-		outb_p(0,dev->iobase+DAS16_TRIG);
+		outb_p(0, dev->iobase + DAS16_TRIG);
 
-		for(i=0;i<DAS16_TIMEOUT;i++){
+		for(i = 0; i < DAS16_TIMEOUT; i++)
+		{
 			if(!(inb(dev->iobase + DAS16_STATUS) & BUSY))
 				break;
 		}
-		if(i==DAS16_TIMEOUT){
+		if(i == DAS16_TIMEOUT)
+		{
 			rt_printk("das16: timeout\n");
 			return -ETIME;
 		}
 		msb = inb(dev->iobase + DAS16_AI_MSB);
 		lsb = inb(dev->iobase + DAS16_AI_LSB);
-		if(thisboard->ai_nbits==12){
+		if(thisboard->ai_nbits == 12){
 			data[n] = ((lsb >> 4) & 0xf) | (msb << 4);
 		}else{
 			data[n] = lsb | (msb << 8);
@@ -1004,7 +1095,7 @@ static int das16_di_rbits(comedi_device *dev,comedi_subdevice *s,comedi_insn *in
 {
 	lsampl_t bits;
 
-	bits = inb(dev->iobase+DAS16_DIO) & 0xf;
+	bits = inb(dev->iobase + DAS16_DIO) & 0xf;
 	data[1] = bits;
 	data[0] = 0;
 
@@ -1017,15 +1108,15 @@ static int das16_do_wbits(comedi_device *dev,comedi_subdevice *s,comedi_insn *in
 
 	// only set bits that have been masked
 	data[0] &= 0xf;
-	wbits = devpriv->do_bits;
+	wbits = s->state;
 	// zero bits that have been masked
 	wbits &= ~data[0];
 	// set masked bits
 	wbits |= data[0] & data[1];
-	devpriv->do_bits = wbits;
+	s->state = wbits;
 	data[1] = wbits;
 
-	outb(devpriv->do_bits, dev->iobase + DAS16_DIO);
+	outb(s->state, dev->iobase + DAS16_DIO);
 
 	return 2;
 }
@@ -1036,122 +1127,141 @@ static int das16_ao_winsn(comedi_device *dev,comedi_subdevice *s,comedi_insn *in
 	int lsb,msb;
 	int chan;
 
-	chan=CR_CHAN(insn->chanspec);
+	chan = CR_CHAN(insn->chanspec);
 
-	for(i=0;i<insn->n;i++){
-		if(thisboard->ao_nbits==12){
-			lsb=(data[i]<<4)&0xff;
-			msb=(data[i]>>4)&0xff;
-		}else{
-			lsb=data[i]&0xff;
-			msb=(data[i]>>8)&0xff;
+	for(i = 0; i < insn->n; i++)
+	{
+		if(thisboard->ao_nbits == 12)
+		{
+			lsb = (data[i] << 4) & 0xff;
+			msb = (data[i] >> 4) & 0xff;
+		}else
+		{
+			lsb = data[i] & 0xff;
+			msb = (data[i] >> 8) & 0xff;
 		}
-
-#if 0
-		outb(lsb,dev->iobase+devpriv->ao_offset_lsb[chan]);
-		outb(msb,dev->iobase+devpriv->ao_offset_msb[chan]);
-#else
 		outb(lsb,dev->iobase+DAS16_AO_LSB(chan));
 		outb(msb,dev->iobase+DAS16_AO_MSB(chan));
-#endif
 	}
 
 	return i;
 }
 
 
-static void das16_interrupt(int irq, void *d, struct pt_regs *regs)
+static irqreturn_t das16_dma_interrupt(int irq, void *d, struct pt_regs *regs)
 {
-	int i;
 	int status;
-	unsigned long flags;
 	comedi_device *dev = d;
+
+	status = inb(dev->iobase + DAS16_STATUS);
+
+	if((status & DAS16_INT ) == 0)
+	{
+		DEBUG_PRINT( "spurious interrupt\n" );
+		return IRQ_NONE;
+	}
+
+	/* clear interrupt */
+	outb(0x00, dev->iobase + DAS16_STATUS);
+
+	das16_interrupt(dev);
+	return IRQ_HANDLED;
+}
+
+static void das16_timer_interrupt(unsigned long arg)
+{
+	comedi_device *dev = (comedi_device*) arg;
+
+	das16_interrupt(dev);
+
+	if(devpriv->timer_running)
+		mod_timer(&devpriv->timer, jiffies + timer_period);
+}
+
+static void das16_interrupt( comedi_device *dev )
+{
+	unsigned long dma_flags, spin_flags;
 	comedi_subdevice *s = dev->read_subdev;
 	comedi_async *async;
-	unsigned int max_points, num_points, residue, leftover;
-	sampl_t dpnt;
+	comedi_cmd *cmd;
+	int num_bytes, residue;
+	int buffer_index;
 
 	if(dev->attached == 0)
 	{
 		comedi_error(dev, "premature interrupt");
 		return;
 	}
-	// initialize async here to make sure s is not NULL
+	// initialize async here to make sure it is not NULL
 	async = s->async;
-	async->events = 0;
+	cmd = &async->cmd;
 
-	status = inb(dev->iobase + DAS16_STATUS);
-
-	if((status & DAS16_INT ) == 0)
+	if( devpriv->dma_chan == 0 )
 	{
-		comedi_error(dev, "spurious interrupt");
+		comedi_error(dev, "interrupt with no dma channel?");
 		return;
 	}
 
-	flags = claim_dma_lock();
+	comedi_spin_lock_irqsave( &dev->spinlock, spin_flags );
+	if( ( devpriv->control_state & DMA_ENABLE ) == 0 )
+	{
+		comedi_spin_unlock_irqrestore( &dev->spinlock, spin_flags );
+		DEBUG_PRINT( "interrupt while dma disabled?\n" );
+		return;
+	}
+
+	dma_flags = claim_dma_lock();
 	disable_dma(devpriv->dma_chan);
 	/* clear flip-flop to make sure 2-byte registers for
 	 * count and address get set correctly */
 	clear_dma_ff(devpriv->dma_chan);
 
 	// figure out how many points to read
-	max_points = devpriv->dma_transfer_size  / sample_size;
-	/* residue is the number of points left to be done on the dma
-	 * transfer.  It should always be zero at this point unless
-	 * the stop_src is set to external triggering.
+
+	/* residue is the number of bytes left to be done on the dma
+	 * transfer.
 	 */
-	residue = get_dma_residue(devpriv->dma_chan) / sample_size;
-	num_points = max_points - residue;
-	if(devpriv->adc_count < num_points &&
-		async->cmd.stop_src == TRIG_COUNT)
-		num_points = devpriv->adc_count;
-
-	// figure out how many points will be stored next time
-	leftover = 0;
-	if(async->cmd.stop_src != TRIG_COUNT)
+	residue = get_dma_residue(devpriv->dma_chan);
+	if(residue > devpriv->dma_transfer_size)
 	{
-		leftover = devpriv->dma_transfer_size / sample_size;
-	}else if(devpriv->adc_count > num_points)
+		comedi_error(dev, "residue > transfer size!\n");
+		async->events |= COMEDI_CB_ERROR | COMEDI_CB_EOA;
+		num_bytes = 0;
+	}else
+		num_bytes = devpriv->dma_transfer_size - residue;
+
+	if(cmd->stop_src == TRIG_COUNT &&
+		num_bytes > devpriv->adc_byte_count)
 	{
-		leftover = devpriv->adc_count - num_points;
-		if(leftover > max_points)
-			leftover = max_points;
-	}
-	/* there should only be a residue if collection was stopped by having
-	 * the stop_src set to an external trigger, in which case there
-	 * will be no more data
-	 */
-	if(residue)
-		leftover = 0;
-
-	for(i = 0; i < num_points; i++)
-	{
-		/* write data point to comedi buffer */
-		dpnt = devpriv->dma_buffer[i];
-		if(thisboard->ai_nbits == 12)
-			dpnt = (dpnt >> 4) & 0xfff;
-		comedi_buf_put(async, dpnt);
-		if(devpriv->adc_count > 0) devpriv->adc_count--;
-	}
-	// re-enable  dma
-	set_dma_addr(devpriv->dma_chan, virt_to_bus(devpriv->dma_buffer));
-	set_dma_count(devpriv->dma_chan, leftover * sample_size);
-	enable_dma(devpriv->dma_chan);
-	release_dma_lock(flags);
-
-	async->events |= COMEDI_CB_BLOCK;
-
-	if(async->cmd.stop_src == TRIG_COUNT &&
-		devpriv->adc_count == 0)
-	{	/* end of acquisition */
-		das16_cancel(dev, s);
+		num_bytes = devpriv->adc_byte_count;
 		async->events |= COMEDI_CB_EOA;
 	}
 
-	comedi_event(dev, s, async->events);
+	buffer_index = devpriv->current_buffer;
+	devpriv->current_buffer = ( devpriv->current_buffer + 1 ) % 2;
+	devpriv->adc_byte_count -= num_bytes;
 
-	/* clear interrupt */
-	outb(0x00, dev->iobase + DAS16_STATUS);
+	// figure out how many bytes for next transfer
+	if(cmd->stop_src == TRIG_COUNT && devpriv->timer_mode == 0 &&
+		devpriv->dma_transfer_size > devpriv->adc_byte_count )
+		devpriv->dma_transfer_size = devpriv->adc_byte_count;
+
+	// re-enable  dma
+	if(( async->events & COMEDI_CB_EOA ) == 0)
+	{
+		set_dma_addr( devpriv->dma_chan,
+			devpriv->dma_buffer_addr[ devpriv->current_buffer ] );
+		set_dma_count( devpriv->dma_chan, devpriv->dma_transfer_size );
+		enable_dma(devpriv->dma_chan);
+	}
+	release_dma_lock(dma_flags);
+
+	comedi_spin_unlock_irqrestore( &dev->spinlock, spin_flags );
+
+	das16_write_array_to_buffer( dev,
+		devpriv->dma_buffer[ buffer_index ], num_bytes );
+
+	cfc_handle_events( dev, s );
 }
 
 static unsigned int das16_set_pacer(comedi_device *dev, unsigned int ns, int rounding_flags)
@@ -1166,23 +1276,21 @@ static unsigned int das16_set_pacer(comedi_device *dev, unsigned int ns, int rou
 	return ns;
 }
 
-#ifdef DEBUG
 static void reg_dump(comedi_device *dev)
 {
-	rt_printk("********DAS1600 REGISTER DUMP********\n");
-	rt_printk("DAS16_MUX: %x\n", inb(dev->iobase+DAS16_MUX) );
-	rt_printk("DAS16_DIO: %x\n", inb(dev->iobase+DAS16_DIO) );
-	rt_printk("DAS16_STATUS: %x\n", inb(dev->iobase+DAS16_STATUS) );
-	rt_printk("DAS16_CONTROL: %x\n", inb(dev->iobase+DAS16_CONTROL) );
-	rt_printk("DAS16_PACER: %x\n", inb(dev->iobase+DAS16_PACER) );
-	rt_printk("DAS16_GAIN: %x\n", inb(dev->iobase+DAS16_GAIN) );
-	rt_printk("DAS16_CNTR_CONTROL: %x\n", inb(dev->iobase+DAS16_CNTR_CONTROL) );
-	rt_printk("DAS1600_CONV: %x\n", inb(dev->iobase+DAS1600_CONV) );
-	rt_printk("DAS1600_BURST: %x\n", inb(dev->iobase+DAS1600_BURST) );
-	rt_printk("DAS1600_ENABLE: %x\n", inb(dev->iobase+DAS1600_ENABLE) );
-	rt_printk("DAS1600_STATUS_B: %x\n", inb(dev->iobase+DAS1600_STATUS_B) );
+	DEBUG_PRINT("********DAS1600 REGISTER DUMP********\n");
+	DEBUG_PRINT("DAS16_MUX: %x\n", inb(dev->iobase+DAS16_MUX) );
+	DEBUG_PRINT("DAS16_DIO: %x\n", inb(dev->iobase+DAS16_DIO) );
+	DEBUG_PRINT("DAS16_STATUS: %x\n", inb(dev->iobase+DAS16_STATUS) );
+	DEBUG_PRINT("DAS16_CONTROL: %x\n", inb(dev->iobase+DAS16_CONTROL) );
+	DEBUG_PRINT("DAS16_PACER: %x\n", inb(dev->iobase+DAS16_PACER) );
+	DEBUG_PRINT("DAS16_GAIN: %x\n", inb(dev->iobase+DAS16_GAIN) );
+	DEBUG_PRINT("DAS16_CNTR_CONTROL: %x\n", inb(dev->iobase+DAS16_CNTR_CONTROL) );
+	DEBUG_PRINT("DAS1600_CONV: %x\n", inb(dev->iobase+DAS1600_CONV) );
+	DEBUG_PRINT("DAS1600_BURST: %x\n", inb(dev->iobase+DAS1600_BURST) );
+	DEBUG_PRINT("DAS1600_ENABLE: %x\n", inb(dev->iobase+DAS1600_ENABLE) );
+	DEBUG_PRINT("DAS1600_STATUS_B: %x\n", inb(dev->iobase+DAS1600_STATUS_B) );
 }
-#endif
 
 static int das16_probe(comedi_device *dev, comedi_devconfig *it)
 {
@@ -1212,9 +1320,9 @@ static int das16_probe(comedi_device *dev, comedi_devconfig *it)
 	printk(" id bits are 0x%02x\n",diobits);
 	if(thisboard->id != diobits)
 	{
-		printk(" requested board's id bits are 0x%x\n", thisboard->id);
-		return -1;
+		printk(" requested board's id bits are 0x%x (ignore)\n", thisboard->id);
 	}
+
 	return 0;
 }
 
@@ -1232,6 +1340,8 @@ static int das1600_mode_detect(comedi_device *dev)
 		printk(" 1MHz pacer clock\n");
 	}
 
+	reg_dump(dev);
+
 	return 0;
 }
 
@@ -1251,11 +1361,15 @@ static int das16_attach(comedi_device *dev, comedi_devconfig *it)
 	int ret, irq;
 	int iobase;
 	int dma_chan;
+	int timer_mode;
 	unsigned long flags;
 	comedi_krange *user_ai_range, *user_ao_range;
 
 	iobase = it->options[0];
-
+	irq = it->options[1];
+	timer_mode = it->options[8];
+	if( timer_mode ) irq = 0;
+	
 	printk("comedi%d: das16:",dev->minor);
 
 	// check that clock setting is valid
@@ -1326,12 +1440,11 @@ static int das16_attach(comedi_device *dev, comedi_devconfig *it)
 	}
 
 	/* now for the irq */
-	irq=it->options[1];
-	if(irq > 1 && irq < 8)
+	if( irq > 1 && irq < 8 )
 	{
-		if((ret=comedi_request_irq(irq,das16_interrupt,0,"das16",dev))<0)
+		if((ret=comedi_request_irq(irq, das16_dma_interrupt, 0, "das16",dev)) < 0)
 			return ret;
-		dev->irq=irq;
+		dev->irq = irq;
 		printk(" ( irq = %d )",irq);
 	}else if(irq == 0){
 		printk(" ( no irq )");
@@ -1345,10 +1458,15 @@ static int das16_attach(comedi_device *dev, comedi_devconfig *it)
 	dma_chan = it->options[2];
 	if(dma_chan == 1 || dma_chan == 3)
 	{
-		// allocate dma buffer
-		devpriv->dma_buffer = kmalloc(DAS16_DMA_SIZE, GFP_KERNEL | GFP_DMA);
-		if(devpriv->dma_buffer == NULL)
-			return -ENOMEM;
+		// allocate dma buffers
+		int i;
+		for( i = 0; i < 2; i++)
+		{
+			devpriv->dma_buffer[i] = pci_alloc_consistent( NULL,
+				DAS16_DMA_SIZE, &devpriv->dma_buffer_addr[i] );
+			if( devpriv->dma_buffer[i] == NULL )
+				return -ENOMEM;
+		}
 		if(request_dma(dma_chan, "das16"))
 		{
 			printk(" failed to allocate dma channel %i\n", dma_chan);
@@ -1396,8 +1514,15 @@ static int das16_attach(comedi_device *dev, comedi_devconfig *it)
 		user_ao_range->flags = UNIT_volt;
 	}
 
-	dev->n_subdevices = 5;
-	if((ret=alloc_subdevices(dev))<0)
+	if(timer_mode)
+	{
+		init_timer(&(devpriv->timer));
+		devpriv->timer.function = das16_timer_interrupt;
+		devpriv->timer.data = (unsigned long) dev;
+		devpriv->timer_mode = timer_mode ? 1 : 0;
+	}
+
+	if((ret=alloc_subdevices(dev, 5))<0)
 		return ret;
 
 	s=dev->subdevices+0;
@@ -1436,7 +1561,7 @@ static int das16_attach(comedi_device *dev, comedi_devconfig *it)
 	/* ao */
 	if(thisboard->ao){
 		s->type = COMEDI_SUBD_AO;
-		s->subdev_flags = SDF_WRITEABLE;
+		s->subdev_flags = SDF_WRITABLE;
 		s->n_chan = 2;
 		s->maxdata = (1 << thisboard->ao_nbits) - 1;
 		if(devpriv->user_ao_range_table)
@@ -1468,11 +1593,13 @@ static int das16_attach(comedi_device *dev, comedi_devconfig *it)
 	/* do */
 	if(thisboard->do_){
 		s->type = COMEDI_SUBD_DO;
-		s->subdev_flags = SDF_WRITEABLE | SDF_READABLE;
+		s->subdev_flags = SDF_WRITABLE | SDF_READABLE;
 		s->n_chan = 4;
 		s->maxdata = 1;
 		s->range_table = &range_digital;
 		s->insn_bits = thisboard->do_;
+		// initialize digital output lines
+		outb(s->state, dev->iobase + DAS16_DIO);
 	}else{
 		s->type = COMEDI_SUBD_UNUSED;
 	}
@@ -1487,9 +1614,7 @@ static int das16_attach(comedi_device *dev, comedi_devconfig *it)
 	}
 
 	das16_reset(dev);
-	// initialize digital output lines
-	outb(devpriv->do_bits, dev->iobase + DAS16_DIO);
-	/* set the interrupt level,enable pacer clock */
+	/* set the interrupt level */
 	devpriv->control_state = DAS16_IRQ(dev->irq);
 	outb(devpriv->control_state, dev->iobase + DAS16_CONTROL);
 
@@ -1514,20 +1639,15 @@ static int das16_detach(comedi_device *dev)
 	if(dev->subdevices)
 		subdev_8255_cleanup(dev,dev->subdevices+4);
 
-	if(dev->irq)
-		comedi_free_irq(dev->irq, dev);
-
-	if(thisboard->size<0x400){
-		release_region(dev->iobase,thisboard->size);
-	}else{
-		release_region(dev->iobase,0x10);
-		release_region(dev->iobase+0x400,thisboard->size&0x3ff);
-	}
-
 	if(devpriv)
 	{
-		if(devpriv->dma_buffer)
-			kfree(devpriv->dma_buffer);
+		int i;
+		for( i = 0; i < 2; i++ )
+		{
+			if( devpriv->dma_buffer[i] )
+				pci_free_consistent( NULL, DAS16_DMA_SIZE, devpriv->dma_buffer[i],
+					devpriv->dma_buffer_addr[i] );
+		}
 		if(devpriv->dma_chan)
 			free_dma(devpriv->dma_chan);
 		if(devpriv->user_ai_range_table)
@@ -1536,17 +1656,37 @@ static int das16_detach(comedi_device *dev)
 			kfree(devpriv->user_ao_range_table);
 	}
 
+	if(dev->irq)
+		comedi_free_irq(dev->irq, dev);
+
+	if( dev->iobase )
+	{
+		if(thisboard->size < 0x400){
+			release_region(dev->iobase, thisboard->size);
+		}else{
+			release_region(dev->iobase, 0x10);
+			release_region(dev->iobase + 0x400, thisboard->size&0x3ff);
+		}
+	}
+
 	return 0;
 }
 
 COMEDI_INITCLEANUP(driver_das16);
 
 // utility function that suggests a dma transfer size in bytes
-static unsigned int das16_suggest_transfer_size(comedi_cmd cmd)
+static unsigned int das16_suggest_transfer_size(comedi_device *dev, comedi_cmd cmd)
 {
 	unsigned int size;
 	unsigned int freq;
 
+	/* if we are using timer interrupt, we don't care how long it
+	 * will take to complete transfer since it will be interrupted
+	 * by timer interrupt */
+	if(devpriv->timer_mode) return DAS16_DMA_SIZE;
+
+	/* otherwise, we are relying on dma terminal count interrupt,
+	 * so pick a reasonable size */
 	if(cmd.convert_src == TRIG_TIMER)
 		freq = 1000000000 / cmd.convert_arg;
 	else if(cmd.scan_begin_src == TRIG_TIMER)
@@ -1570,6 +1710,50 @@ static unsigned int das16_suggest_transfer_size(comedi_cmd cmd)
 	else if(size < sample_size)
 		size = sample_size;
 
+	if( cmd.stop_src == TRIG_COUNT && size > devpriv->adc_byte_count )
+		size = devpriv->adc_byte_count;
+	
 	return size;
 }
 
+static void init_munge_info( struct munge_info *info )
+{
+	info->have_byte = 0;
+}
+
+/* we want to be able to write one byte at a time to buffer to deal with
+ * possibility that 8-bit dma transfer will be interrupted inbetween
+ * least significant and most significant byte of a sample */
+static void write_byte_to_buffer( comedi_device *dev, comedi_subdevice *subd, uint8_t raw_byte )
+{
+	sampl_t data;
+	struct munge_info *info = &devpriv->ai_munge_info;
+
+	if( info->have_byte == 0 )
+	{
+		info->byte = raw_byte;
+		info->have_byte = 1;
+	}else
+	{
+		info->have_byte = 0;
+		if( thisboard->ai_nbits == 12 )
+		{
+			data = ( raw_byte << 4 ) & 0xff0;
+			data |= ( info->byte >> 4 ) & 0xf;
+		}else
+		{
+			data = ( raw_byte << 8 ) & 0xff00;
+			data |= info->byte & 0xff;
+		}
+		cfc_write_to_buffer( subd, data );
+	}
+}
+
+static void das16_write_array_to_buffer( comedi_device *dev, void *data, unsigned int num_bytes )
+{
+	unsigned int i;
+	uint8_t *array = data;
+
+	for( i = 0; i < num_bytes; i++ )
+		write_byte_to_buffer( dev, dev->read_subdev, array[i] );
+};
