@@ -6,7 +6,7 @@
     Copyright (C) 2001 Frank Mori Hess <fmhess@users.sourceforge.net>
 
     COMEDI - Linux Control and Measurement Device Interface
-    Copyright (C) 2000 David A. Schleef <ds@stm.lbl.gov>
+    Copyright (C) 2000 David A. Schleef <ds@schleef.org>
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -27,7 +27,7 @@
 /*
 Driver: das16m1.o
 Description: CIO-DAS16/M1
-Authors: Frank Mori Hess <fmhess@uiuc.edu>
+Author: Frank Mori Hess <fmhess@users.sourceforge.net>
 Devices: [MeasurementComputing] CIO-DAS16/M1 (cio-das16/m1)
 Status: works
 
@@ -58,16 +58,12 @@ Options:
 irq can be omitted, although the cmd interface will not work without it.
 */
 
-#include <linux/kernel.h>
-#include <linux/module.h>
 #include <linux/comedidev.h>
-#include <linux/errno.h>
 #include <linux/ioport.h>
-#include <asm/io.h>
-#include <linux/slab.h>
-#include <linux/delay.h>
+
 #include "8255.h"
 #include "8253.h"
+#include "comedi_fc.h"
 
 #define DAS16M1_SIZE 16
 #define DAS16M1_SIZE2 8
@@ -98,7 +94,6 @@ irq can be omitted, although the cmd interface will not work without it.
 
 #define DAS16M1_AI             0	// 16-bit wide register
 #define   AI_CHAN(x)             ((x) & 0xf)
-#define   AI_DATA(x)             (((x) >> 4) & 0xfff)
 #define DAS16M1_CS             2
 #define   EXT_TRIG_BIT           0x1
 #define   OVRUN                  0x20
@@ -146,7 +141,7 @@ static int das16m1_cmd_exec(comedi_device *dev,comedi_subdevice *s);
 static int das16m1_cancel(comedi_device *dev, comedi_subdevice *s);
 
 static int das16m1_poll(comedi_device *dev, comedi_subdevice *s);
-static void das16m1_interrupt(int irq, void *d, struct pt_regs *regs);
+static irqreturn_t das16m1_interrupt(int irq, void *d, struct pt_regs *regs);
 static void das16m1_handler(comedi_device *dev, unsigned int status);
 
 static unsigned int das16m1_set_pacer(comedi_device *dev, unsigned int ns, int round_flag);
@@ -186,6 +181,7 @@ struct das16m1_private_struct {
 	 * needed to keep track of whether new count has been loaded into
 	 * counter yet (loaded by first sample conversion) */
 	u16 initial_hw_count;
+	sampl_t ai_buffer[ FIFO_SIZE ];
 	unsigned int do_bits;	// saves status of digital output bits
 	unsigned int divisor1;	// divides master clock to obtain conversion speed
 	unsigned int divisor2;	// divides master clock to obtain conversion speed
@@ -195,6 +191,11 @@ struct das16m1_private_struct {
 
 COMEDI_INITCLEANUP(driver_das16m1);
 
+static inline sampl_t munge_sample( sampl_t data )
+{
+	return ( data >> 4 ) & 0xfff;
+}
+
 static int das16m1_cmd_test(comedi_device *dev,comedi_subdevice *s, comedi_cmd *cmd)
 {
 	unsigned int err=0, tmp, i;
@@ -413,7 +414,7 @@ static int das16m1_ai_rinsn(comedi_device *dev,comedi_subdevice *s,comedi_insn *
 			comedi_error(dev, "timeout");
 			return -ETIME;
 		}
-		data[n] = AI_DATA(inw(dev->iobase));
+		data[n] = munge_sample( inw( dev->iobase ) );
 	}
 
 	return n;
@@ -460,10 +461,10 @@ static int das16m1_poll(comedi_device *dev, comedi_subdevice *s)
 	das16m1_handler(dev, status);
 	comedi_spin_unlock_irqrestore(&dev->spinlock, flags);
 
-	return s->async->buf_int_count - s->async->buf_user_count;
+	return s->async->buf_write_count - s->async->buf_read_count;
 }
 
-static void das16m1_interrupt(int irq, void *d, struct pt_regs *regs)
+static irqreturn_t das16m1_interrupt(int irq, void *d, struct pt_regs *regs)
 {
 	int status;
 	comedi_device *dev = d;
@@ -471,7 +472,7 @@ static void das16m1_interrupt(int irq, void *d, struct pt_regs *regs)
 	if(dev->attached == 0)
 	{
 		comedi_error(dev, "premature interrupt");
-		return;
+		return IRQ_HANDLED;
 	}
 
 	// prevent race with comedi_poll()
@@ -483,7 +484,7 @@ static void das16m1_interrupt(int irq, void *d, struct pt_regs *regs)
 	{
 		comedi_error(dev, "spurious interrupt");
 		spin_unlock(&dev->spinlock);
-		return;
+		return IRQ_NONE;
 	}
 
 	das16m1_handler(dev, status);
@@ -492,12 +493,21 @@ static void das16m1_interrupt(int irq, void *d, struct pt_regs *regs)
 	outb(0, dev->iobase + DAS16M1_CLEAR_INTR);
 
 	spin_unlock(&dev->spinlock);
+	return IRQ_HANDLED;
+}
+
+static void munge_sample_array( sampl_t *array, unsigned int num_elements )
+{
+	unsigned int i;
+
+	for(i = 0; i < num_elements; i++)
+	{
+		array[i] = munge_sample( array[i] );
+	}
 }
 
 static void das16m1_handler(comedi_device *dev, unsigned int status)
 {
-	int i;
-	sampl_t data[FIFO_SIZE];
 	comedi_subdevice *s;
 	comedi_async *async;
 	comedi_cmd *cmd;
@@ -535,12 +545,11 @@ static void das16m1_handler(comedi_device *dev, unsigned int status)
 	// make sure we dont try to get too many points if fifo has overrun
 	if(num_samples > FIFO_SIZE)
 		num_samples = FIFO_SIZE;
-	insw(dev->iobase, data, num_samples);
-	for(i = 0; i < num_samples; i++)
-	{
-		comedi_buf_put(async, AI_DATA(data[i]));
-		devpriv->adc_count++;
-	}
+	insw( dev->iobase, devpriv->ai_buffer, num_samples );
+	munge_sample_array( devpriv->ai_buffer, num_samples );
+	cfc_write_array_to_buffer( s, devpriv->ai_buffer, num_samples * sizeof( sampl_t ) );
+	devpriv->adc_count += num_samples;
+
 	if(cmd->stop_src == TRIG_COUNT)
 	{
 		if(devpriv->adc_count >= cmd->stop_arg * cmd->chanlist_len)
@@ -559,7 +568,6 @@ static void das16m1_handler(comedi_device *dev, unsigned int status)
 		comedi_error(dev, "fifo overflow");
 	}
 
-	async->events |= COMEDI_CB_BLOCK;
 	comedi_event(dev, s, async->events);
 
 }
@@ -674,8 +682,7 @@ static int das16m1_attach(comedi_device *dev, comedi_devconfig *it)
 		return -EINVAL;
 	}
 
-	dev->n_subdevices = 4;
-	if((ret = alloc_subdevices(dev)) < 0)
+	if((ret = alloc_subdevices(dev, 4)) < 0)
 		return ret;
 
 	s = dev->subdevices + 0;
@@ -706,7 +713,7 @@ static int das16m1_attach(comedi_device *dev, comedi_devconfig *it)
 	s = dev->subdevices + 2;
 	/* do */
 	s->type = COMEDI_SUBD_DO;
-	s->subdev_flags = SDF_WRITEABLE | SDF_READABLE;
+	s->subdev_flags = SDF_WRITABLE | SDF_READABLE;
 	s->n_chan = 4;
 	s->maxdata = 1;
 	s->range_table = &range_digital;
