@@ -3,7 +3,7 @@
     hardware driver for Data Translation DT2814
 
     COMEDI - Linux Control and Measurement Device Interface
-    Copyright (C) 1998 David A. Schleef <ds@stm.lbl.gov>
+    Copyright (C) 1998 David A. Schleef <ds@schleef.org>
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -39,21 +39,10 @@ a power of 10, from 1 to 10^7, of which only 3 or 4 are useful.  In
 addition, the clock does not seem to be very accurate.
 */
 
-
-#include <linux/kernel.h>
-#include <linux/module.h>
-#include <linux/sched.h>
-#include <linux/mm.h>
-#include <linux/slab.h>
-#include <linux/errno.h>
-#include <linux/ioport.h>
-#include <linux/delay.h>
-#include <linux/interrupt.h>
-#include <linux/timex.h>
-#include <linux/timer.h>
-#include <asm/io.h>
 #include <linux/comedidev.h>
 
+#include <linux/ioport.h>
+#include <linux/delay.h>
 
 
 #define DT2814_SIZE 2
@@ -81,7 +70,7 @@ static comedi_driver driver_dt2814={
 };
 COMEDI_INITCLEANUP(driver_dt2814);
 
-static void dt2814_interrupt(int irq,void *dev,struct pt_regs * regs);
+static irqreturn_t dt2814_interrupt(int irq,void *dev,struct pt_regs * regs);
 
 typedef struct{
 	int ntrig;
@@ -107,7 +96,7 @@ static int dt2814_ai_insn_read(comedi_device *dev,comedi_subdevice *s,
 		for(i=0;i<DT2814_TIMEOUT;i++){
 			status = inb(dev->iobase+DT2814_CSR);
 printk("dt2814: status: %02x\n",status);
-udelay(10);
+comedi_udelay(10);
 			if(status&DT2814_FINISH)
 				break;
 		}
@@ -119,7 +108,7 @@ udelay(10);
 		hi=inb(dev->iobase+DT2814_DATA);
 		lo=inb(dev->iobase+DT2814_DATA);
 
-		data[i]=(hi<<4)|(lo>>4);
+		data[n]=(hi<<4)|(lo>>4);
 	}
 	
 	return n;
@@ -132,7 +121,7 @@ static int dt2814_ns_to_timer(unsigned int *ns,unsigned int flags)
 
 	/* XXX ignores flags */
 
-	f = 1e4; /* ns */
+	f = 10000; /* ns */
 	for(i=0;i<8;i++){
 		if((2*(*ns))<(f*11))break;
 		f *= 10;
@@ -261,7 +250,7 @@ static int dt2814_attach(comedi_device *dev,comedi_devconfig *it)
 	dev->board_name = "dt2814";
 
 	outb(0,dev->iobase+DT2814_CSR);
-	udelay(100);
+	comedi_udelay(100);
 	if(inb(dev->iobase+DT2814_CSR)&DT2814_ERR){
 		printk("reset error (fatal)\n");
 		return -EIO;
@@ -278,7 +267,7 @@ static int dt2814_attach(comedi_device *dev,comedi_devconfig *it)
 	
 		outb(0,dev->iobase+DT2814_CSR);
 
-		udelay(100);
+		comedi_udelay(100);
 
 		irq=probe_irq_off(irqs);
 		restore_flags(flags);
@@ -300,9 +289,8 @@ static int dt2814_attach(comedi_device *dev,comedi_devconfig *it)
 	}else{
 		printk("(probe returned multiple irqs--bad)\n");
 	}
-	
-	dev->n_subdevices=1;
-	if((ret=alloc_subdevices(dev))<0)
+
+	if((ret=alloc_subdevices(dev, 1))<0)
 		return ret;
 	if((ret=alloc_private(dev,sizeof(dt2814_private)))<0)
 		return ret;
@@ -310,7 +298,7 @@ static int dt2814_attach(comedi_device *dev,comedi_devconfig *it)
 	s=dev->subdevices+0;
 	dev->read_subdev = s;
 	s->type=COMEDI_SUBD_AI;
-	s->subdev_flags=SDF_READABLE;
+	s->subdev_flags=SDF_READABLE|SDF_GROUND;
 	s->n_chan=16;			/* XXX */
 	s->len_chanlist=1;
 	s->insn_read = dt2814_ai_insn_read;
@@ -338,37 +326,35 @@ static int dt2814_detach(comedi_device *dev)
 }
 
 
-static void dt2814_interrupt(int irq,void *d,struct pt_regs * regs)
+static irqreturn_t dt2814_interrupt(int irq,void *d,struct pt_regs * regs)
 {
-        int lo,hi;
-        comedi_device *dev=d;
+	int lo,hi;
+	comedi_device *dev=d;
 	comedi_subdevice *s = dev->subdevices + 0;
 	int data;
 
-        hi=inb(dev->iobase+DT2814_DATA);
-        lo=inb(dev->iobase+DT2814_DATA);
+	hi=inb(dev->iobase+DT2814_DATA);
+	lo=inb(dev->iobase+DT2814_DATA);
 
-        data=(hi<<4)|(lo>>4);
+	data=(hi<<4)|(lo>>4);
 
 	if(!(--devpriv->ntrig)){
-		int flags,i;
+		int i;
 
 		outb(0,dev->iobase+DT2814_CSR);
 		/* note: turning off timed mode triggers another
 			sample. */
 
-		save_flags(flags);
-		cli();	/* FIXME */
 		for(i=0;i<DT2814_TIMEOUT;i++){
 			if(inb(dev->iobase+DT2814_CSR)&DT2814_FINISH)
 				break;
 		}
 		inb(dev->iobase+DT2814_DATA);
 		inb(dev->iobase+DT2814_DATA);
-		restore_flags(flags);
-	
+
 		s->async->events |= COMEDI_CB_EOA;
 	}
 	comedi_event(dev,s,s->async->events);
+	return IRQ_HANDLED;
 }
 
