@@ -3,7 +3,7 @@
    hardware driver for Data Translation DT2821 series
 
    COMEDI - Linux Control and Measurement Device Interface
-   Copyright (C) 1997-8 David A. Schleef <ds@stm.lbl.gov>
+   Copyright (C) 1997-8 David A. Schleef <ds@schleef.org>
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -54,18 +54,9 @@ Notes:
 */
 
 #include <linux/comedidev.h>
-#include <linux/module.h>
-#include <linux/kernel.h>
-#include <linux/sched.h>
-#include <linux/mm.h>
-#include <linux/slab.h>
-#include <linux/errno.h>
+
 #include <linux/ioport.h>
-#include <linux/delay.h>
 #include <linux/interrupt.h>
-#include <linux/timex.h>
-#include <linux/timer.h>
-#include <asm/io.h>
 #include <asm/dma.h>
 
 #define DEBUG
@@ -335,7 +326,7 @@ typedef struct {
 		short *buf;	/* DMA buffer */
 		int size;	/* size of current transfer */
 	}dma[2];
-	int dma_maxsize;	/* max size of DMA transfer     */
+	int dma_maxsize;	/* max size of DMA transfer (in bytes) */
 	int usedma;		/* driver uses DMA              */
 	int current_dma_chan;
 	int dma_dir;
@@ -363,7 +354,7 @@ typedef struct {
 		int _i;					\
 		for(_i=0;_i<DT2821_TIMEOUT;_i++){	\
 			if(a){_i=0;break;}		\
-			udelay(5);			\
+			comedi_udelay(5);			\
 		}					\
 		if(_i){b}				\
 	}while(0)
@@ -391,71 +382,38 @@ static int dt282x_ns_to_timer(int *nanosec,int round_mode);
 
 static int dt282x_grab_dma(comedi_device *dev,int dma1,int dma2);
 
-static void dt282x_cleanup_buffer(comedi_device *dev,unsigned short *buf,unsigned int len)
+static void dt282x_copy_to_buffer(comedi_device *dev,sampl_t *buf,
+	unsigned int nbytes)
 {
+	comedi_async *async = dev->subdevices[0].async;
 	unsigned int i;
 	unsigned short mask=(1<<boardtype.adbits)-1;
 	unsigned short sign=1<<(boardtype.adbits-1);
+	unsigned short *abuf;
+	int n;
 
 	if(devpriv->ad_2scomp){
-		for(i=0;i<len;i++){
-			buf[i]&=mask;
-			buf[i]^=sign;
-		}
+		sign = 1<<(boardtype.adbits-1);
 	}else{
-		for(i=0;i<len;i++){
-			buf[i]&=mask;
+		sign = 0;
+	}
+
+	abuf = async->prealloc_buf + async->buf_write_ptr;
+	if(async->buf_write_ptr + nbytes >= async->prealloc_bufsz){
+		n = (async->prealloc_bufsz - async->buf_write_ptr - nbytes)/2;
+		for(i=0;i<n;i++){
+			abuf[i] = (buf[i]&mask)^sign;
 		}
+		nbytes -= n;
+		abuf = async->prealloc_buf;
+		buf = buf + i;
+	}
+	n = nbytes/2;
+	for(i=0;i<n;i++){
+		abuf[i] = (buf[i]&mask)^sign;
 	}
 }
 
-static void copy_to_buf(comedi_device *dev,comedi_subdevice *s,void *buf,unsigned int n_bytes)
-{
-	unsigned int n;
-
-	n=n_bytes;
-	if(s->async->buf_int_ptr+n >= s->async->data_len){
-		n=s->async->data_len-s->async->buf_int_ptr;
-		memcpy(s->async->data+s->async->buf_int_ptr,buf,n);
-		buf+=n;
-		s->async->buf_int_count+=n;
-		s->async->buf_int_ptr=0;
-
-		n=n_bytes-n;
-	}
-	memcpy(s->async->data+s->async->buf_int_ptr,buf,n);
-	buf+=n;
-	s->async->buf_int_count+=n;
-	s->async->buf_int_ptr+=n;
-}
-
-static int copy_from_buf(comedi_device *dev,comedi_subdevice *s,void *buf,unsigned int n_bytes)
-{
-	unsigned int n,m;
-
-	n=s->async->buf_int_count-s->async->buf_user_count;
-	if(n==0)return 0;
-	if(n>n_bytes)
-		n=n_bytes;
-
-	n_bytes=n;
-	if(s->async->buf_int_ptr+n >= s->async->data_len){
-		m=s->async->data_len-s->async->buf_int_ptr;
-		memcpy(buf,s->async->data+s->async->buf_int_ptr,m);
-		buf+=m;
-		s->async->buf_int_count+=m;
-		s->async->buf_int_ptr=0;
-
-		n-=m;
-	}
-	memcpy(buf,s->async->data+s->async->buf_int_ptr,n);
-	s->async->buf_int_count+=n;
-	s->async->buf_int_ptr+=n;
-
-	return n_bytes;
-}
-
-
 static void dt282x_ao_dma_interrupt(comedi_device * dev)
 {
 	void *ptr;
@@ -465,7 +423,7 @@ static void dt282x_ao_dma_interrupt(comedi_device * dev)
 
 	update_supcsr(DT2821_CLRDMADNE);
 
-	if(!s->async->data){
+	if(!s->async->prealloc_buf){
 		printk("async->data disappeared.  dang!\n");
 		return;
 	}
@@ -477,19 +435,20 @@ static void dt282x_ao_dma_interrupt(comedi_device * dev)
 
 	devpriv->current_dma_chan=1-i;
 
-	size=copy_from_buf(dev,s,ptr,devpriv->dma_maxsize*2);
-	if(!size){
-		printk("dt282x: AO underrun\n");
+	size = comedi_buf_read_n_available(s->async);
+	if(size>devpriv->dma_maxsize)size=devpriv->dma_maxsize;
+	if( size == 0){
+		rt_printk("dt282x: AO underrun\n");
 		dt282x_ao_cancel(dev,s);
-		s->async->events |= COMEDI_CB_EOA | COMEDI_CB_ERROR;
+		s->async->events |= COMEDI_CB_OVERFLOW;
 		comedi_event(dev,s,s->async->events);
 		return;
 	}
-	prep_ao_dma(dev,i,size/2);
-
+	comedi_buf_memcpy_from(s->async, 0, ptr, size);
+	comedi_buf_read_free(s->async, size);
+	prep_ao_dma(dev,i,size);
 	enable_dma(devpriv->dma[i].chan);
 
-	s->async->events |= COMEDI_CB_BLOCK;
 	comedi_event(dev,s,s->async->events);
 	return;
 }
@@ -499,25 +458,35 @@ static void dt282x_ai_dma_interrupt(comedi_device * dev)
 	void *ptr;
 	int size;
 	int i;
+	int ret;
 	comedi_subdevice *s=dev->subdevices;
 
 	update_supcsr(DT2821_CLRDMADNE);
 
-	if(!s->async->data){
+	if(!s->async->prealloc_buf){
 		printk("async->data disappeared.  dang!\n");
 		return;
 	}
 
-	i=devpriv->current_dma_chan;
-	ptr=devpriv->dma[i].buf;
-	size=devpriv->dma[i].size;
+	i = devpriv->current_dma_chan;
+	ptr = devpriv->dma[i].buf;
+	size = devpriv->dma[i].size;
 
 	disable_dma(devpriv->dma[i].chan);
 
-	devpriv->current_dma_chan=1-i;
-	dt282x_cleanup_buffer(dev,ptr,size);
-	copy_to_buf(dev,s,ptr,size*2);
-	devpriv->nread-=size;
+	devpriv->current_dma_chan = 1-i;
+
+	ret = comedi_buf_write_alloc(s->async, size);
+	if(!ret){
+		rt_printk("dt282x: AI buffer overflow\n");
+		s->async->events |= COMEDI_CB_OVERFLOW;
+		comedi_event(dev,s,s->async->events);
+		return;
+	}
+	dt282x_copy_to_buffer(dev, ptr, size);
+	comedi_buf_write_free(s->async, size);
+
+	devpriv->nread-=size/2;
 
 	if(devpriv->nread<0){
 		printk("dt282x: off by one\n");
@@ -535,8 +504,6 @@ static void dt282x_ai_dma_interrupt(comedi_device * dev)
 		comedi_event(dev,s,s->async->events);
 
 		return;
-	}else{
-		s->async->events |= COMEDI_CB_BLOCK;
 	}
 
 #if 1
@@ -566,9 +533,9 @@ static int prep_ai_dma(comedi_device * dev,int chan,int n)
 
 	if(n==0)
 		n = devpriv->dma_maxsize;
-	if (n >= devpriv->ntrig)
-		n = devpriv->ntrig;
-	devpriv->ntrig -= n;
+	if (n >= devpriv->ntrig*2)
+		n = devpriv->ntrig*2;
+	devpriv->ntrig -= n/2;
 
 	devpriv->dma[chan].size = n;
 	dma_chan = devpriv->dma[chan].chan;
@@ -577,7 +544,7 @@ static int prep_ai_dma(comedi_device * dev,int chan,int n)
 	set_dma_mode(dma_chan, DMA_MODE_READ);
 	flags=claim_dma_lock();
 	set_dma_addr(dma_chan, dma_ptr);
-	set_dma_count(dma_chan, n << 1);
+	set_dma_count(dma_chan, n);
 	release_dma_lock(flags);
 
 	return n;
@@ -596,26 +563,28 @@ static int prep_ao_dma(comedi_device * dev,int chan,int n)
 	set_dma_mode(dma_chan, DMA_MODE_WRITE);
 	flags=claim_dma_lock();
 	set_dma_addr(dma_chan, dma_ptr);
-	set_dma_count(dma_chan, n*2 );
+	set_dma_count(dma_chan, n );
 	release_dma_lock(flags);
 
 	return n;
 }
 
-static void dt282x_interrupt(int irq, void *d, struct pt_regs *regs)
+static irqreturn_t dt282x_interrupt(int irq, void *d, struct pt_regs *regs)
 {
 	comedi_device *dev = d;
 	comedi_subdevice *s = dev->subdevices+0;
+	comedi_subdevice *s_ao = dev->subdevices+1;
 	unsigned int supcsr, adcsr, dacsr;
 	sampl_t data;
+	int ret;
 
 	adcsr=inw(dev->iobase + DT2821_ADCSR);
 	if (adcsr & DT2821_ADERR) {
 		comedi_error(dev, "A/D error");
 		dt282x_ai_cancel(dev,s);
-		s->async->events |= COMEDI_CB_EOA | COMEDI_CB_ERROR;
-		comedi_event(dev,s,s->async->events);	
-		return;
+		s->async->events |= COMEDI_CB_ERROR;
+		comedi_event(dev,s,s->async->events);
+		return IRQ_HANDLED;
 	}
 	supcsr = inw(dev->iobase + DT2821_SUPCSR);
 	/*printk("supcsr=%02x\n",supcsr);*/
@@ -624,7 +593,7 @@ static void dt282x_interrupt(int irq, void *d, struct pt_regs *regs)
 			dt282x_ai_dma_interrupt(dev);
 		else
 			dt282x_ao_dma_interrupt(dev);
-		return;
+		return IRQ_HANDLED;
 	}
 	if ((dacsr = inw(dev->iobase + DT2821_DACSR)) & DT2821_DAERR) {
 #if 0
@@ -635,10 +604,10 @@ static void dt282x_interrupt(int irq, void *d, struct pt_regs *regs)
 		}
 #endif
 		comedi_error(dev, "D/A error");
-		dt282x_ao_cancel(dev,s);
-		s->async->events |= COMEDI_CB_EOA | COMEDI_CB_ERROR;
-		comedi_event(dev,s,s->async->events);	
-		return;
+		dt282x_ao_cancel(dev,s_ao);
+		s->async->events |= COMEDI_CB_ERROR;
+		comedi_event(dev,s_ao,s->async->events);
+		return IRQ_HANDLED;
 	}
 	if (adcsr & DT2821_ADDONE) {
 		data = (sampl_t) inw(dev->iobase + DT2821_ADDAT);
@@ -646,12 +615,9 @@ static void dt282x_interrupt(int irq, void *d, struct pt_regs *regs)
 		if(devpriv->ad_2scomp){
 			data^=1<<(boardtype.adbits-1);
 		}
-		*(sampl_t *)(s->async->data+s->async->buf_int_ptr)=data;
-		s->async->buf_int_ptr+=sizeof(sampl_t);
-		s->async->buf_int_count+=sizeof(sampl_t);
-		if(s->async->buf_int_ptr>=s->async->data_len){
-			s->async->buf_int_ptr = 0;
-			s->async->events |= COMEDI_CB_EOBUF;
+		ret = comedi_buf_put( s->async, data );
+		if(ret==0){
+			s->async->events |= COMEDI_CB_OVERFLOW;
 		}
 
 		devpriv->nread--;
@@ -662,9 +628,10 @@ static void dt282x_interrupt(int irq, void *d, struct pt_regs *regs)
 				update_supcsr(DT2821_STRIG);
 		}
 
-		comedi_event(dev,s,s->async->events);	
-		return;
+		comedi_event(dev,s,s->async->events);
+		return IRQ_HANDLED;
 	}
+	return IRQ_HANDLED;
 }
 
 
@@ -986,8 +953,7 @@ static int dt282x_ao_cmdtest(comedi_device *dev,comedi_subdevice *s,comedi_cmd *
 	if(!cmd->scan_end_src || tmp!=cmd->scan_end_src)err++;
 
 	tmp=cmd->stop_src;
-	//cmd->stop_src &= TRIG_COUNT|TRIG_NONE;
-	cmd->stop_src &= TRIG_COUNT;	/* XXX */
+	cmd->stop_src &= TRIG_NONE;
 	if(!cmd->stop_src || tmp!=cmd->stop_src)err++;
 
 	if(err)return 1;
@@ -1006,7 +972,7 @@ static int dt282x_ao_cmdtest(comedi_device *dev,comedi_subdevice *s,comedi_cmd *
 		cmd->start_arg=0;
 		err++;
 	}
-	if(cmd->scan_begin_arg <= 5000 /* XXX unknown */){
+	if(cmd->scan_begin_arg < 5000 /* XXX unknown */){
 		cmd->scan_begin_arg = 5000;
 		err++;
 	}
@@ -1050,14 +1016,28 @@ static int dt282x_ao_inttrig(comedi_device *dev,comedi_subdevice *s,
 
 	if(x!=0)return -EINVAL;
 
-	size=copy_from_buf(dev,s,devpriv->dma[0].buf,devpriv->dma_maxsize*2);
-	prep_ao_dma(dev,0,size/2);
+	size = comedi_buf_read_n_available(s->async);
+	if(size>devpriv->dma_maxsize)size=devpriv->dma_maxsize;
+	if( size == 0){
+		rt_printk("dt282x: AO underrun\n");
+		return -EPIPE;
+	}
+	comedi_buf_memcpy_from(s->async, 0, devpriv->dma[0].buf, size);
+	comedi_buf_read_free(s->async, size);
+	prep_ao_dma(dev,0,size);
 	enable_dma(devpriv->dma[0].chan);
 
-	size=copy_from_buf(dev,s,devpriv->dma[1].buf,devpriv->dma_maxsize*2);
-	prep_ao_dma(dev,1,size/2);
+	size = comedi_buf_read_n_available(s->async);
+	if(size>devpriv->dma_maxsize)size=devpriv->dma_maxsize;
+	if( size == 0){
+		rt_printk("dt282x: AO underrun\n");
+		return -EPIPE;
+	}
+	comedi_buf_memcpy_from(s->async, 0, devpriv->dma[1].buf, size);
+	comedi_buf_read_free(s->async, size);
+	prep_ao_dma(dev,1,size);
 	enable_dma(devpriv->dma[1].chan);
-	
+
 	update_supcsr(DT2821_STRIG);
 	s->async->inttrig=NULL;
 
@@ -1187,8 +1167,7 @@ enum{	opt_iobase=0, opt_irq, opt_dma1, opt_dma2,	/* i/o base, irq, dma channels
  */
 static int dt282x_attach(comedi_device * dev, comedi_devconfig * it)
 {
-	int i, irqs, irq;
-	long flags;
+	int i, irq;
 	int ret;
 	comedi_subdevice *s;
 	int iobase;
@@ -1234,14 +1213,18 @@ static int dt282x_attach(comedi_device * dev, comedi_devconfig * it)
 	/* should do board test */
 
 	irq = it->options[opt_irq];
+#if 0
 	if (irq < 0) {
+		unsigned long flags;
+		int irqs;
+		
 		save_flags(flags);
 		sti();
 		irqs = probe_irq_on();
 
 		/* trigger interrupt */
 
-		udelay(100);
+		comedi_udelay(100);
 
 		irq = probe_irq_off(irqs);
 		restore_flags(flags);
@@ -1249,7 +1232,7 @@ static int dt282x_attach(comedi_device * dev, comedi_devconfig * it)
 			printk(" error probing irq (bad)");
 		}
 	}
-	dev->irq = 0;
+#endif
 	if (irq > 0) {
 		printk(" ( irq = %d )", irq);
 		ret = comedi_request_irq(irq, dt282x_interrupt, 0, "dt282x", dev);
@@ -1271,8 +1254,7 @@ static int dt282x_attach(comedi_device * dev, comedi_devconfig * it)
 	if(ret<0)
 		return ret;
 
-	dev->n_subdevices = 3;
-	if((ret=alloc_subdevices(dev))<0)
+	if((ret=alloc_subdevices(dev, 3))<0)
 		return ret;
 
 	s=dev->subdevices+0;
@@ -1296,14 +1278,14 @@ static int dt282x_attach(comedi_device * dev, comedi_devconfig * it)
 		/* ao subsystem */
 		s->type=COMEDI_SUBD_AO;
 		dev->write_subdev=s;
-		s->subdev_flags=SDF_WRITEABLE;
+		s->subdev_flags=SDF_WRITABLE;
 		s->insn_read=dt282x_ao_insn_read;
 		s->insn_write=dt282x_ao_insn_write;
 		s->do_cmdtest=dt282x_ao_cmdtest;
 		s->do_cmd=dt282x_ao_cmd;
 		s->cancel=dt282x_ao_cancel;
 		s->maxdata=(1<<boardtype.dabits)-1;
-		s->len_chanlist=1;			/* XXX could do 2 */
+		s->len_chanlist=2;
 		s->range_table_list=devpriv->darangelist;
 		devpriv->darangelist[0]=
 			opt_ao_range_lkup(it->options[opt_ao0_range]);
@@ -1318,7 +1300,7 @@ static int dt282x_attach(comedi_device * dev, comedi_devconfig * it)
 	s++;
 	/* dio subsystem */
 	s->type=COMEDI_SUBD_DIO;
-	s->subdev_flags=SDF_READABLE|SDF_WRITEABLE;
+	s->subdev_flags=SDF_READABLE|SDF_WRITABLE;
 	s->n_chan=16;
 	s->insn_bits = dt282x_dio_insn_bits;
 	s->insn_config = dt282x_dio_insn_config;
@@ -1387,9 +1369,9 @@ static int dt282x_grab_dma(comedi_device *dev,int dma1,int dma2)
 		return -EBUSY;
 	devpriv->dma[1].chan=dma2;
 
-	devpriv->dma_maxsize = PAGE_SIZE >> 1;
-	devpriv->dma[0].buf = (void *) get_free_page(GFP_KERNEL | GFP_DMA);
-	devpriv->dma[1].buf = (void *) get_free_page(GFP_KERNEL | GFP_DMA);
+	devpriv->dma_maxsize = PAGE_SIZE;
+	devpriv->dma[0].buf = (void *) __get_free_page(GFP_KERNEL | GFP_DMA);
+	devpriv->dma[1].buf = (void *) __get_free_page(GFP_KERNEL | GFP_DMA);
 	if (!devpriv->dma[0].buf || !devpriv->dma[1].buf) {
 		printk(" can't get DMA memory");
 		return -ENOMEM;
