@@ -29,20 +29,10 @@ Devices: [Quanser Consulting] MultiQ-3 (multiq3)
 
 */
 
-#include <linux/kernel.h>
-#include <linux/module.h>
-#include <linux/sched.h>
-#include <linux/mm.h>
-#include <linux/slab.h>
-#include <linux/errno.h>
-#include <linux/ioport.h>
-#include <linux/delay.h>
-#include <linux/interrupt.h>
-#include <linux/timex.h>
-#include <linux/timer.h>
-#include <asm/io.h>
 #include <linux/comedidev.h>
 
+#include <linux/ioport.h>
+
 
 #define MULTIQ3_SIZE 16
 
@@ -260,8 +250,7 @@ static int multiq3_attach(comedi_device * dev, comedi_devconfig * it)
        printk("comedi%d: no irq\n", dev->minor);
       }
       dev->board_name = "multiq3";
-      dev->n_subdevices = 5;
-      result = alloc_subdevices(dev);
+      result = alloc_subdevices(dev, 5);
       if(result<0)return result;
 
       result = alloc_private(dev,sizeof(struct multiq3_private));
@@ -270,7 +259,7 @@ static int multiq3_attach(comedi_device * dev, comedi_devconfig * it)
       s = dev->subdevices + 0;
       /* ai subdevice */
       s->type = COMEDI_SUBD_AI;
-      s->subdev_flags = SDF_READABLE;
+      s->subdev_flags = SDF_READABLE|SDF_GROUND;
       s->n_chan = 8;
       s->insn_read = multiq3_ai_insn_read;
       s->maxdata = 0x1fff;
@@ -279,7 +268,7 @@ static int multiq3_attach(comedi_device * dev, comedi_devconfig * it)
       s = dev->subdevices + 1;
       /* ao subdevice */
       s->type = COMEDI_SUBD_AO;
-      s->subdev_flags = SDF_WRITEABLE;
+      s->subdev_flags = SDF_WRITABLE;
       s->n_chan = 8;
       s->insn_read = multiq3_ao_insn_read;
       s->insn_write = multiq3_ao_insn_write;
@@ -298,7 +287,7 @@ static int multiq3_attach(comedi_device * dev, comedi_devconfig * it)
       s = dev->subdevices + 3;
       /* do subdevice */
       s->type = COMEDI_SUBD_DO;
-      s->subdev_flags = SDF_WRITEABLE;
+      s->subdev_flags = SDF_WRITABLE;
       s->n_chan = 16;
       s->insn_bits = multiq3_do_insn_bits;
       s->maxdata = 1;
