@@ -3,7 +3,7 @@
     Hardware driver for NI PCI-MIO E series cards
 
     COMEDI - Linux Control and Measurement Device Interface
-    Copyright (C) 1997-8 David A. Schleef <ds@stm.lbl.gov>
+    Copyright (C) 1997-8 David A. Schleef <ds@schleef.org>
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -18,68 +18,74 @@
     You should have received a copy of the GNU General Public License
     along with this program; if not, write to the Free Software
     Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
-
 */
 /*
 Driver: ni_pcimio.o
 Description: National Instruments PCI-MIO-E series (all boards)
-Author: ds
-Status: mainly limited by Comedi infrastructure
+Author: ds, John Hallen, Frank Mori Hess, Rolf Mueller, Herbert Peremans,
+  Herman Bruyninckx
+Status: works
 Devices: [National Instruments] PCI-MIO-16XE-50 (ni_pcimio),
   PCI-MIO-16XE-10, PXI-6030E, PCI-MIO-16E-1, PCI-MIO-16E-4, PCI-6040E,
   PXI-6040E, PCI-6031E, PCI-6032E, PCI-6033E, PCI-6071E, PCI-6023E,
   PCI-6024E, PCI-6025E, PXI-6025E, PCI-6034E, PCI-6035E, PCI-6052E,
-  PCI-6110E, PCI-6111E, PCI-6711, PCI-6713, PXI-6071E, PXI-6070E,
-  PXI-6052E, PCI-6036E
+  PCI-6110, PCI-6111, PCI-6711, PCI-6713, PXI-6071E, PXI-6070E,
+  PXI-6052E, PCI-6036E, PCI-6731, PCI-6733
+Updated: Sat, 16 Mar 2002 17:34:48 -0800
 
 These boards are almost identical to the AT-MIO E series, except that
-they use the PCI bus instead of ISA (i.e., AT).  See the notes above for
-ni_atmio.o for additional information about these boards.
+they use the PCI bus instead of ISA (i.e., AT).  See the notes for
+the ni_atmio.o driver for additional information about these boards.
 
-Comedi knows the PCI ID codes for many of the boards in this series,
-but the NI documentation is incomplete in this matter.  If you have
-a PCI-MIO board that Comedi doesn't recognize, send me the PCI device
-ID, as can be found in /proc/pci or the output of lspci.  The vendor
-code for National Instruments is 0x1093.  I will include the ID in
-the next version.
+Autocalibration is supported on many of the devices, using the
+calibration utility in Comedilib.
+
+By default, the driver uses DMA to transfer analog input data to
+memory.  When DMA is enabled, not all triggering features are
+supported.
+
+Streaming analog output is not supported on PCI-671x and PCI-673x.
+
+PCI IDs are not known for PCI-6731 and PCI-6733.  Digital I/O may not
+work on 673x.
+
+Information (number of channels, bits, etc.) for some devices may be
+incorrect.  Please check this and submit a bug if there are problems
+for your device.
+
+Bugs:
+ - When DMA is enabled, COMEDI_EV_SCAN_END and COMEDI_EV_CONVERT do
+   not work correctly.
+ - There are reported problems with the 67xx boards.
 
-DMA is halfway completed, but not yet operational.
 */
 /*
 	The PCI-MIO E series driver was originally written by
 	Tomasz Motylewski <...>, and ported to comedi by ds.
 
 
-	References for specifications:
-	
-	   321747b.pdf  Register Level Programmer Manual (obsolete)
-	   321747c.pdf  Register Level Programmer Manual (new)
-	   DAQ-STC reference manual
+	References:
+
+	   341079b.pdf  PCI E Series Register-Level Programmer Manual
+	   340934b.pdf  DAQ-STC reference manual
+
+	   322080b.pdf  6711/6713/6715 User Manual
+
+	   320945c.pdf  PCI E Series User Manual
+	   322138a.pdf  PCI-6052E and DAQPad-6052E User Manual
 
-	Other possibly relevant info:
-	
-	   320517c.pdf  User manual (obsolete)
-	   320517f.pdf  User manual (new)
-	   320889a.pdf  delete
-	   320906c.pdf  maximum signal ratings
-	   321066a.pdf  about 16x
-	   321791a.pdf  discontinuation of at-mio-16e-10 rev. c
-	   321808a.pdf  about at-mio-16e-10 rev P
-	   321837a.pdf  discontinuation of at-mio-16de-10 rev d
-	   321838a.pdf  about at-mio-16de-10 rev N
-	
 	ISSUES:
 
 	need to deal with external reference for DAC, and other DAC
 	properties in board properties
-	
+
 	deal with at-mio-16de-10 revision D to N changes, etc.
-	
+
 	need to add other CALDAC type
-	
+
 	need to slow down DAC loading.  I don't trust NI's claim that
 	two writes to the PCI bus slows IO enough.  I would prefer to
-	use udelay().  Timing specs: (clock)
+	use comedi_udelay().  Timing specs: (clock)
 		AD8522		30ns
 		DAC8043		120ns
 		DAC8800		60ns
@@ -87,65 +93,50 @@ DMA is halfway completed, but not yet operational.
 
 */
 
-#include <linux/kernel.h>
-#include <linux/module.h>
-#include <linux/errno.h>
-#include <linux/ioport.h>
-#include <linux/delay.h>
-#include <linux/mm.h>
-#include <linux/interrupt.h>
-#include <linux/slab.h>
 #include <linux/comedidev.h>
-#include <linux/init.h>
 
-#include <asm/io.h>
+#include <linux/delay.h>
 
 #include "ni_stc.h"
 #include "mite.h"
 
-#define PCI_DEBUG
+//#define PCI_DEBUG
 
 #define PCIDMA
 
 #define PCIMIO 1
 #undef ATMIO
 
-static struct caldac_struct *type1[]={&caldac_dac8800,&caldac_dac8043,NULL};
-static struct caldac_struct *type2[]={&caldac_dac8800,&caldac_dac8043,&caldac_ad8522};
-static struct caldac_struct *type3[]={&caldac_mb88341,NULL,NULL};
-static struct caldac_struct *type4[]={&caldac_mb88341,&caldac_mb88341,&caldac_ad8522};
+#define MAX_N_CALDACS (16+16+2)
 
-#define MAX_N_CALDACS (12+12+2)
-
-#define NI_VENDOR_ID 0x1093
 /* The following two tables must be in the same order */
 static struct pci_device_id ni_pci_table[] __devinitdata = {
-	{ NI_VENDOR_ID, 0x0162, PCI_ANY_ID, PCI_ANY_ID, 0, 0, 0 },
-	{ NI_VENDOR_ID, 0x1170, PCI_ANY_ID, PCI_ANY_ID, 0, 0, 0 },
-	{ NI_VENDOR_ID, 0x11d0, PCI_ANY_ID, PCI_ANY_ID, 0, 0, 0 },
-	{ NI_VENDOR_ID, 0x1180, PCI_ANY_ID, PCI_ANY_ID, 0, 0, 0 },
-	{ NI_VENDOR_ID, 0x1190, PCI_ANY_ID, PCI_ANY_ID, 0, 0, 0 },
-	{ NI_VENDOR_ID, 0x11c0, PCI_ANY_ID, PCI_ANY_ID, 0, 0, 0 },
-	{ NI_VENDOR_ID, 0x1330, PCI_ANY_ID, PCI_ANY_ID, 0, 0, 0 },
-	{ NI_VENDOR_ID, 0x1270, PCI_ANY_ID, PCI_ANY_ID, 0, 0, 0 },
-	{ NI_VENDOR_ID, 0x1340, PCI_ANY_ID, PCI_ANY_ID, 0, 0, 0 },
-	{ NI_VENDOR_ID, 0x1350, PCI_ANY_ID, PCI_ANY_ID, 0, 0, 0 },
-	{ NI_VENDOR_ID, 0x2a60, PCI_ANY_ID, PCI_ANY_ID, 0, 0, 0 },
-	{ NI_VENDOR_ID, 0x2a70, PCI_ANY_ID, PCI_ANY_ID, 0, 0, 0 },
-	{ NI_VENDOR_ID, 0x2a80, PCI_ANY_ID, PCI_ANY_ID, 0, 0, 0 },
-	{ NI_VENDOR_ID, 0x2ab0, PCI_ANY_ID, PCI_ANY_ID, 0, 0, 0 },
-	{ NI_VENDOR_ID, 0x2ca0, PCI_ANY_ID, PCI_ANY_ID, 0, 0, 0 },
-	{ NI_VENDOR_ID, 0x2c80, PCI_ANY_ID, PCI_ANY_ID, 0, 0, 0 },
-	{ NI_VENDOR_ID, 0x18b0, PCI_ANY_ID, PCI_ANY_ID, 0, 0, 0 },
-	{ NI_VENDOR_ID, 0x14e0, PCI_ANY_ID, PCI_ANY_ID, 0, 0, 0 },
-	{ NI_VENDOR_ID, 0x14f0, PCI_ANY_ID, PCI_ANY_ID, 0, 0, 0 },
-	{ NI_VENDOR_ID, 0x1880, PCI_ANY_ID, PCI_ANY_ID, 0, 0, 0 },
-	{ NI_VENDOR_ID, 0x1870, PCI_ANY_ID, PCI_ANY_ID, 0, 0, 0 },
-	{ NI_VENDOR_ID, 0x15b0, PCI_ANY_ID, PCI_ANY_ID, 0, 0, 0 },
-	{ NI_VENDOR_ID, 0x11b0, PCI_ANY_ID, PCI_ANY_ID, 0, 0, 0 },
-	{ NI_VENDOR_ID, 0x18c0, PCI_ANY_ID, PCI_ANY_ID, 0, 0, 0 },
-	{ NI_VENDOR_ID, 0x1580, PCI_ANY_ID, PCI_ANY_ID, 0, 0, 0 },
-	{ NI_VENDOR_ID, 0x2890, PCI_ANY_ID, PCI_ANY_ID, 0, 0, 0 },
+	{ PCI_VENDOR_ID_NATINST, 0x0162, PCI_ANY_ID, PCI_ANY_ID, 0, 0, 0 },
+	{ PCI_VENDOR_ID_NATINST, 0x1170, PCI_ANY_ID, PCI_ANY_ID, 0, 0, 0 },
+	{ PCI_VENDOR_ID_NATINST, 0x11d0, PCI_ANY_ID, PCI_ANY_ID, 0, 0, 0 },
+	{ PCI_VENDOR_ID_NATINST, 0x1180, PCI_ANY_ID, PCI_ANY_ID, 0, 0, 0 },
+	{ PCI_VENDOR_ID_NATINST, 0x1190, PCI_ANY_ID, PCI_ANY_ID, 0, 0, 0 },
+	{ PCI_VENDOR_ID_NATINST, 0x11c0, PCI_ANY_ID, PCI_ANY_ID, 0, 0, 0 },
+	{ PCI_VENDOR_ID_NATINST, 0x1330, PCI_ANY_ID, PCI_ANY_ID, 0, 0, 0 },
+	{ PCI_VENDOR_ID_NATINST, 0x1270, PCI_ANY_ID, PCI_ANY_ID, 0, 0, 0 },
+	{ PCI_VENDOR_ID_NATINST, 0x1340, PCI_ANY_ID, PCI_ANY_ID, 0, 0, 0 },
+	{ PCI_VENDOR_ID_NATINST, 0x1350, PCI_ANY_ID, PCI_ANY_ID, 0, 0, 0 },
+	{ PCI_VENDOR_ID_NATINST, 0x2a60, PCI_ANY_ID, PCI_ANY_ID, 0, 0, 0 },
+	{ PCI_VENDOR_ID_NATINST, 0x2a70, PCI_ANY_ID, PCI_ANY_ID, 0, 0, 0 },
+	{ PCI_VENDOR_ID_NATINST, 0x2a80, PCI_ANY_ID, PCI_ANY_ID, 0, 0, 0 },
+	{ PCI_VENDOR_ID_NATINST, 0x2ab0, PCI_ANY_ID, PCI_ANY_ID, 0, 0, 0 },
+	{ PCI_VENDOR_ID_NATINST, 0x2ca0, PCI_ANY_ID, PCI_ANY_ID, 0, 0, 0 },
+	{ PCI_VENDOR_ID_NATINST, 0x2c80, PCI_ANY_ID, PCI_ANY_ID, 0, 0, 0 },
+	{ PCI_VENDOR_ID_NATINST, 0x18b0, PCI_ANY_ID, PCI_ANY_ID, 0, 0, 0 },
+	{ PCI_VENDOR_ID_NATINST, 0x14e0, PCI_ANY_ID, PCI_ANY_ID, 0, 0, 0 },
+	{ PCI_VENDOR_ID_NATINST, 0x14f0, PCI_ANY_ID, PCI_ANY_ID, 0, 0, 0 },
+	{ PCI_VENDOR_ID_NATINST, 0x1880, PCI_ANY_ID, PCI_ANY_ID, 0, 0, 0 },
+	{ PCI_VENDOR_ID_NATINST, 0x1870, PCI_ANY_ID, PCI_ANY_ID, 0, 0, 0 },
+	{ PCI_VENDOR_ID_NATINST, 0x15b0, PCI_ANY_ID, PCI_ANY_ID, 0, 0, 0 },
+	{ PCI_VENDOR_ID_NATINST, 0x11b0, PCI_ANY_ID, PCI_ANY_ID, 0, 0, 0 },
+	{ PCI_VENDOR_ID_NATINST, 0x18c0, PCI_ANY_ID, PCI_ANY_ID, 0, 0, 0 },
+	{ PCI_VENDOR_ID_NATINST, 0x1580, PCI_ANY_ID, PCI_ANY_ID, 0, 0, 0 },
+	{ PCI_VENDOR_ID_NATINST, 0x2890, PCI_ANY_ID, PCI_ANY_ID, 0, 0, 0 },
 	{ 0 }
 };
 MODULE_DEVICE_TABLE(pci, ni_pci_table);
@@ -163,7 +154,7 @@ static ni_board ni_boards[]={
 		aobits:         12,
 		ao_fifo_depth:  0,
 		ao_unipolar:    0,
-		caldac:         type1,
+		caldac:         {dac8800,dac8043},
 		has_8255:       0,
 	},
 	{       device_id:      0x1170,
@@ -178,7 +169,7 @@ static ni_board ni_boards[]={
 		aobits:         16,
 		ao_fifo_depth:  2048,
 		ao_unipolar:    1,
-		caldac:         type2,
+		caldac:         {dac8800,dac8043,ad8522},
 		has_8255:       0,
 	},
 	{       device_id:      0x11d0,
@@ -193,7 +184,7 @@ static ni_board ni_boards[]={
 		aobits:         16,
 		ao_fifo_depth:  2048,
 		ao_unipolar:    1,
-		caldac:         type2,
+		caldac:         {dac8800,dac8043,ad8522},
 		has_8255:       0,
 	},
 
@@ -209,7 +200,7 @@ static ni_board ni_boards[]={
 		aobits:         12,
 		ao_fifo_depth:  2048,
 		ao_unipolar:    1,
-		caldac:         type3,
+		caldac:         {mb88341},
 		has_8255:       0,
 	},
 	{       device_id:      0x1190,
@@ -219,12 +210,14 @@ static ni_board ni_boards[]={
 		ai_fifo_depth:  512,
 		alwaysdither:   0,
 		gainlkup:       ai_gain_16,
+		/* Note: there have been reported problems with full speed
+		 * on this board */
 		ai_speed:	2000,
 		n_aochan:       2,
 		aobits:         12,
 		ao_fifo_depth:  512,
 		ao_unipolar:    1,
-		caldac:         type3,
+		caldac:         {ad8804_debug}, // doc says mb88341
 		has_8255:       0,
 	},
 	{       device_id:      0x11c0,
@@ -239,7 +232,7 @@ static ni_board ni_boards[]={
 		aobits:         12,
 		ao_fifo_depth:  512,
 		ao_unipolar:    1,
-		caldac:         type3,
+		caldac:         {mb88341},
 		has_8255:       0,
 	},
 
@@ -255,7 +248,7 @@ static ni_board ni_boards[]={
 		aobits:         16,
 		ao_fifo_depth:  2048,
 		ao_unipolar:    1,
-		caldac:         type2,
+		caldac:         {dac8800,dac8043,ad8522},
 		has_8255:       0,
 	},
 	{       device_id:      0x1270,
@@ -270,7 +263,7 @@ static ni_board ni_boards[]={
 		aobits:         0,
 		ao_fifo_depth:  0,
 		ao_unipolar:    1,
-		caldac:         type2,
+		caldac:         {dac8800,dac8043,ad8522},
 		has_8255:       0,
 	},
 	{       device_id:      0x1340,
@@ -285,7 +278,7 @@ static ni_board ni_boards[]={
 		aobits:         0,
 		ao_fifo_depth:  0,
 		ao_unipolar:    1,
-		caldac:         type2,
+		caldac:         {dac8800,dac8043,ad8522},
 		has_8255:       0,
 	},
 	{       device_id:      0x1350,
@@ -300,7 +293,7 @@ static ni_board ni_boards[]={
 		aobits:         12,
 		ao_fifo_depth:  2048,
 		ao_unipolar:    1,
-		caldac:         type3,
+		caldac:         {ad8804_debug},
 		has_8255:       0,
 	},
 	{       device_id:      0x2a60,
@@ -314,7 +307,7 @@ static ni_board ni_boards[]={
 		n_aochan:       0,
 		aobits:         0,
 		ao_unipolar:    0,
-		caldac:         type3,
+		caldac:         {ad8804}, /* manual is wrong */
 		has_8255:	0,
 	},
 	{       device_id:      0x2a70,
@@ -328,8 +321,8 @@ static ni_board ni_boards[]={
 		n_aochan:       2,
 		aobits:         12,
 		ao_fifo_depth:  0,
-		ao_unipolar:    1,
-		caldac:         type3,
+		ao_unipolar:    0,
+		caldac:         {ad8804}, /* manual is wrong */
 		has_8255:	0,
 	},
 	{       device_id:      0x2a80,
@@ -343,8 +336,8 @@ static ni_board ni_boards[]={
 		n_aochan:       2,
 		aobits:         12,
 		ao_fifo_depth:  0,
-		ao_unipolar:    1,
-		caldac:         type3,
+		ao_unipolar:    0,
+		caldac:         {ad8804}, /* manual is wrong */
 		has_8255:	1,
 	},
 	{       device_id:      0x2ab0,
@@ -359,7 +352,7 @@ static ni_board ni_boards[]={
 		aobits:         12,
 		ao_fifo_depth:  0,
 		ao_unipolar:    1,
-		caldac:         type3,
+		caldac:         {ad8804}, /* manual is wrong */
 		has_8255:	1,
 	},
 
@@ -375,7 +368,7 @@ static ni_board ni_boards[]={
 		aobits:         0,
 		ao_fifo_depth:  0,
 		ao_unipolar:    0,
-		caldac:         type3,
+		caldac:         {ad8804_debug},
 		has_8255:	0,
 	},
 	{       device_id:      0x2c80,
@@ -390,7 +383,7 @@ static ni_board ni_boards[]={
 		aobits:         12,
 		ao_fifo_depth:  0,
 		ao_unipolar:    0,
-		caldac:         type3,
+		caldac:         {ad8804_debug},
 		has_8255:	0,
 	},
 	{       device_id:      0x18b0,
@@ -405,94 +398,175 @@ static ni_board ni_boards[]={
 		aobits:         16,
 		ao_unipolar:    1,
 		ao_fifo_depth:  2048,
-		caldac:         type4,
+		caldac:         {ad8804_debug,ad8804_debug,ad8522}, /* manual is wrong */
 	},
-	{       device_id:      0x14e0,		/* unknown */
-		name:           "pci-6110e",
+#if 0
+	{       device_id:      0x0000, /* unknown */
+		name:           "pci-6053e",
+		n_adchan:       64,
+		adbits:         16,
+		ai_fifo_depth:  512,
+		alwaysdither:   1,
+		gainlkup:       ai_gain_16,
+		ai_speed:	3000,
+		n_aochan:       2,
+		aobits:         16,
+		ao_unipolar:    1,
+		ao_fifo_depth:  2048,
+		caldac:         {ad8804,mb88341,ad8522}, /* manual is wrong */
+	},
+#endif
+	{       device_id:      0x14e0,
+		name:           "pci-6110",
 		n_adchan:       4, 
 		adbits:         12,
-		ai_fifo_depth:  512,
-		alwaysdither:   0,
-		gainlkup:       ai_gain_611x,
-		ai_speed:	200,
-		aobits:         16,
-		ao_unipolar:    0,
-		ao_fifo_depth:  2048,
-		caldac:         type4,		/* XXX */
-	},
-	{       device_id:      0x14f0,		/* unknown */
-		name:           "pci-6111e",
-		n_adchan:       2,
-		adbits:         12,
-		ai_fifo_depth:  512,
+		ai_fifo_depth:  8192,
 		alwaysdither:   0,
 		gainlkup:       ai_gain_611x,
 		ai_speed:	200,
 		n_aochan:       2,
-		aobits:         12,
+		aobits:         16,
+		ao_671x:	1,
 		ao_unipolar:    0,
 		ao_fifo_depth:  2048,
-		caldac:         type4,		/* XXX */
+		reg_611x:	1,
+		caldac:         {ad8804,ad8804},
 	},
+	{       device_id:      0x14f0,
+		name:           "pci-6111",
+		n_adchan:       2,
+		adbits:         12,
+		ai_fifo_depth:  8192,
+		alwaysdither:   0,
+		gainlkup:       ai_gain_611x,
+		ai_speed:	200,
+		n_aochan:       2,
+		aobits:         16,
+		ao_671x:	1,
+		ao_unipolar:    0,
+		ao_fifo_depth:  2048,
+		reg_611x:	1,
+		caldac:         {ad8804,ad8804},
+	},
+#if 0
+	/* The 6115 boards probably need their own driver */
+	{       device_id:      0x2ed0,
+		name:           "pci-6115",
+		n_adchan:       4,
+		adbits:         12,
+		ai_fifo_depth:  8192,
+		alwaysdither:   0,
+		gainlkup:       ai_gain_611x,
+		ai_speed:	100,
+		n_aochan:       2,
+		aobits:         16,
+		ao_671x:	1,
+		ao_unipolar:    0,
+		ao_fifo_depth:  2048,
+		reg_611x:	1,
+		caldac:         {ad8804_debug,ad8804_debug,ad8804_debug},/* XXX */
+	},
+#endif
+#if 0
+	{       device_id:      0x0000,
+		name:           "pxi-6115",
+		n_adchan:       4,
+		adbits:         12,
+		ai_fifo_depth:  8192,
+		alwaysdither:   0,
+		gainlkup:       ai_gain_611x,
+		ai_speed:	100,
+		n_aochan:       2,
+		aobits:         16,
+		ao_671x:	1,
+		ao_unipolar:    0,
+		ao_fifo_depth:  2048,
+		reg_611x:	1,
+		caldac:         {ad8804_debug,ad8804_debug,ad8804_debug},/* XXX */
+	},
+#endif
 	{       device_id:      0x1880,
 		name:           "pci-6711",
-		n_adchan:       0,
-		adbits:         0,
-		ai_fifo_depth:  0,
-		alwaysdither:   0,
-		gainlkup:       0,
-		ai_speed:	0,
+		n_adchan:       0, /* no analog input */
 		n_aochan:	4,
 		aobits:         12,
 		ao_unipolar:    0,
 		ao_fifo_depth:  8192,
-		caldac:         type4,		/* XXX */
+		ao_671x:	1,
+		caldac:         {mb88341,mb88341},/* XXX */
 	},
 	{       device_id:      0x1870,
 		name:           "pci-6713",
-		n_adchan:       0,
-		adbits:         0,
-		ai_fifo_depth:  0,
-		alwaysdither:   0,
-		gainlkup:       0,
-		ai_speed:	0,
+		n_adchan:       0, /* no analog input */
 		n_aochan:	8,
 		aobits:         12,
 		ao_unipolar:    0,
 		ao_fifo_depth:  16384,
-		caldac:         type4,		/* XXX */
+		ao_671x:	1,
+		caldac:         {mb88341,mb88341},/* XXX */
+	},
+	{       device_id:      0x2b80,
+		name:           "pxi-6713",
+		n_adchan:       0, /* no analog input */
+		n_aochan:	8,
+		aobits:         12,
+		ao_unipolar:    0,
+		ao_fifo_depth:  16384,
+		ao_671x:	1,
+		caldac:         {mb88341,mb88341},/* XXX */
+	},
+#if 0
+	{       device_id:      0x1880,
+		name:           "pci-6731",
+		n_adchan:       0, /* no analog input */
+		n_aochan:	4,
+		aobits:         16,
+		ao_unipolar:    0,
+		ao_fifo_depth:  8192,
+		ao_671x:	1,
+		caldac:         {mb88341,mb88341},/* XXX */
+	},
+	{       device_id:      0x1870,
+		name:           "pci-6733",
+		n_adchan:       0, /* no analog input */
+		n_aochan:	8,
+		aobits:         16,
+		ao_unipolar:    0,
+		ao_fifo_depth:  16384,
+		ao_671x:	1,
+		caldac:         {mb88341,mb88341},/* XXX */
+	},
+#endif
+	{	device_id:      0x15b0,
+		name:           "pxi-6071e",
+		n_adchan:       64,
+		adbits:         12,
+		ai_fifo_depth:  512,
+		alwaysdither:   1,
+		gainlkup:       ai_gain_16,
+		ai_speed:       800,
+		n_aochan:       2,
+		aobits:         12,
+		ao_fifo_depth:  2048,
+		ao_unipolar:    1,
+		caldac:         {ad8804_debug},
+		has_8255:       0,
+	},
+	{	device_id:      0x11b0,
+		name:           "pxi-6070e",
+		n_adchan:       16,
+		adbits:         12,
+		ai_fifo_depth:  512,
+		alwaysdither:   1,
+		gainlkup:       ai_gain_16,
+		ai_speed:       800,
+		n_aochan:       2,
+		aobits:         12,
+		ao_fifo_depth:  2048,
+		ao_unipolar:    1,
+		caldac:         {mb88341},
+		has_8255:       0,
 	},
-        {       device_id:      0x15b0,
-                name:           "pxi-6071e",
-                n_adchan:       64,
-                adbits:         12,
-                ai_fifo_depth:  512,
-                alwaysdither:   1,
-                gainlkup:       ai_gain_16,
-                ai_speed:       800,
-                n_aochan:       2,
-                aobits:         12,
-                ao_fifo_depth:  2048,
-                ao_unipolar:    1,
-                caldac:         type3,
-                has_8255:       0,
-        },
-
-        {       device_id:      0x11b0,
-                name:           "pxi-6070e",
-                n_adchan:       16,
-                adbits:         12,
-                ai_fifo_depth:  512,
-                alwaysdither:   1,
-                gainlkup:       ai_gain_16,
-                ai_speed:       800,
-                n_aochan:       2,
-                aobits:         12,
-                ao_fifo_depth:  2048,
-                ao_unipolar:    1,
-                caldac:         type3,
-                has_8255:       0,
-        },
  	{	device_id:      0x18c0,
 		name:           "pxi-6052e",
 		n_adchan:       16,
@@ -505,9 +579,9 @@ static ni_board ni_boards[]={
 		aobits:         16,
 		ao_unipolar:    1,
 		ao_fifo_depth:  2048,
-		caldac:         type4,
+		caldac:         {mb88341,mb88341,ad8522},
 	},
- 	{	device_id:      0x1580,
+	{	device_id:      0x1580,
 		name:           "pxi-6031e",
 		n_adchan:       64,
 		adbits:         16,
@@ -519,7 +593,7 @@ static ni_board ni_boards[]={
 		aobits:         16,
 		ao_fifo_depth:  2048,
 		ao_unipolar:    1,
-		caldac:         type2,
+		caldac:         {dac8800,dac8043,ad8522},
 	},
 	{       device_id:      0x2890,
 		name:           "pci-6036e",
@@ -533,7 +607,7 @@ static ni_board ni_boards[]={
 		aobits:         16,
 		ao_fifo_depth:  0,
 		ao_unipolar:    0,
-		caldac:         type3,
+		caldac:         {ad8804_debug},
 		has_8255:	0,
 	},
 };
@@ -552,12 +626,68 @@ COMEDI_INITCLEANUP(driver_pcimio);
 
 /* How we access registers */
 
+#define ni_writel(a,b)		(writel((a),dev->iobase+(b)))
+#define ni_readl(a)		(readl(dev->iobase+(a)))
 #define ni_writew(a,b)		(writew((a),dev->iobase+(b)))
 #define ni_readw(a)		(readw(dev->iobase+(a)))
 #define ni_writeb(a,b)		(writeb((a),dev->iobase+(b)))
 #define ni_readb(a)		(readb(dev->iobase+(a)))
-#define ni_writeb_p(a,b)	(ni_writeb(a,b),ni_writeb(a,b))
-#define ni_readb_p(a)		(ni_readb(a),ni_readb(a))
+
+/* How we access STC registers */
+
+/* We automatically take advantage of STC registers that can be
+ * read/written directly in the I/O space of the board.  Most
+ * PCIMIO devices map the low 8 STC registers to iobase+addr*2.
+ * The 611x devices map the write registers to iobase+addr*2, and
+ * the read registers to iobase+(addr-1)*2. */
+/* However, the 611x boards still aren't working, so I'm disabling
+ * non-windowed STC access temporarily */
+
+#define win_out(data,addr) __win_out(dev,data,addr)
+static inline void __win_out(comedi_device *dev, unsigned short data, int addr)
+{
+	unsigned long flags;
+
+	comedi_spin_lock_irqsave(&dev->spinlock,flags);
+	ni_writew(addr,Window_Address);
+	ni_writew(data,Window_Data);
+	comedi_spin_unlock_irqrestore(&dev->spinlock,flags);
+}
+
+#define win_out2(data,addr) do{ \
+	win_out((data)>>16, (addr)); \
+	win_out((data)&0xffff, (addr)+1); \
+}while(0)
+
+#define win_in(addr) __win_in(dev,addr)
+static inline unsigned short __win_in(comedi_device *dev, int addr)
+{
+	unsigned long flags;
+	int ret;
+
+	comedi_spin_lock_irqsave(&dev->spinlock,flags);
+	ni_writew(addr,Window_Address);
+	ret = ni_readw(Window_Data);
+	comedi_spin_unlock_irqrestore(&dev->spinlock,flags);
+
+	return ret;
+}
+
+#define ao_win_out(data,addr) __ao_win_out(dev,data,addr)
+static inline void __ao_win_out( comedi_device *dev, uint16_t data, int addr )
+{
+       unsigned long flags;
+
+       comedi_spin_lock_irqsave(&dev->spinlock,flags);
+       ni_writew(addr,AO_Window_Address_671x);
+       ni_writew(data,AO_Window_Data_671x);
+       comedi_spin_unlock_irqrestore(&dev->spinlock,flags);
+}
+
+#define ao_win_out2(data,addr) do{ \
+       ao_win_out((data)>>16, (addr)); \
+       ao_win_out((data)&0xffff, (addr)+1); \
+}while(0)
 
 
 #define interrupt_pin(a)	0
@@ -570,6 +700,8 @@ typedef struct{
 	struct mite_struct *mite;
 
 	NI_PRIVATE_COMMON
+
+	unsigned int last_buf_write_count;
 }ni_private;
 #define devpriv ((ni_private *)dev->private)
 
@@ -578,6 +710,10 @@ typedef struct{
 
 
 static int pcimio_find_device(comedi_device *dev,int bus,int slot);
+static int pcimio_ai_change(comedi_device *dev, comedi_subdevice *s,
+	unsigned long new_size);
+static int pcimio_ao_change(comedi_device *dev, comedi_subdevice *s,
+	unsigned long new_size);
 
 
 /* cleans up allocated resources */
@@ -598,9 +734,9 @@ static int pcimio_detach(comedi_device *dev)
 static int pcimio_attach(comedi_device *dev,comedi_devconfig *it)
 {
 	int		ret;
-	
+
 	printk("comedi%d: ni_pcimio:",dev->minor);
-	
+
 	ret=alloc_private(dev,sizeof(ni_private));
 	if(ret<0)return ret;
 
@@ -609,21 +745,34 @@ static int pcimio_attach(comedi_device *dev,comedi_devconfig *it)
 
 	printk(" %s",boardtype.name);
 	dev->board_name=boardtype.name;
-	
-	dev->iobase=mite_setup(devpriv->mite);
+
+	ret = mite_setup(devpriv->mite);
+	if(ret < 0)
+	{
+		printk(" error setting up mite\n");
+		return ret;
+	}
+	dev->iobase = mite_iobase(devpriv->mite);
 
 	dev->irq=mite_irq(devpriv->mite);
 
-        if(dev->irq==0){
+	if(dev->irq==0){
 		printk(" unknown irq (bad)\n");
 	}else{
-        	printk(" ( irq = %d )",dev->irq);
-        	if( (ret=comedi_request_irq(dev->irq,ni_E_interrupt,NI_E_IRQ_FLAGS,"ni_pcimio",dev))<0 ){
-                	printk(" irq not available\n");
+		printk(" ( irq = %d )",dev->irq);
+		if( (ret=comedi_request_irq(dev->irq,ni_E_interrupt,NI_E_IRQ_FLAGS,"ni_pcimio",dev))<0 ){
+			printk(" irq not available\n");
 			dev->irq=0;
-        	}
+		}
 	}
-	return ni_E_init(dev,it);
+
+	ret = ni_E_init(dev,it);
+	if(ret<0)return ret;
+
+	dev->subdevices[0].buf_change = pcimio_ai_change;
+	dev->subdevices[1].buf_change = pcimio_ao_change;
+
+	return ret;
 }
 
 
@@ -635,15 +784,9 @@ static int pcimio_find_device(comedi_device *dev,int bus,int slot)
 	for(mite=mite_devices;mite;mite=mite->next){
 		if(mite->used)continue;
 		if(bus || slot){
-#ifdef PCI_SUPPORT_VER1
-			if(bus!=mite->pci_bus ||
-			   slot!=PCI_SLOT(mite->pci_device_fn))
-				continue;
-#else
 			if(bus!=mite->pcidev->bus->number ||
 			   slot!=PCI_SLOT(mite->pcidev->devfn))
 				continue;
-#endif
 		}
 
 		for(i=0;i<n_pcimio_boards;i++){
@@ -660,3 +803,24 @@ static int pcimio_find_device(comedi_device *dev,int bus,int slot)
 	return -EIO;
 }
 
+static int pcimio_ai_change(comedi_device *dev, comedi_subdevice *s,
+	unsigned long new_size)
+{
+	int ret;
+
+	ret = mite_buf_change(devpriv->mite, AI_DMA_CHAN, s->async, new_size);
+	if(ret<0)return ret;
+
+	return 0;
+}
+
+static int pcimio_ao_change(comedi_device *dev, comedi_subdevice *s,
+	unsigned long new_size)
+{
+	int ret;
+
+	ret = mite_buf_change(devpriv->mite, AO_DMA_CHAN, s->async, new_size);
+	if(ret<0)return ret;
+
+	return 0;
+}
