@@ -3,7 +3,7 @@
     Register descriptions for NI DAQ-STC chip
 
     COMEDI - Linux Control and Measurement Device Interface
-    Copyright (C) 1998-9 David A. Schleef <ds@stm.lbl.gov>
+    Copyright (C) 1998-9 David A. Schleef <ds@schleef.org>
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -82,6 +82,8 @@
 #define AI_Status_2_Register		5
 
 #define Interrupt_B_Ack_Register	3
+#define AO_BC_TC_Interrupt_Ack _bit8
+
 #define AO_Status_1_Register		3
 #define Interrupt_B_St				_bit15
 #define AO_FIFO_Full_St				_bit14
@@ -132,7 +134,9 @@
 #define AO_UPDATE2_Pulse			_bit1
 #define AO_START1_Pulse				_bit0
 
-#define DIO_Input_Register		7
+#define AO_Status_2_Register		6
+
+#define DIO_Parallel_Input_Register	7
 #define DIO_SDIN        			_bit4
 
 #define AI_Command_1_Register		8
@@ -233,7 +237,14 @@
 #define AO_Trigger_Once				_bit0
 
 #define AO_Mode_2_Register		39
-#define AO_FIFO_Mode(x)				((x)<<14)
+#define AO_FIFO_Mode_Mask ( 0x3 << 14 )
+enum AO_FIFO_Mode_Bits
+{
+	AO_FIFO_Mode_HF_to_F = (3<<14),
+	AO_FIFO_Mode_F = (2<<14),
+	AO_FIFO_Mode_HF = (1<<14),
+	AO_FIFO_Mode_E = (0<<14),
+};
 #define AO_FIFO_Retransmit_Enable		_bit13
 #define AO_START1_Disable			_bit12
 #define AO_UC_Initial_Load_Source		_bit11
@@ -261,13 +272,39 @@
 #define AO_UC_Load_A_Register_Low	49
 
 #define Clock_and_FOUT_Register		56
-#define DIO_Serial_Out_Divide_By_2              _bit13
-#define Slow_Internal_Timebase                  _bit11
+#define FOUT_Enable				_bit15
+#define FOUT_Timebase_Select			_bit14
+#define DIO_Serial_Out_Divide_By_2		_bit13
+#define Slow_Internal_Time_Divide_By_2		_bit12
+#define Slow_Internal_Timebase			_bit11
+#define G_Source_Divide_By_2			_bit10
+#define Clock_To_Board_Divide_By_2		_bit9
+#define Clock_To_Board				_bit8
+#define AI_Output_Divide_By_2			_bit7
+#define AI_Source_Divide_By_2			_bit6
+#define AO_Output_Divide_By_2			_bit5
+#define AO_Source_Divide_By_2			_bit4
+#define FOUT_Divider(x)				((x)<<0)
 
 #define IO_Bidirection_Pin_Register	57
 
 #define Interrupt_Control_Register	59
+#define Interrupt_B_Enable			_bit15
+#define Interrupt_B_Output_Select(x)		((x)<<12)
+#define Interrupt_A_Enable			_bit11
+#define Interrupt_A_Output_Select(x)		((x)<<8)
+#define Pass_Thru_0_Interrupt_Polarity		_bit3
+#define Pass_Thru_1_Interrupt_Polarity		_bit2
+#define Interrupt_Output_On_3_Pins		_bit1
+#define Interrupt_Output_Polarity		_bit0
+
 #define AI_Output_Control_Register	60
+#define AI_START_Output_Select			_bit10
+#define AI_SCAN_IN_PROG_Output_Select(x)	((x)<<8)
+#define AI_EXTMUX_CLK_Output_Select(x)		((x)<<6)
+#define AI_LOCALMUX_CLK_Output_Select(x)	((x)<<4)
+#define AI_SC_TC_Output_Select(x)		((x)<<2)
+#define AI_CONVERT_Output_Select(x)		((x)<<0)
 
 #define AI_START_STOP_Select_Register	62
 #define AI_START_Polarity			_bit15
@@ -324,6 +361,8 @@
 #define AI_Configuration_End			_bit8
 #define AO_Configuration_Start			_bit5
 #define AI_Configuration_Start			_bit4
+#define G1_Reset				_bit3
+#define G0_Reset				_bit2
 #define AO_Reset				_bit1
 #define AI_Reset				_bit0
 
@@ -355,7 +394,28 @@
 
 #define Second_IRQ_B_Enable_Register	76
 #define AI_Personal_Register		77
+#define AI_SHIFTIN_Pulse_Width			_bit15
+#define AI_EOC_Polarity				_bit14
+#define AI_SOC_Polarity				_bit13
+#define AI_SHIFTIN_Polarity			_bit12
+#define AI_CONVERT_Pulse_Timebase		_bit11
+#define AI_CONVERT_Pulse_Width			_bit10
+#define AI_CONVERT_Original_Pulse		_bit9
+#define AI_FIFO_Flags_Polarity			_bit8
+#define AI_Overrun_Mode				_bit7
+#define AI_EXTMUX_CLK_Pulse_Width		_bit6
+#define AI_LOCALMUX_CLK_Pulse_Width		_bit5
+#define AI_AIFREQ_Polarity			_bit4
+
 #define AO_Personal_Register		78
+enum AO_Personal_Bits
+{
+	AO_BC_Source_Select = 1 << 4,
+	AO_UPDATE_Pulse_Width = 1 << 5,
+	AO_AOFREQ_Polarity = 1 << 9,
+	AO_FIFO_Enable = 1 << 10,
+	AO_TMRDACWR_Pulse_Width = 1 << 12,
+};
 #define Write_Strobe_0_Register		82
 #define Write_Strobe_1_Register		83
 #define Write_Strobe_2_Register		84
@@ -386,25 +446,8 @@
 #define AI_External_Gate_Polarity		_bit5
 #define AI_External_Gate_Select(a)		(a)
 
-
-/* 16 bit registers shadowed from DAQ-STC */
-#define Window_Address			0x00
-#define Window_Data			0x02
-#define Interrupt_A_Ack			0x04
-#define AI_Status_1			0x04
-#define Interrupt_B_Ack			0x06
-#define AO_Status_1			0x06
-#define AI_Command_2			0x08
-#define G_Status			0x08
-#define AO_Command_2			0x0a
-#define AI_Status_2			0x0a
-#define G0_Command			0x0c
-#define AO_Status_2			0x0c
-#define G1_Command			0x0e
-#define DIO_Parallel_Input		0x0e
-
 #define G_Autoincrement_Register(a)	(68+(a))
-#define G_Command_Register(a)		(6+(a))	
+#define G_Command_Register(a)		(6+(a))
 #define G_HW_Save_Register(a)		(8+(a)*2)
 #define G_HW_Save_Register_High(a)	(8+(a)*2)
 #define G_HW_Save_Register_Low(a)	(9+(a)*2)
@@ -490,8 +533,6 @@
 #define G0_Save_St			_bit0
 
 /* general purpose counter timer */
-#define G0_Reset                        _bit2
-#define G1_Reset                        _bit3
 #define G0_TC_Interrupt_Enable          _bit6
 #define G1_TC_Interrupt_Enable          _bit9
 #define G0_Gate_Interrupt_Enable        _bit8
@@ -520,6 +561,24 @@
 
 /* Additional windowed registers unique to E series */
 
+/* 16 bit registers shadowed from DAQ-STC */
+#define Window_Address			0x00
+#define Window_Data			0x02
+#if 0
+#define Interrupt_A_Ack			0x04
+#define AI_Status_1			0x04
+#define Interrupt_B_Ack			0x06
+#define AO_Status_1			0x06
+#define AI_Command_2			0x08
+#define G_Status			0x08
+#define AO_Command_2			0x0a
+#define AI_Status_2			0x0a
+#define G0_Command			0x0c
+#define AO_Status_2			0x0c
+#define G1_Command			0x0e
+#define DIO_Parallel_Input		0x0e
+#endif
+
 #define Configuration_Memory_Clear	82
 #define ADC_FIFO_Clear			83
 #define DAC_FIFO_Clear			84
@@ -529,6 +588,11 @@
 
 /* 8 bit registers */
 #define XXX_Status			0x01
+enum XXX_Status_Bits
+{
+	PROMOUT = 0x1,
+	AI_FIFO_LOWER_NOT_EMPTY = 0x8,
+};
 #define Serial_Command			0x0d
 #define Misc_Command			0x0f
 #define Port_A				0x19
@@ -545,7 +609,24 @@
 /* 16 bit registers */
 
 #define Configuration_Memory_Low	0x10
+enum Configuration_Memory_Low_Bits
+{
+	AI_DITHER	= 0x200,
+	AI_LAST_CHANNEL	= 0x8000,
+};
 #define Configuration_Memory_High	0x12
+enum Configuration_Memory_High_Bits
+{
+	AI_AC_COUPLE	= 0x800,
+	AI_DIFFERENTIAL = 0x1000,
+	AI_COMMON = 0x2000,
+	AI_GROUND = 0x3000,
+};
+static inline unsigned int AI_CONFIG_CHANNEL( unsigned int channel )
+{
+	return ( channel & 0x3f );
+}
+
 #define ADC_FIFO_Data_Register		0x1c
 
 #define AO_Configuration		0x16
@@ -560,6 +641,38 @@
 #define DAC1_Direct_Data		0x1a
 
 
+/* 611x registers (these boards differ from the e-series) */
+
+#define Magic_611x			0x19 /* w8 (new) */
+#define Calibration_Channel_Select_611x	0x1a /* w16 (new) */
+#define ADC_FIFO_Data_611x		0x1c /* r32 (incompatible) */
+#define AI_FIFO_Offset_Load_611x	0x05 /* r8 (new) */
+#define AO_FIFO_Offset_Load_611x       0x13 /* W32? */
+#define DAC_FIFO_Data_611x		0x14 /* w32 (incompatible) */
+#define AO_Window_Addr_611x		0x18 /* w16 */
+#define AO_Window_Data_611x		0x1e /* w16 */
+#define Cal_Gain_Select_611x		0x05 /* w8 (new) */
+
+/* AO Windowed registers */
+
+/* 671x, 611x registers */
+
+#define AO_Window_Address_671x		0x18 /* W 16 */
+#define AO_Window_Data_671x		0x1e /* W 16 */
+
+/* 671xi, 611x windowed ao registers */
+
+#define DACx_Direct_Data_671x(x)	(x) /* W 16 */
+#define AO_Immediate_671x		0x11 /* W 16 */
+#define AO_Timed_611x                   0x10 /* W 16 */
+#define AO_Later_Single_Point_Updates   0x14 /* W 16 */
+#define AO_Waveform_Generation_611x     0x15 /* W 16 */
+#define AO_Misc_611x                    0x16 /* W 16 */
+enum AO_Misc_611x_Bits
+{
+	CLEAR_WG = 1,
+};
+
 #define SerDacLd(x)			(0x08<<(x))
 
 /*
@@ -567,37 +680,44 @@
 	but I thought I'd put it here anyway.
 */
 
+/* our default usage of mite channels */
+enum mite_dma_channel{
+	AI_DMA_CHAN = 0,
+	AO_DMA_CHAN = 1,
+	GPC0_DMA_CHAN = 2,
+	GPC1_DMA_CHAN = 3,
+};
+
 enum{ ai_gain_16=0, ai_gain_8, ai_gain_14, ai_gain_4, ai_gain_611x };
-extern struct caldac_struct caldac_mb88341,
-	caldac_dac8800,
-	caldac_dac8043,
-	caldac_ad8522;
+enum caldac_enum { caldac_none=0, mb88341, dac8800, dac8043, ad8522,
+	ad8804, ad8842, ad8804_debug };
 
 typedef struct ni_board_struct{
 	int device_id;
+	int isapnp_id;
 	char *name;
-	
+
 	int n_adchan;
 	int adbits;
-	
+
 	int ai_fifo_depth;
-	int alwaysdither : 1;
+	unsigned int alwaysdither : 1;
 	int gainlkup;
 	int ai_speed;
 
 	int n_aochan;
 	int aobits;
-	
+
 	int ao_fifo_depth;
 	int aorangelkup;
-	
-	int ao_unipolar : 1;
-	
-	int has_8255 : 1;
 
-	int has_analog_trig : 1;
+	unsigned int ao_unipolar : 1;
+	unsigned int has_8255 : 1;
+	unsigned int has_analog_trig : 1;
+	unsigned int ao_671x : 1;
+	unsigned int reg_611x : 1;
 
-	struct caldac_struct **caldac;
+	enum caldac_enum caldac[3];
 }ni_board;
 
 static ni_board ni_boards[];
@@ -617,14 +737,18 @@ static ni_board ni_boards[];
 	int ai_continuous;					\
 	int blocksize;						\
 	int n_left;						\
+	unsigned int ai_calib_source;				\
 								\
 	int changain_state;					\
 	unsigned int changain_spec;				\
 								\
 	unsigned int caldac_maxdata_list[MAX_N_CALDACS];	\
 	unsigned short ao[2];					\
-	unsigned short caldacs[12];				\
+	unsigned short caldacs[MAX_N_CALDACS];				\
 								\
+	volatile unsigned short ai_cmd2;	\
+								\
+	unsigned short ao_conf[2];				\
 	unsigned short ao_mode1;				\
 	unsigned short ao_mode2;				\
 	unsigned short ao_mode3;				\
@@ -639,19 +763,21 @@ static ni_board ni_boards[];
 	int gpct_cur_operation[2];				\
 	unsigned short an_trig_etc_reg;				\
 								\
-	unsigned short ai_xorlist[512];				\
+	unsigned short ai_offset[512];				\
 								\
 	unsigned long serial_interval_ns;                       \
 	unsigned char serial_hw_mode;                           \
 	unsigned short clock_and_fout;				\
 								\
-	unsigned short int_a_enable_reg;			\
-	unsigned short int_b_enable_reg;			\
+	volatile unsigned short int_a_enable_reg;			\
+	volatile unsigned short int_b_enable_reg;			\
 	unsigned short io_bidirection_pin_reg;			\
 								\
 	unsigned short atrig_mode;				\
 	unsigned short atrig_high;				\
 	unsigned short atrig_low;				\
+								\
+	sampl_t ai_fifo_buffer[0x2000];				\
 
 #endif /* _COMEDI_NI_STC_H */
 
