@@ -1,7 +1,7 @@
 /*
    module/pcl818.c
 
-   Author:  Michal Dobes <majkl@tesnet.cz>  
+   Author:  Michal Dobes <dobes@tesnet.cz>  
 
    hardware driver for Advantech cards:
     card:   PCL-818L, PCL-818H, PCL-818HD, PCL-818HG, PCL-818, PCL-718
@@ -10,7 +10,7 @@
 /*
 Driver: pcl818.o
 Description: Advantech PCL-818 cards, PCL-718
-Author: Michal Dobes <majkl@tesnet.cz>
+Author: Michal Dobes <dobes@tesnet.cz>
 Devices: [Advantech] PCL-818L (pcl818l), PCL-818H (pcl818h),
   PCL-818HD (pcl818hd), PCL-818HG (pcl818hg), PCL-818 (pcl818),
   PCL-718 (pcl718)
@@ -39,9 +39,7 @@ A word or two about DMA. Driver support DMA operations at two ways:
    If you've data dropouts with DMA mode 2 then:
     a) disable IDE DMA
     b) switch text mode console to fb.
-See the head of the source file pcl818.c for configuration options.
-*/
-/*
+
    Options for PCL-818L:
     [0] - IO Base
     [1] - IRQ	(0=disable, 2, 3, 4, 5, 6, 7)
@@ -100,12 +98,13 @@ See the head of the source file pcl818.c for configuration options.
 
 */
 
+#include <linux/comedidev.h>
+
 #include <linux/ioport.h>
-#include <linux/module.h>
 #include <linux/mc146818rtc.h>
 #include <linux/delay.h>
 #include <asm/dma.h>
-#include <linux/comedidev.h>
+
 #include "8253.h"
 
 // #define PCL818_MODE13_AO 1
@@ -329,7 +328,6 @@ typedef struct {
 	unsigned int 	act_chanlist[16];// MUX setting for actual AI operations
 	unsigned int 	act_chanlist_len;// how long is actual MUX list
 	unsigned int 	act_chanlist_pos;// actual position in MUX list
-	unsigned int 	buf_ptr;	// data buffer ptr in samples
 	comedi_subdevice *sub_ai;	// ptr to AI subdevice
 	unsigned char 	usefifo;	// 1=use fifo
 	struct timer_list rtc_irq_timer;// timer for RTC sanity check
@@ -370,22 +368,22 @@ static int pcl818_ai_insn_read(comedi_device *dev, comedi_subdevice *s,
 	int n;
         int timeout; 
 
+	/* software trigger, DMA and INT off */
+       	outb(0, dev->iobase+PCL818_CONTROL);
+
+	/* select channel */
+       	outb(muxonechan[CR_CHAN(insn->chanspec)],
+		dev->iobase+PCL818_MUX);
+
+	/* select gain */
+       	outb(CR_RANGE(insn->chanspec),
+		dev->iobase+PCL818_RANGE);
+
 	for(n=0;n<insn->n;n++){
-		/* software trigger, DMA and INT off */
-        	outb(0, dev->iobase+PCL818_CONTROL);
 
 		/* clear INT (conversion end) flag */
         	outb(0, dev->iobase+PCL818_CLRINT);
 
-		/* select channel */
-        	outb(muxonechan[CR_CHAN(insn->chanspec)],
-			dev->iobase+PCL818_MUX);
-
-		/* select gain */
-        	outb(CR_RANGE(insn->chanspec),
-			dev->iobase+PCL818_RANGE);
-
-        	udelay(5);
 		/* start conversion */
 		outb(0, dev->iobase+PCL818_AD_LO);
 
@@ -393,6 +391,7 @@ static int pcl818_ai_insn_read(comedi_device *dev, comedi_subdevice *s,
         	while (timeout--) {
 			if (inb(dev->iobase + PCL818_STATUS) & 0x10)
 				goto conv_finish;
+			comedi_udelay(1);
         	}
         	comedi_error(dev,"A/D insn timeout");
 		/* clear INT (conversion end) flag */
@@ -484,82 +483,69 @@ static int pcl818_do_insn_bits(comedi_device *dev, comedi_subdevice *s,
 /*
 ==============================================================================
    analog input interrupt mode 1 & 3, 818 cards
-   one sample per interrupt version   
+   one sample per interrupt version
 */
-static void interrupt_pcl818_ai_mode13_int(int irq, void *d, struct pt_regs *regs) 
+static irqreturn_t interrupt_pcl818_ai_mode13_int(int irq, void *d, struct pt_regs *regs)
 {
-        comedi_device *dev = d;
+	comedi_device *dev = d;
 	comedi_subdevice *s = dev->subdevices + 0;
-        int low;
-        int timeout=50; /* wait max 50us */
+	int low;
+	int timeout=50; /* wait max 50us */
 
 	while (timeout--) {
 		if (inb(dev->iobase + PCL818_STATUS) & 0x10) goto conv_finish;
-		udelay(1);
-        }
-        outb(0,dev->iobase+PCL818_STATUS); /* clear INT request */
-        comedi_error(dev,"A/D mode1/3 IRQ without DRDY!");
+		comedi_udelay(1);
+	}
+	outb(0,dev->iobase+PCL818_STATUS); /* clear INT request */
+	comedi_error(dev,"A/D mode1/3 IRQ without DRDY!");
 	pcl818_ai_cancel(dev,s);
 	s->async->events |= COMEDI_CB_EOA | COMEDI_CB_ERROR;
 	comedi_event(dev, s, s->async->events);
-	return;
+	return IRQ_HANDLED;
 
 conv_finish:
         low=inb(dev->iobase + PCL818_AD_LO);
-	*(sampl_t *)(s->async->data+devpriv->buf_ptr)=((inb(dev->iobase + PCL818_AD_HI) << 4) | (low >> 4)); // get one sample
-	devpriv->buf_ptr+=sizeof(sampl_t);
+	comedi_buf_put( s->async,
+		((inb(dev->iobase + PCL818_AD_HI) << 4) | (low >> 4)) ); // get one sample
         outb(0,dev->iobase+PCL818_CLRINT); /* clear INT request */
 
-        if ((low & 0xf)!=devpriv->act_chanlist[devpriv->act_chanlist_pos]) { // dropout!
+	if ((low & 0xf)!=devpriv->act_chanlist[devpriv->act_chanlist_pos]) { // dropout!
 		rt_printk("comedi: A/D mode1/3 IRQ - channel dropout %x!=%x !\n",(low & 0xf),devpriv->act_chanlist[devpriv->act_chanlist_pos]);
 		pcl818_ai_cancel(dev,s);
 		s->async->events |= COMEDI_CB_EOA | COMEDI_CB_ERROR;
 		comedi_event(dev, s, s->async->events);
-		return;
-        }
-        s->async->buf_int_ptr+=sizeof(sampl_t);
-        s->async->buf_int_count+=sizeof(sampl_t);
-
-	s->async->cur_chan++;
-        if (s->async->cur_chan>=s->async->cmd.chanlist_len){
-		s->async->cur_chan=0;
-		s->async->events |= COMEDI_CB_BLOCK;
+		return IRQ_HANDLED;
+	}
+	if (s->async->cur_chan == 0) {
 		// rt_printk("E");
 		devpriv->int13_act_scan--;
-        }
-
-	if (s->async->buf_int_ptr>=s->async->data_len) { /* buffer rollover */
-		s->async->buf_int_ptr=0;
-		devpriv->buf_ptr=0;
-		//printk("B ");
-		s->async->events |= COMEDI_CB_EOBUF;
-        }
+	}
 
 	if (!devpriv->neverending_ai){
 		if ( devpriv->int13_act_scan == 0 ) { /* all data sampled */
 			pcl818_ai_cancel(dev,s);
 			s->async->events |= COMEDI_CB_EOA;
-			return;
 		}
 	}
 	comedi_event(dev, s, s->async->events);
+	return IRQ_HANDLED;
 }
 
 /*
 ==============================================================================
    analog input dma mode 1 & 3, 818 cards
 */
-static void interrupt_pcl818_ai_mode13_dma(int irq, void *d, struct pt_regs *regs)
+static irqreturn_t interrupt_pcl818_ai_mode13_dma(int irq, void *d, struct pt_regs *regs)
 {
 	comedi_device *dev = d;
 	comedi_subdevice *s = dev->subdevices + 0;
 	int i,len,bufptr;
 	unsigned long flags;
-        sampl_t *ptr;
+	sampl_t *ptr;
 
-        disable_dma(devpriv->dma);
-        devpriv->next_dma_buf=1-devpriv->next_dma_buf;
-        if ((devpriv->dma_runs_to_end)>-1) {  // switch dma bufs
+	disable_dma(devpriv->dma);
+	devpriv->next_dma_buf=1-devpriv->next_dma_buf;
+	if ((devpriv->dma_runs_to_end)>-1) {  // switch dma bufs
 		set_dma_mode(devpriv->dma, DMA_MODE_READ);
 		flags=claim_dma_lock();
 		set_dma_addr(devpriv->dma, devpriv->hwdmaptr[devpriv->next_dma_buf]);
@@ -567,71 +553,57 @@ static void interrupt_pcl818_ai_mode13_dma(int irq, void *d, struct pt_regs *reg
 					else { set_dma_count(devpriv->dma, devpriv->last_dma_run); }
 		release_dma_lock(flags);
 		enable_dma(devpriv->dma);
-        }
+	}
 
-        devpriv->dma_runs_to_end--;
-        outb(0,dev->iobase+PCL818_CLRINT); /* clear INT request */
-        ptr=(sampl_t *)devpriv->dmabuf[1-devpriv->next_dma_buf];
+	devpriv->dma_runs_to_end--;
+	outb(0,dev->iobase+PCL818_CLRINT); /* clear INT request */
+	ptr=(sampl_t *)devpriv->dmabuf[1-devpriv->next_dma_buf];
 
-        len=devpriv->hwdmasize[0] >> 1;
-        bufptr=0;
+	len=devpriv->hwdmasize[0] >> 1;
+	bufptr=0;
 
-        for (i=0;i<len;i++) {
+	for (i=0;i<len;i++) {
 		if ((ptr[bufptr] & 0xf)!=devpriv->act_chanlist[devpriv->act_chanlist_pos]) { // dropout!
-		        rt_printk("comedi: A/D mode1/3 DMA - channel dropout %d!=%d !\n",(ptr[bufptr] & 0xf),devpriv->act_chanlist[devpriv->act_chanlist_pos]);
-		        pcl818_ai_cancel(dev,s);
+			rt_printk("comedi: A/D mode1/3 DMA - channel dropout %d!=%d !\n",(ptr[bufptr] & 0xf),devpriv->act_chanlist[devpriv->act_chanlist_pos]);
+			pcl818_ai_cancel(dev,s);
 			s->async->events |= COMEDI_CB_EOA | COMEDI_CB_ERROR;
 			comedi_event(dev, s, s->async->events);
-			return;
+			return IRQ_HANDLED;
 		}
 
-		*(sampl_t *)(s->async->data+s->async->buf_int_ptr)=
-			ptr[bufptr++] >> 4; // get one sample
-		devpriv->buf_ptr++;
+		comedi_buf_put( s->async, ptr[bufptr++] >> 4 ); // get one sample
 
-		s->async->buf_int_ptr+=sizeof(sampl_t);
-		s->async->buf_int_count+=sizeof(sampl_t);
 		devpriv->act_chanlist_pos++;
 
-		s->async->cur_chan++;
-		if(s->async->cur_chan>=s->async->cmd.chanlist_len){
-			s->async->cur_chan=0;
-		}
-
-		if (s->async->buf_int_ptr>=s->async->data_len) { /* buffer rollover */
-	    		s->async->buf_int_ptr=0;
-			devpriv->buf_ptr=0;
-			s->async->events |= COMEDI_CB_EOBUF;
-		}
-
 		if (!devpriv->neverending_ai)
 	    		if ( devpriv->int13_act_scan == 0 ) { /* all data sampled */
 				pcl818_ai_cancel(dev,s);
 				s->async->events |= COMEDI_CB_EOA;
 				comedi_event(dev, s, s->async->events);
 				// printk("done int ai13 dma\n");
-				return;
+				return IRQ_HANDLED;
 			}
 	}
 
 	if (len>0) comedi_event(dev, s, s->async->events);
+	return IRQ_HANDLED;
 }
 
 /*
 ==============================================================================
    analog input dma mode 1 & 3 over RTC, 818 cards
 */
-static void interrupt_pcl818_ai_mode13_dma_rtc(int irq, void *d, struct pt_regs *regs)
+static irqreturn_t interrupt_pcl818_ai_mode13_dma_rtc(int irq, void *d, struct pt_regs *regs)
 {
-        comedi_device *dev = d;
+	comedi_device *dev = d;
 	comedi_subdevice *s = dev->subdevices + 0;
-        unsigned long tmp;
-        unsigned int top1,top2,i,bufptr;
-        long ofs_dats;
-        sampl_t *dmabuf=(sampl_t *)devpriv->dmabuf[0];
+	unsigned long tmp;
+	unsigned int top1,top2,i,bufptr;
+	long ofs_dats;
+	sampl_t *dmabuf=(sampl_t *)devpriv->dmabuf[0];
 
-        //outb(2,0x378);
-        switch(devpriv->int818_mode) {
+	//outb(2,0x378);
+	switch(devpriv->int818_mode) {
 	case INT_TYPE_AI1_DMA_RTC:
 	case INT_TYPE_AI3_DMA_RTC:
 		tmp = (CMOS_READ(RTC_INTR_FLAGS) & 0xF0);
@@ -643,12 +615,12 @@ static void interrupt_pcl818_ai_mode13_dma_rtc(int irq, void *d, struct pt_regs
 			if (top1==top2) break;
 		}
 
-		if (top1!=top2) return;
+		if (top1!=top2) return IRQ_HANDLED;
 		top1=devpriv->hwdmasize[0]-top1; // where is now DMA in buffer
 		top1>>=1;
 		ofs_dats=top1-devpriv->last_top_dma;  // new samples from last call
 		if (ofs_dats<0) ofs_dats=(devpriv->dmasamplsize)+ofs_dats;
-		if (!ofs_dats) return; // exit=no new samples from last call
+		if (!ofs_dats) return IRQ_HANDLED; // exit=no new samples from last call
 		// obsluz data
 		i=devpriv->last_top_dma-1;
 		i&=(devpriv->dmasamplsize-1);
@@ -659,7 +631,7 @@ static void interrupt_pcl818_ai_mode13_dma_rtc(int irq, void *d, struct pt_regs
 			pcl818_ai_cancel(dev,s);
 			s->async->events |= COMEDI_CB_EOA | COMEDI_CB_ERROR;
 			comedi_event(dev, s, s->async->events);
-			return;
+			return IRQ_HANDLED;
 		}
 		//rt_printk("r %ld ",ofs_dats);
 
@@ -671,36 +643,23 @@ static void interrupt_pcl818_ai_mode13_dma_rtc(int irq, void *d, struct pt_regs
 				pcl818_ai_cancel(dev,s);
 				s->async->events |= COMEDI_CB_EOA | COMEDI_CB_ERROR;
 				comedi_event(dev, s, s->async->events);
-				return;
+				return IRQ_HANDLED;
 			}
 
-			*(sampl_t *)(s->async->data+s->async->buf_int_ptr)=
-				dmabuf[bufptr++] >> 4; // get one sample
-			devpriv->buf_ptr++;
+			comedi_buf_put( s->async, dmabuf[bufptr++] >> 4); // get one sample
 			bufptr&=(devpriv->dmasamplsize-1);
 
-			s->async->buf_int_ptr+=sizeof(sampl_t);
-			s->async->buf_int_count+=sizeof(sampl_t);
-
-			s->async->cur_chan++;
-			if(s->async->cur_chan>=s->async->cmd.chanlist_len){
-				s->async->cur_chan++;
+			if(s->async->cur_chan == 0){
 				devpriv->int13_act_scan--;
 			}
 
-			if (s->async->buf_int_ptr>=s->async->data_len) { /* buffer rollover */
-				s->async->buf_int_ptr=0;
-				devpriv->buf_ptr=0;
-				s->async->events |= COMEDI_CB_EOBUF;
-			}
-
 			if (!devpriv->neverending_ai)
 				if ( devpriv->int13_act_scan == 0 ) { /* all data sampled */
 					pcl818_ai_cancel(dev,s);
 					s->async->events |= COMEDI_CB_EOA;
 					comedi_event(dev, s, s->async->events);
 					//printk("done int ai13 dma\n");
-					return;
+					return IRQ_HANDLED;
 				}
 		}
 
@@ -710,135 +669,124 @@ static void interrupt_pcl818_ai_mode13_dma_rtc(int irq, void *d, struct pt_regs
 		dmabuf[bufptr]=MAGIC_DMA_WORD;
 		comedi_event(dev, s, s->async->events);
 		//outb(0,0x378);
-		return;
+		return IRQ_HANDLED;
 	}
 
 	//outb(0,0x378);
+	return IRQ_HANDLED;
 }
 
 /*
 ==============================================================================
    analog input interrupt mode 1 & 3, 818HD/HG cards
 */
-static void interrupt_pcl818_ai_mode13_fifo(int irq, void *d, struct pt_regs *regs)
+static irqreturn_t interrupt_pcl818_ai_mode13_fifo(int irq, void *d, struct pt_regs *regs)
 {
-        comedi_device *dev = d;
-        comedi_subdevice *s = dev->subdevices + 0;
-        int i,len,lo;
+	comedi_device *dev = d;
+	comedi_subdevice *s = dev->subdevices + 0;
+	int i,len,lo;
 
-        outb(0, dev->iobase + PCL818_FI_INTCLR);  // clear fifo int request
+	outb(0, dev->iobase + PCL818_FI_INTCLR);  // clear fifo int request
 
-        lo=inb(dev->iobase + PCL818_FI_STATUS);
+	lo=inb(dev->iobase + PCL818_FI_STATUS);
 
-        if (lo&4) {
+	if (lo&4) {
 		comedi_error(dev,"A/D mode1/3 FIFO overflow!");
 		pcl818_ai_cancel(dev,s);
 		s->async->events |= COMEDI_CB_EOA | COMEDI_CB_ERROR;
 		comedi_event(dev, s, s->async->events);
-		return;
-        }
+		return IRQ_HANDLED;
+	}
 
-        if (lo&1) {
+	if (lo&1) {
 		comedi_error(dev,"A/D mode1/3 FIFO interrupt without data!");
 		pcl818_ai_cancel(dev,s);
 		s->async->events |= COMEDI_CB_EOA | COMEDI_CB_ERROR;
 		comedi_event(dev, s, s->async->events);
-		return;
+		return IRQ_HANDLED;
 	}
 
-        if (lo&2) { len=512; }
-	    else { len=0; }
+	if (lo&2) { len=512; }
+	else { len=0; }
 
-        for (i=0;i<len;i++) {
+	for (i=0;i<len;i++) {
 		lo=inb(dev->iobase + PCL818_FI_DATALO);
 		if ((lo & 0xf)!=devpriv->act_chanlist[devpriv->act_chanlist_pos]) { // dropout!
 			rt_printk("comedi: A/D mode1/3 FIFO - channel dropout %d!=%d !\n",(lo & 0xf),devpriv->act_chanlist[devpriv->act_chanlist_pos]);
 			pcl818_ai_cancel(dev,s);
 			s->async->events |= COMEDI_CB_EOA | COMEDI_CB_ERROR;
 			comedi_event(dev, s, s->async->events);
-			return;
+			return IRQ_HANDLED;
 		}
 
-		*(sampl_t *)(s->async->data+s->async->buf_int_ptr)=
-			(lo >> 4)|(inb(dev->iobase + PCL818_FI_DATAHI) << 4); // get one sample
-		devpriv->buf_ptr++;
-		s->async->buf_int_ptr+=sizeof(sampl_t);
-		s->async->buf_int_count+=sizeof(sampl_t);
+		comedi_buf_put( s->async, (lo >> 4)|(inb(dev->iobase + PCL818_FI_DATAHI) << 4) ); // get one sample
 
-		s->async->cur_chan++;
-		if(s->async->cur_chan>=s->async->cmd.chanlist_len){
-			s->async->cur_chan = 0;
+		if(s->async->cur_chan == 0){
 			devpriv->int13_act_scan--;
 		}
 
-		if (s->async->buf_int_ptr>=s->async->data_len) { /* buffer rollover */
-			s->async->buf_int_ptr=0;
-			devpriv->buf_ptr=0;
-			s->async->events |= COMEDI_CB_EOBUF;
-		}
-
 		if (!devpriv->neverending_ai)
 			if ( devpriv->int13_act_scan == 0 ) { /* all data sampled */
 				pcl818_ai_cancel(dev,s);
 				s->async->events |= COMEDI_CB_EOA;
 				comedi_event(dev, s, s->async->events);
-				return;
+				return IRQ_HANDLED;
 			}
 	}
 
 	if (len>0) comedi_event(dev, s, s->async->events);
+	return IRQ_HANDLED;
 }
 
 /*
 ==============================================================================
     INT procedure
 */
-static void interrupt_pcl818(int irq, void *d, struct pt_regs *regs)
+static irqreturn_t interrupt_pcl818(int irq, void *d, struct pt_regs *regs)
 {
-        comedi_device *dev = d;
+	comedi_device *dev = d;
 
 	if(!dev->attached)
 	{
 		comedi_error(dev, "premature interrupt");
-		return;
+		return IRQ_HANDLED;
 	}
 
-        //rt_printk("I\n");
+	//rt_printk("I\n");
 
-        switch (devpriv->int818_mode) {
+	switch (devpriv->int818_mode) {
 	case INT_TYPE_AI1_DMA:
 	case INT_TYPE_AI3_DMA:
-		interrupt_pcl818_ai_mode13_dma(irq, d, regs);
-		return;
+		return interrupt_pcl818_ai_mode13_dma(irq, d, regs);
 	case INT_TYPE_AI1_INT:
 	case INT_TYPE_AI3_INT:
-		interrupt_pcl818_ai_mode13_int(irq, d, regs);
-		return;
+		return interrupt_pcl818_ai_mode13_int(irq, d, regs);
 	case INT_TYPE_AI1_FIFO:
 	case INT_TYPE_AI3_FIFO:
-		interrupt_pcl818_ai_mode13_fifo(irq, d, regs);
-		return;
+		return interrupt_pcl818_ai_mode13_fifo(irq, d, regs);
 #ifdef PCL818_MODE13_AO
 	case INT_TYPE_AO1_INT:
 	case INT_TYPE_AO3_INT:
-		interrupt_pcl818_ao_mode13_int(irq, d, regs);
-		return;
+		return interrupt_pcl818_ao_mode13_int(irq, d, regs);
 #endif
+	default:
+		break;
 	}
 
 	outb(0,dev->iobase+PCL818_CLRINT); /* clear INT request */
 
-	if ((!dev->irq)|(!devpriv->irq_free)|(!devpriv->irq_blocked)|(!devpriv->int818_mode)) {
+	if ((!dev->irq)||(!devpriv->irq_free)||(!devpriv->irq_blocked)||(!devpriv->int818_mode)) {
 		if (devpriv->irq_was_now_closed) {
 			devpriv->irq_was_now_closed=0;
 			// comedi_error(dev,"last IRQ..");
-			return;
+			return IRQ_HANDLED;
 		}
 		comedi_error(dev,"bad IRQ!");
-		return;
+		return IRQ_NONE;
 	}
 
-        comedi_error(dev,"IRQ from unknow source!");
+	comedi_error(dev,"IRQ from unknow source!");
+	return IRQ_NONE;
 }
 
 /*
@@ -934,7 +882,7 @@ static int pcl818_ai_mode13(int mode, comedi_device * dev, comedi_subdevice * s,
 	start_pacer(dev, -1, 0, 0); // stop pacer
 
         if (!check_and_setup_channel_list(dev, s, it)) return -EINVAL;
-	udelay(1);
+	comedi_udelay(1);
 
         devpriv->int13_act_scan=it->n;
         devpriv->int13_act_chan=0;
@@ -1103,7 +1051,7 @@ static void start_pacer(comedi_device * dev, int mode, unsigned int divisor1, un
         outb(0xb4, dev->iobase + PCL818_CTRCTL);
         outb(0x74, dev->iobase + PCL818_CTRCTL);
 	outb(0x30, dev->iobase + PCL818_CTRCTL);
-        udelay(1);
+        comedi_udelay(1);
   
         if (mode==1) {
 		outb(divisor2 & 0xff, dev->iobase + PCL818_CTR2);
@@ -1176,7 +1124,7 @@ static int check_and_setup_channel_list(comedi_device * dev, comedi_subdevice *
 		outb(CR_RANGE(it->chanlist[i]), dev->iobase+PCL818_RANGE); /* select gain */
 	}
 
-	udelay(1);
+	comedi_udelay(1);
 
 	/* select channel interval to sca n*/
 	outb(devpriv->act_chanlist[0] | (devpriv->act_chanlist[seglen-1] << 4),
@@ -1223,7 +1171,7 @@ static int pcl818_ai_cancel(comedi_device * dev, comedi_subdevice * s)
 		case INT_TYPE_AO3_INT:
 #endif
 			outb(inb(dev->iobase+PCL818_CONTROL)& 0x73, dev->iobase+PCL818_CONTROL); /* Stop A/D */
-			udelay(1);
+			comedi_udelay(1);
 			outb(0, dev->iobase+PCL818_CONTROL); /* Stop A/D */
 			outb(0xb4, dev->iobase + PCL818_CTRCTL);/* Stop pacer */
 			outb(0x74, dev->iobase + PCL818_CTRCTL);
@@ -1256,15 +1204,15 @@ static int pcl818_ai_cancel(comedi_device * dev, comedi_subdevice * s)
 static int pcl818_check(int iobase) 
 {
         outb(0x00, iobase + PCL818_MUX);
-        udelay(1); 
+        comedi_udelay(1); 
         if (inb(iobase + PCL818_MUX)!=0x00) return 1; //there isn't card
         outb(0x55, iobase + PCL818_MUX);
-        udelay(1); 
+        comedi_udelay(1); 
         if (inb(iobase + PCL818_MUX)!=0x55) return 1; //there isn't card
         outb(0x00, iobase + PCL818_MUX);
-        udelay(1); 
+        comedi_udelay(1); 
         outb(0x18, iobase + PCL818_CONTROL); 
-        udelay(1); 
+        comedi_udelay(1); 
         if (inb(iobase + PCL818_CONTROL)!=0x18) return 1; //there isn't card
         return 0; // ok, card exist
 }
@@ -1282,10 +1230,10 @@ static void pcl818_reset(comedi_device * dev)
         }
         outb(0, dev->iobase + PCL818_DA_LO); // DAC=0V
         outb(0, dev->iobase + PCL818_DA_HI);
-        udelay(1);
+        comedi_udelay(1);
         outb(0, dev->iobase + PCL818_DO_HI); // DO=$0000
         outb(0, dev->iobase + PCL818_DO_LO);
-        udelay(1);
+        comedi_udelay(1);
         outb(0, dev->iobase + PCL818_CONTROL);
         outb(0, dev->iobase + PCL818_CNTENABLE);
         outb(0, dev->iobase + PCL818_MUX);
@@ -1544,8 +1492,7 @@ no_rtc:
 
 no_dma:
 
-        dev->n_subdevices = 4;
-        if((ret=alloc_subdevices(dev))<0) return ret;
+        if((ret=alloc_subdevices(dev, 4))<0) return ret;
 
 	s = dev->subdevices + 0;
 	if(!this_board->n_aichan_se){
@@ -1598,7 +1545,7 @@ no_dma:
 	}else{
 		dev->write_subdev = s;
 		s->type = COMEDI_SUBD_AO;
-		s->subdev_flags = SDF_WRITEABLE|SDF_GROUND|SDF_RT;
+		s->subdev_flags = SDF_WRITABLE|SDF_GROUND;
 		s->n_chan = this_board->n_aochan;
 		s->maxdata = this_board->ao_maxdata;
 		s->len_chanlist = this_board->ao_chanlist;
@@ -1631,7 +1578,7 @@ no_dma:
 		s->type = COMEDI_SUBD_UNUSED;
 	}else{
 		s->type = COMEDI_SUBD_DI;
-		s->subdev_flags = SDF_READABLE|SDF_RT;
+		s->subdev_flags = SDF_READABLE;
 		s->n_chan = this_board->n_dichan;
 		s->maxdata = 1;
 		s->len_chanlist = this_board->n_dichan;
@@ -1644,7 +1591,7 @@ no_dma:
 		s->type = COMEDI_SUBD_UNUSED;
 	}else{
 		s->type = COMEDI_SUBD_DO;
-		s->subdev_flags = SDF_WRITEABLE|SDF_RT;
+		s->subdev_flags = SDF_WRITABLE;
 		s->n_chan = this_board->n_dochan;
 		s->maxdata = 1;
 		s->len_chanlist = this_board->n_dochan;
