@@ -1,16 +1,15 @@
 /* plx9080.h
+ *
+ * Copyright (C) 2002,2003 Frank Mori Hess <fmhess@users.sourceforge.net>
  *
  * I modified this file from the plx9060.h header for the
  * wanXL device driver in the linux kernel,
- * for the register offsets and bit definitions.  Minor modifications,
+ * for the register offsets and bit definitions.  Made minor modifications,
  * added plx9080 registers and
- * stripped out stuff that was specifically for the wanXL driver.  I
- * use this for the plx9080 chip used in the cards supported by
- * the cb_pcidas64.c driver.
- * Frank Mori Hess
+ * stripped out stuff that was specifically for the wanXL driver.
+ *
  ********************************************************************
  *
- * SBE wanXL device driver
  * Copyright (C) 1999 RG Studio s.c., http://www.rgstudio.com.pl/
  * Written by Krzysztof Halasa <khc@rgstudio.com.pl>
  *
@@ -28,13 +27,13 @@
 // descriptor block used for chained dma transfers
 struct plx_dma_desc
 {
-	u32 pci_start_addr;
-	u32 local_start_addr;
-	// transfer_size is in bytes, only first 23 bits of register are used
-	u32 transfer_size;
+	volatile uint32_t pci_start_addr;
+	volatile uint32_t local_start_addr;
+	/* transfer_size is in bytes, only first 23 bits of register are used */
+	volatile uint32_t transfer_size;
 	/* address of next descriptor (quad word aligned), plus some
 	 * additional bits (see PLX_DMA0_DESCRIPTOR_REG) */
-	u32 next;
+	volatile uint32_t next;
 };
 
 /**********************************************************************
@@ -46,6 +45,7 @@ struct plx_dma_desc
 **********************************************************************/
 
 #define PLX_LAS0RNG_REG         0x0000 /* L, Local Addr Space 0 Range Register */
+#define PLX_LAS1RNG_REG         0x00f0 /* L, Local Addr Space 1 Range Register */
 #define  LRNG_IO           0x00000001 /* Map to: 1=I/O, 0=Mem */
 #define  LRNG_ANY32        0x00000000 /* Locate anywhere in 32 bit */
 #define  LRNG_LT1MB        0x00000002 /* Locate in 1st meg */
@@ -54,6 +54,7 @@ struct plx_dma_desc
 #define  LRNG_IO_MASK     0xfffffffa	// bits that specify range for normal io
 
 #define PLX_LAS0MAP_REG         0x0004 /* L, Local Addr Space 0 Remap Register */
+#define PLX_LAS1MAP_REG         0x00f4 /* L, Local Addr Space 1 Remap Register */
 #define  LMAP_EN           0x00000001 /* Enable slave decode */
 #define  LMAP_MEM_MASK     0xfffffff0	// bits that specify decode for memory io
 #define  LMAP_IO_MASK     0xfffffffa	// bits that specify decode bits for normal io
@@ -165,12 +166,12 @@ struct plx_dma_desc
 #define  ICS_MBIA(x)       (0x10000000 << ((x) & 0x3)) // mailbox x is active
 
 #define PLX_CONTROL_REG        0x006C /* L, EEPROM Cntl & PCI Cmd Codes */
-#define  CTL_RDMA          0x0000000F /* DMA Read Command */
-#define  CTL_WDMA          0x000000F0 /* DMA Write Command */
-#define  CTL_RMEM          0x00000F00 /* Memory Read Command */
-#define  CTL_WMEM          0x0000F000 /* Memory Write Command */
-#define  CTL_USER0         0x00010000 /* USER0 pin control bit */
-#define  CTL_USER1         0x00020000 /* USER1 pin control bit */
+#define  CTL_RDMA          0x0000000E /* DMA Read Command */
+#define  CTL_WDMA          0x00000070 /* DMA Write Command */
+#define  CTL_RMEM          0x00000600 /* Memory Read Command */
+#define  CTL_WMEM          0x00007000 /* Memory Write Command */
+#define  CTL_USERO         0x00010000 /* USERO output pin control bit */
+#define  CTL_USERI         0x00020000 /* USERI input pin bit */
 #define  CTL_EE_CLK        0x01000000 /* EEPROM Clock line */
 #define  CTL_EE_CS         0x02000000 /* EEPROM Chip Select */
 #define  CTL_EE_W          0x04000000 /* EEPROM Write bit */
@@ -185,19 +186,27 @@ struct plx_dma_desc
 #define PLX_REVISION_REG	0x74	// silicon revision
 
 #define PLX_DMA0_MODE_REG	0x80	// dma channel 0 mode register
+#define PLX_DMA1_MODE_REG	0x94	// dma channel 0 mode register
 #define  PLX_LOCAL_BUS_16_WIDE_BITS	0x1
+#define  PLX_LOCAL_BUS_32_WIDE_BITS	0x3
 #define  PLX_LOCAL_BUS_WIDTH_MASK	0x3
 #define  PLX_DMA_EN_READYIN_BIT	0x40	// enable ready in input
+#define  PLX_EN_BTERM_BIT	0x80	// enable BTERM# input
+#define  PLX_DMA_LOCAL_BURST_EN_BIT	0x100	// enable local burst mode
 #define  PLX_EN_CHAIN_BIT	0x200	// enables chaining
 #define  PLX_EN_DMA_DONE_INTR_BIT	0x400	// enables interrupt on dma done
 #define  PLX_LOCAL_ADDR_CONST_BIT	0x800	// hold local address constant (don't increment)
+#define  PLX_DEMAND_MODE_BIT	0x1000	// enables demand-mode for dma transfer
 #define  PLX_DMA_INTR_PCI_BIT	0x20000	// routes dma interrupt to pci bus (instead of local bus)
 
 #define PLX_DMA0_PCI_ADDRESS_REG	0x84	// pci address that dma transfers start at
+#define PLX_DMA1_PCI_ADDRESS_REG	0x98
 
 #define PLX_DMA0_LOCAL_ADDRESS_REG	0x88	// local address that dma transfers start at
+#define PLX_DMA1_LOCAL_ADDRESS_REG	0x9c
 
 #define PLX_DMA0_TRANSFER_SIZE_REG	0x8c	// number of bytes to transfer (first 23 bits)
+#define PLX_DMA1_TRANSFER_SIZE_REG	0xa0
 
 #define PLX_DMA0_DESCRIPTOR_REG	0x90	// descriptor pointer register
 #define PLX_DMA1_DESCRIPTOR_REG	0xa4
@@ -347,4 +356,55 @@ struct plx_dma_desc
 #define MBX_ADDR_SPACE_360 0x80	/* wanXL100s/200/400 */
 #define MBX_ADDR_MASK_360 (MBX_ADDR_SPACE_360-1)
 
+static inline int plx9080_abort_dma( unsigned long iobase, unsigned int channel )
+{
+	unsigned long dma_cs_addr;
+	uint8_t dma_status;
+	const int timeout = 10000;
+	unsigned int i;
+
+	if( channel )
+		dma_cs_addr = iobase + PLX_DMA1_CS_REG;
+	else
+		dma_cs_addr = iobase + PLX_DMA0_CS_REG;
+
+	// abort dma transfer if necessary
+	dma_status = readb( dma_cs_addr );
+	if( ( dma_status & PLX_DMA_EN_BIT ) == 0 )
+	{
+		return 0;
+	}
+
+	// wait to make sure done bit is zero
+	for( i = 0; ( dma_status & PLX_DMA_DONE_BIT ) && i < timeout; i++ )
+	{
+		dma_status = readb( dma_cs_addr );
+		comedi_udelay( 1 );
+	}
+	if( i == timeout )
+	{
+		rt_printk("plx9080: cancel() timed out waiting for dma %i done clear\n", channel);
+		return -ETIMEDOUT;
+	}
+	// disable channel
+	writeb( 0, dma_cs_addr );
+	comedi_udelay( 1 );
+	// abort channel
+	writeb( PLX_DMA_ABORT_BIT, dma_cs_addr );
+	// wait for dma done bit
+	dma_status = readb( dma_cs_addr );
+	for( i = 0; ( dma_status & PLX_DMA_DONE_BIT ) == 0 && i < timeout; i++ )
+	{
+		comedi_udelay( 1 );
+		dma_status = readb( dma_cs_addr );
+	}
+	if( i == timeout )
+	{
+		rt_printk("plx9080: cancel() timed out waiting for dma %i done set\n", channel);
+		return -ETIMEDOUT;
+	}
+
+	return 0;
+}
+
 #endif /* __COMEDI_PLX9080_H */
