@@ -3,7 +3,7 @@
    hardware driver for Analog Devices RTI-800/815 board
 
    COMEDI - Linux Control and Measurement Device Interface
-   Copyright (C) 1998 David A. Schleef <ds@stm.lbl.gov>
+   Copyright (C) 1998 David A. Schleef <ds@schleef.org>
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -51,20 +51,10 @@ Configuration options:
   [7] - DAC 1 encoding (same as DAC 0)
 */
 
-#include <linux/kernel.h>
-#include <linux/module.h>
-#include <linux/sched.h>
-#include <linux/mm.h>
-#include <linux/slab.h>
-#include <linux/errno.h>
-#include <linux/ioport.h>
-#include <linux/delay.h>
-#include <linux/interrupt.h>
-#include <linux/timex.h>
-#include <linux/timer.h>
-#include <asm/io.h>
 #include <linux/comedidev.h>
 
+#include <linux/ioport.h>
+
 
 #define RTI800_SIZE 16
 
@@ -149,7 +139,7 @@ static comedi_driver driver_rti800={
 };
 COMEDI_INITCLEANUP(driver_rti800);
 
-static void rti800_interrupt(int irq, void *dev, struct pt_regs *regs);
+static irqreturn_t rti800_interrupt(int irq, void *dev, struct pt_regs *regs);
 
 typedef struct {
 	enum {
@@ -173,15 +163,15 @@ typedef struct {
 
 #define devpriv ((rti800_private *)dev->private)
 
-#define RTI800_TIMEOUT 10
+#define RTI800_TIMEOUT 100
 
-static void rti800_interrupt(int irq, void *dev, struct pt_regs *regs)
+static irqreturn_t rti800_interrupt(int irq, void *dev, struct pt_regs *regs)
 {
-
-
+	return IRQ_HANDLED;
 }
 
-static int gaindelay[]={10,20,40,80};
+// settling delay times in usec for different gains
+//static int gaindelay[]={10,20,40,80};
 
 static int rti800_ai_insn_read(comedi_device *dev,comedi_subdevice *s,
 	comedi_insn *insn,lsampl_t *data)
@@ -196,12 +186,16 @@ static int rti800_ai_insn_read(comedi_device *dev,comedi_subdevice *s,
 
 	outb(chan | (gain << 5), dev->iobase + RTI800_MUXGAIN);
 
-	/* contrary to the docs, there needs to be a delay here */
-	udelay(gaindelay[gain]);
+	/* without a delay here, you the RTI_OVERRUN bit
+	 * gets set, and you will have an error.  Not
+	 * sure if this is a long enough delay though.
+	 * comedi_udelay( gaindelay[ gain ] ) definitely
+	 * works */
+	comedi_udelay(1);
 
 	for(i=0;i<insn->n;i++){
 		outb(0, dev->iobase + RTI800_CONVERT);
-    		for (t = RTI800_TIMEOUT; t; t--) {
+		for (t = RTI800_TIMEOUT; t; t--) {
 			status=inb(dev->iobase+RTI800_CSR);
 			if(status & RTI800_OVERRUN){
 				rt_printk("rti800: a/d overrun\n");
@@ -209,7 +203,7 @@ static int rti800_ai_insn_read(comedi_device *dev,comedi_subdevice *s,
 				return -EIO;
 			}
 			if (status & RTI800_DONE)break;
-			//udelay(8);
+			comedi_udelay(1);
 		}
 		if(t == 0){
 			rt_printk("rti800: timeout\n");
@@ -340,8 +334,7 @@ static int rti800_attach(comedi_device * dev, comedi_devconfig * it)
 
 	dev->board_name = this_board->name;
 
-	dev->n_subdevices=4;
-	if((ret=alloc_subdevices(dev))<0)
+	if((ret=alloc_subdevices(dev, 4))<0)
 		return ret;
 	if((ret=alloc_private(dev,sizeof(rti800_private)))<0)
 		return ret;
@@ -357,7 +350,7 @@ static int rti800_attach(comedi_device * dev, comedi_devconfig * it)
 	s=dev->subdevices+0;
 	/* ai subdevice */
 	s->type=COMEDI_SUBD_AI;
-	s->subdev_flags=SDF_READABLE;
+	s->subdev_flags=SDF_READABLE|SDF_GROUND;
 	s->n_chan=(devpriv->adc_mux? 16 : 8);
 	s->insn_read=rti800_ai_insn_read;
 	s->maxdata=0xfff;
@@ -377,7 +370,7 @@ static int rti800_attach(comedi_device * dev, comedi_devconfig * it)
 	if (this_board->has_ao){
 		/* ao subdevice (only on rti815) */
 		s->type=COMEDI_SUBD_AO;
-		s->subdev_flags=SDF_WRITEABLE;
+		s->subdev_flags=SDF_WRITABLE;
 		s->n_chan=2;
 		s->insn_read=rti800_ao_insn_read;
 		s->insn_write=rti800_ao_insn_write;
@@ -415,7 +408,7 @@ static int rti800_attach(comedi_device * dev, comedi_devconfig * it)
 	s++;
 	/* do */
 	s->type=COMEDI_SUBD_DO;
-	s->subdev_flags=SDF_WRITEABLE;
+	s->subdev_flags=SDF_WRITABLE;
 	s->n_chan=8;
 	s->insn_bits=rti800_do_insn_bits;
 	s->maxdata=1;
